@@ -21,7 +21,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_webirc.c 23965 2007-05-18 19:55:53Z jilles $
+ *  $Id: m_webirc.c 26401 2009-01-15 23:15:55Z androsyn $
  */
 /* Usage:
  * auth {
@@ -42,19 +42,20 @@
  */
 
 #include "stdinc.h"
+#include "ratbox_lib.h"
+#include "struct.h"
 #include "client.h"		/* client struct */
-#include "irc_string.h"
 #include "hostmask.h"
+#include "reject.h"
 #include "send.h"		/* sendto_one */
 #include "numeric.h"		/* ERR_xxx */
 #include "ircd.h"		/* me */
-#include "msg.h"
 #include "parse.h"
+#include "match.h"
 #include "modules.h"
 #include "s_serv.h"
 #include "hash.h"
 #include "s_conf.h"
-#include "reject.h"
 
 static int mr_webirc(struct Client *, struct Client *, int, const char **);
 
@@ -64,7 +65,8 @@ struct Message webirc_msgtab = {
 };
 
 mapi_clist_av1 webirc_clist[] = { &webirc_msgtab, NULL };
-DECLARE_MODULE_AV1(webirc, NULL, NULL, webirc_clist, NULL, NULL, "$Revision: 20702 $");
+
+DECLARE_MODULE_AV1(webirc, NULL, NULL, webirc_clist, NULL, NULL, "$Revision: 26401 $");
 
 /*
  * mr_webirc - webirc message handler
@@ -80,57 +82,53 @@ mr_webirc(struct Client *client_p, struct Client *source_p, int parc, const char
 	struct ConfItem *aconf;
 	const char *encr;
 
-	if (!strchr(parv[4], '.') && !strchr(parv[4], ':'))
+	if(!strchr(parv[4], '.') && !strchr(parv[4], ':'))
 	{
 		sendto_one(source_p, "NOTICE * :Invalid IP");
 		return 0;
 	}
 
-	aconf = find_address_conf(client_p->host, client_p->sockhost, 
-				IsGotId(client_p) ? client_p->username : "webirc",
-				(struct sockaddr *) &client_p->localClient->ip,
-				client_p->localClient->ip.ss_family);
-	if (aconf == NULL || !(aconf->status & CONF_CLIENT))
+	aconf = find_address_conf(client_p->host, client_p->sockhost,
+				  IsGotId(client_p) ? client_p->username : "webirc",
+				  (struct sockaddr *)&client_p->localClient->ip,
+				  client_p->localClient->ip.ss_family);
+	if(aconf == NULL || !(aconf->status & CONF_CLIENT))
 		return 0;
-	if (!IsConfDoSpoofIp(aconf) || irccmp(aconf->name, "webirc."))
+	if(!IsConfDoSpoofIp(aconf) || irccmp(aconf->info.name, "webirc."))
 	{
 		/* XXX */
 		sendto_one(source_p, "NOTICE * :Not a CGI:IRC auth block");
 		return 0;
 	}
-	if (EmptyString(aconf->passwd))
+	if(EmptyString(aconf->passwd))
 	{
 		sendto_one(source_p, "NOTICE * :CGI:IRC auth blocks must have a password");
 		return 0;
 	}
 
-	if (EmptyString(parv[1]))
+	if(EmptyString(parv[1]))
 		encr = "";
-	else if (IsConfEncrypted(aconf))
+	else if(IsConfEncrypted(aconf))
 		encr = crypt(parv[1], aconf->passwd);
 	else
 		encr = parv[1];
 
-	if (strcmp(encr, aconf->passwd))
+	if(strcmp(encr, aconf->passwd))
 	{
 		sendto_one(source_p, "NOTICE * :CGI:IRC password incorrect");
 		return 0;
 	}
 
-
-	strlcpy(source_p->sockhost, parv[4], sizeof(source_p->sockhost));
-
+	rb_strlcpy(source_p->sockhost, parv[4], sizeof(source_p->sockhost));
 	if(strlen(parv[3]) <= HOSTLEN)
-		strlcpy(source_p->host, parv[3], sizeof(source_p->host));
+		rb_strlcpy(source_p->host, parv[3], sizeof(source_p->host));
 	else
-		strlcpy(source_p->host, source_p->sockhost, sizeof(source_p->host));
-	
-	del_unknown_ip(source_p);
-	inetpton_sock(parv[4], (struct sockaddr *)&source_p->localClient->ip);
+		rb_strlcpy(source_p->host, source_p->sockhost, sizeof(source_p->host));
+
+	rb_inet_pton_sock(parv[4], (struct sockaddr *)&source_p->localClient->ip);
 
 	/* Check dlines now, k/glines will be checked on registration */
-	if((aconf = find_dline((struct sockaddr *)&source_p->localClient->ip, 
-			       source_p->localClient->ip.ss_family)))
+	if((aconf = find_dline((struct sockaddr *)&source_p->localClient->ip)))
 	{
 		if(!(aconf->status & CONF_EXEMPTDLINE))
 		{
