@@ -1,10 +1,10 @@
 /*
  *  ircd-ratbox: A slightly useful ircd.
- *  ircd_signal.h: A header for ircd signals.
+ *  rb_signal.h: A header for ircd signals.
  *
  *  Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
  *  Copyright (C) 1996-2002 Hybrid Development Team
- *  Copyright (C) 2002-2004 ircd-ratbox development team
+ *  Copyright (C) 2002-2005 ircd-ratbox development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,15 +18,16 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
  *  USA
  *
- *  $Id: ircd_signal.h 16071 2004-03-15 17:58:08Z leeh $
+ *  $Id: ircd_signal.h 24244 2007-08-22 19:04:55Z androsyn $
  */
 
-#ifndef INCLUDED_ircd_signal_h
-#define INCLUDED_ircd_signal_h
+#ifndef INCLUDED_rb_signal_h
+#define INCLUDED_rb_signal_h
 
-extern void setup_signals(void);
+void setup_signals(void);
+void setup_reboot_signals(void);
 
-#endif /* INCLUDED_ircd_signal_h */
+#endif /* INCLUDED_rb_signal_h */
