@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
  *  Copyright (C) 1996-2002 Hybrid Development Team
- *  Copyright (C) 2002-2004 ircd-ratbox development team
+ *  Copyright (C) 2002-2005 ircd-ratbox development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,37 +18,34 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
  *  USA
  *
- *  $Id: s_user.h 23995 2007-07-17 02:37:06Z androsyn $
+ *  $Id: s_user.h 24650 2007-12-02 02:56:04Z androsyn $
  */
 
 #ifndef INCLUDED_s_user_h
 #define INCLUDED_s_user_h
 
-#include "config.h"
-
 struct Client;
 struct User;
 struct oper_conf;
 extern time_t LastUsedWallops;
 
-extern int valid_hostname(const char *hostname);
-extern int valid_username(const char *username);
+int valid_hostname(const char *hostname);
+int valid_username(const char *username);
 
-extern int user_mode(struct Client *, struct Client *, int, const char **);
-extern void send_umode(struct Client *, struct Client *, int, int, char *);
-extern void send_umode_out(struct Client *, struct Client *, int);
-extern int show_lusers(struct Client *source_p);
-extern int register_local_user(struct Client *, struct Client *, const char *);
+int user_mode(struct Client *, struct Client *, int, const char **);
+void send_umode(struct Client *, struct Client *, int, int, char *);
+void send_umode_out(struct Client *, struct Client *, int);
+int show_lusers(struct Client *source_p);
+int register_local_user(struct Client *, struct Client *, const char *);
 
-extern void introduce_client(struct Client *client_p, struct Client *source_p, 
-			    struct User *user, const char *nick);
+void introduce_client(struct Client *client_p, struct Client *source_p);
 
 extern int user_modes_from_c_to_bitmask[];
-extern void show_isupport(struct Client *);
+void show_isupport(struct Client *);
 
-extern int oper_up(struct Client *, struct oper_conf *);
+#define UserModeBitmask(c) user_modes_from_c_to_bitmask[(unsigned char)c]
 
 #endif
