@@ -18,31 +18,24 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
  *  USA
  *
- *  $Id: m_message.c 22840 2006-07-02 20:51:11Z jilles $
+ *  $Id: m_message.c 26094 2008-09-19 15:33:46Z androsyn $
  */
 
 #include "stdinc.h"
+#include "struct.h"
 #include "client.h"
 #include "ircd.h"
 #include "numeric.h"
-#include "common.h"
 #include "s_conf.h"
-#include "s_serv.h"
-#include "msg.h"
 #include "parse.h"
 #include "modules.h"
 #include "channel.h"
-#include "irc_string.h"
+#include "match.h"
 #include "hash.h"
-#include "class.h"
-#include "msg.h"
-#include "packet.h"
 #include "send.h"
-#include "event.h"
-#include "patricia.h"
 #include "s_newconf.h"
 
 static int m_message(int, const char *, struct Client *, struct Client *, int, const char **);
@@ -50,11 +43,11 @@ static int m_privmsg(struct Client *, struct Client *, int, const char **);
 static int m_notice(struct Client *, struct Client *, int, const char **);
 
 static void expire_tgchange(void *unused);
-
+struct ev_entry *tgchange_ev;
 static int
 modinit(void)
 {
-	eventAddIsh("expire_tgchange", expire_tgchange, NULL, 300);
+	tgchange_ev = rb_event_addish("expire_tgchange", expire_tgchange, NULL, 300);
 	expire_tgchange(NULL);
 	return 0;
 }
@@ -62,20 +55,22 @@ modinit(void)
 static void
 moddeinit(void)
 {
-	eventDelete(expire_tgchange, NULL);
+	rb_event_delete(tgchange_ev);
 }
 
 struct Message privmsg_msgtab = {
 	"PRIVMSG", 0, 0, 0, MFLG_SLOW | MFLG_UNREG,
 	{mg_unreg, {m_privmsg, 0}, {m_privmsg, 0}, mg_ignore, mg_ignore, {m_privmsg, 0}}
 };
+
 struct Message notice_msgtab = {
 	"NOTICE", 0, 0, 0, MFLG_SLOW,
 	{mg_unreg, {m_notice, 0}, {m_notice, 0}, {m_notice, 0}, mg_ignore, {m_notice, 0}}
 };
 
 mapi_clist_av1 message_clist[] = { &privmsg_msgtab, &notice_msgtab, NULL };
-DECLARE_MODULE_AV1(message, modinit, moddeinit, message_clist, NULL, NULL, "$Revision: 22840 $");
+
+DECLARE_MODULE_AV1(message, modinit, moddeinit, message_clist, NULL, NULL, "$Revision: 26094 $");
 
 struct entity
 {
@@ -89,8 +84,7 @@ static int build_target_list(int p_or_n, const char *command,
 			     struct Client *source_p, const char *nicks_channels, const char *text);
 
 static int flood_attack_client(int p_or_n, struct Client *source_p, struct Client *target_p);
-static int flood_attack_channel(int p_or_n, struct Client *source_p,
-				struct Channel *chptr, char *chname);
+static int flood_attack_channel(int p_or_n, struct Client *source_p, struct Channel *chptr);
 static struct Client *find_userhost(const char *, const char *, int *);
 
 #define ENTITY_NONE    0
@@ -115,8 +109,9 @@ static void msg_channel_flags(int p_or_n, const char *command,
 static void msg_client(int p_or_n, const char *command,
 		       struct Client *source_p, struct Client *target_p, const char *text);
 
-static void handle_special(int p_or_n, const char *command,
-			 struct Client *client_p, struct Client *source_p, const char *nick, const char *text);
+static void handle_special(const char *command,
+			   struct Client *client_p, struct Client *source_p, const char *nick,
+			   const char *text);
 
 /*
 ** m_privmsg
@@ -184,8 +179,7 @@ m_message(int p_or_n,
 	/* Finish the flood grace period if theyre not messaging themselves
 	 * as some clients (ircN) do this as a "lag check"
 	 */
-	if(MyClient(source_p) && !IsFloodDone(source_p) &&
-	   irccmp(source_p->name, parv[1]))
+	if(MyClient(source_p) && !IsFloodDone(source_p) && irccmp(source_p->name, parv[1]))
 		flood_endgrace(source_p);
 
 	if(build_target_list(p_or_n, command, client_p, source_p, parv[1], parv[2]) < 0)
@@ -193,24 +187,24 @@ m_message(int p_or_n,
 		return 0;
 	}
 
-	for (i = 0; i < ntargets; i++)
+	for(i = 0; i < ntargets; i++)
 	{
 		switch (targets[i].type)
 		{
 		case ENTITY_CHANNEL:
 			msg_channel(p_or_n, command, client_p, source_p,
-				    (struct Channel *) targets[i].ptr, parv[2]);
+				    (struct Channel *)targets[i].ptr, parv[2]);
 			break;
 
 		case ENTITY_CHANOPS_ON_CHANNEL:
 			msg_channel_flags(p_or_n, command, client_p, source_p,
-					  (struct Channel *) targets[i].ptr,
+					  (struct Channel *)targets[i].ptr,
 					  targets[i].flags, parv[2]);
 			break;
 
 		case ENTITY_CLIENT:
 			msg_client(p_or_n, command, source_p,
-				   (struct Client *) targets[i].ptr, parv[2]);
+				   (struct Client *)targets[i].ptr, parv[2]);
 			break;
 		}
 	}
@@ -245,11 +239,11 @@ build_target_list(int p_or_n, const char *command, struct Client *client_p,
 	struct Channel *chptr = NULL;
 	struct Client *target_p;
 
-	target_list = LOCAL_COPY(nicks_channels);	/* skip strcpy for non-lazyleafs */
+	target_list = LOCAL_COPY(nicks_channels);
 
 	ntargets = 0;
 
-	for (nick = strtoken(&p, target_list, ","); nick; nick = strtoken(&p, NULL, ","))
+	for(nick = rb_strtok_r(target_list, ",", &p); nick; nick = rb_strtok_r(NULL, ",", &p))
 	{
 		char *with_prefix;
 		/*
@@ -273,7 +267,7 @@ build_target_list(int p_or_n, const char *command, struct Client *client_p,
 							   me.name, source_p->name, nick);
 						return (1);
 					}
-					targets[ntargets].ptr = (void *) chptr;
+					targets[ntargets].ptr = (void *)chptr;
 					targets[ntargets++].type = ENTITY_CHANNEL;
 				}
 			}
@@ -302,7 +296,7 @@ build_target_list(int p_or_n, const char *command, struct Client *client_p,
 						   me.name, source_p->name, nick);
 					return (1);
 				}
-				targets[ntargets].ptr = (void *) target_p;
+				targets[ntargets].ptr = (void *)target_p;
 				targets[ntargets].type = ENTITY_CLIENT;
 				targets[ntargets++].flags = 0;
 			}
@@ -314,7 +308,7 @@ build_target_list(int p_or_n, const char *command, struct Client *client_p,
 		type = 0;
 		with_prefix = nick;
 		/*  allow %+@ if someone wants to do that */
-		for (;;)
+		for(;;)
 		{
 			if(*nick == '@')
 				type |= CHFL_CHANOP;
@@ -360,7 +354,7 @@ build_target_list(int p_or_n, const char *command, struct Client *client_p,
 							   me.name, source_p->name, nick);
 						return (1);
 					}
-					targets[ntargets].ptr = (void *) chptr;
+					targets[ntargets].ptr = (void *)chptr;
 					targets[ntargets].type = ENTITY_CHANOPS_ON_CHANNEL;
 					targets[ntargets++].flags = type;
 				}
@@ -376,21 +370,21 @@ build_target_list(int p_or_n, const char *command, struct Client *client_p,
 
 		if(strchr(nick, '@') || (IsOper(source_p) && (*nick == '$')))
 		{
-			handle_special(p_or_n, command, client_p, source_p, nick, text);
+			handle_special(command, client_p, source_p, nick, text);
 			continue;
 		}
 
 		/* no matching anything found - error if not NOTICE */
 		if(p_or_n != NOTICE)
 		{
-			/* dont give this numeric when source is local,
-			 * because its misleading --anfl
+			/* dont give this out when source is local, because
+			 * its misleading --anfl
 			 */
 			if(!MyClient(source_p) && IsDigit(*nick))
 				sendto_one(source_p, ":%s %d %s * :Target left IRC. "
-						"Failed to deliver: [%.20s]",
-						get_id(&me, source_p), ERR_NOSUCHNICK,
-						get_id(source_p, source_p), text);
+					   "Failed to deliver: [%.20s]",
+					   get_id(&me, source_p), ERR_NOSUCHNICK,
+					   get_id(source_p, source_p), text);
 			else
 				sendto_one_numeric(source_p, ERR_NOSUCHNICK,
 						   form_str(ERR_NOSUCHNICK), nick);
@@ -414,7 +408,7 @@ static int
 duplicate_ptr(void *ptr)
 {
 	int i;
-	for (i = 0; i < ntargets; i++)
+	for(i = 0; i < ntargets; i++)
 		if(targets[i].ptr == ptr)
 			return YES;
 	return NO;
@@ -433,7 +427,8 @@ duplicate_ptr(void *ptr)
  */
 static void
 msg_channel(int p_or_n, const char *command,
-	    struct Client *client_p, struct Client *source_p, struct Channel *chptr, const char *text)
+	    struct Client *client_p, struct Client *source_p, struct Channel *chptr,
+	    const char *text)
 {
 	int result;
 
@@ -441,18 +436,16 @@ msg_channel(int p_or_n, const char *command,
 	{
 		/* idle time shouldnt be reset by notices --fl */
 		if(p_or_n != NOTICE)
-			source_p->localClient->last = CurrentTime;
+			source_p->localClient->last = rb_current_time();
 	}
 
 	/* chanops and voiced can flood their own channel with impunity */
 	if((result = can_send(chptr, source_p, NULL)))
 	{
-		if(result == CAN_SEND_OPV ||
-		   !flood_attack_channel(p_or_n, source_p, chptr, chptr->chname))
+		if(result == CAN_SEND_OPV || !flood_attack_channel(p_or_n, source_p, chptr))
 		{
-			sendto_channel_flags(client_p, ALL_MEMBERS, source_p, chptr, 
-					      "%s %s :%s",
-					      command, chptr->chname, text);
+			sendto_channel_flags(client_p, ALL_MEMBERS, source_p, chptr,
+					     "%s %s :%s", command, chptr->chname, text);
 		}
 	}
 	else
@@ -499,7 +492,7 @@ msg_channel_flags(int p_or_n, const char *command, struct Client *client_p,
 	{
 		/* idletime shouldnt be reset by notice --fl */
 		if(p_or_n != NOTICE)
-			source_p->localClient->last = CurrentTime;
+			source_p->localClient->last = rb_current_time();
 	}
 
 	sendto_channel_flags(client_p, type, source_p, chptr, "%s %c%s :%s",
@@ -514,18 +507,18 @@ static void
 expire_tgchange(void *unused)
 {
 	tgchange *target;
-	dlink_node *ptr, *next_ptr;
+	rb_dlink_node *ptr, *next_ptr;
 
-	DLINK_FOREACH_SAFE(ptr, next_ptr, tgchange_list.head)
+	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, tgchange_list.head)
 	{
 		target = ptr->data;
 
-		if(target->expiry < CurrentTime)
+		if(target->expiry < rb_current_time())
 		{
-			dlinkDelete(ptr, &tgchange_list);
-			patricia_remove(tgchange_tree, target->pnode);
-			MyFree(target->ip);
-			MyFree(target);
+			rb_dlinkDelete(ptr, &tgchange_list);
+			rb_patricia_remove(tgchange_tree, target->pnode);
+			rb_free(target->ip);
+			rb_free(target);
 		}
 	}
 }
@@ -533,9 +526,8 @@ expire_tgchange(void *unused)
 static int
 add_target(struct Client *source_p, struct Client *target_p)
 {
-	int i, j;
-
-	/* can msg themselves without using any target slots */
+	unsigned int i, j;
+	/* messaging themselves, doesnt incur any penalties */
 	if(source_p == target_p)
 		return 1;
 
@@ -543,8 +535,7 @@ add_target(struct Client *source_p, struct Client *target_p)
 	{
 		/* hunt for an existing target */
 		for(i = PREV_FREE_TARGET(source_p), j = USED_TARGETS(source_p);
-			j;
-			--j, PREV_TARGET(i))
+		    j; --j, PREV_TARGET(i))
 		{
 			if(source_p->localClient->targets[i] == target_p)
 				return 1;
@@ -556,17 +547,17 @@ add_target(struct Client *source_p, struct Client *target_p)
 		if(!IsTGChange(source_p))
 		{
 			SetTGChange(source_p);
-			source_p->localClient->target_last = CurrentTime;
+			source_p->localClient->target_last = rb_current_time();
 		}
 		/* clear as many targets as we can */
-		else if((i = (CurrentTime - source_p->localClient->target_last) / 60))
+		else if((i = (rb_current_time() - source_p->localClient->target_last) / 60))
 		{
 			if(i > USED_TARGETS(source_p))
 				USED_TARGETS(source_p) = 0;
 			else
 				USED_TARGETS(source_p) -= i;
 
-			source_p->localClient->target_last = CurrentTime;
+			source_p->localClient->target_last = rb_current_time();
 		}
 		/* cant clear any, full target list */
 		else if(USED_TARGETS(source_p) == 10)
@@ -580,7 +571,7 @@ add_target(struct Client *source_p, struct Client *target_p)
 	 */
 	else
 	{
-		source_p->localClient->target_last = CurrentTime;
+		source_p->localClient->target_last = rb_current_time();
 		SetTGChange(source_p);
 	}
 
@@ -611,7 +602,7 @@ msg_client(int p_or_n, const char *command,
 		/* reset idle time for message only if its not to self 
 		 * and its not a notice */
 		if(p_or_n != NOTICE)
-			source_p->localClient->last = CurrentTime;
+			source_p->localClient->last = rb_current_time();
 
 		/* target change stuff, dont limit ctcp replies as that
 		 * would allow people to start filling up random users
@@ -623,7 +614,7 @@ msg_client(int p_or_n, const char *command,
 			if(!add_target(source_p, target_p))
 			{
 				sendto_one(source_p, form_str(ERR_TARGCHANGE),
-						me.name, source_p->name, target_p->name);
+					   me.name, source_p->name, target_p->name);
 				return;
 			}
 		}
@@ -631,9 +622,8 @@ msg_client(int p_or_n, const char *command,
 	else if(source_p->from == target_p->from)
 	{
 		sendto_realops_flags(UMODE_DEBUG, L_ALL,
-				"Send message to %s[%s] dropped from %s(Fake Dir)",
-				target_p->name, target_p->from->name,
-				source_p->name);
+				     "Send message to %s[%s] dropped from %s(Fake Dir)",
+				     target_p->name, target_p->from->name, source_p->name);
 		return;
 	}
 
@@ -665,7 +655,7 @@ msg_client(int p_or_n, const char *command,
 				}
 
 				if((target_p->localClient->last_caller_id_time +
-				    ConfigFileEntry.caller_id_wait) < CurrentTime)
+				    ConfigFileEntry.caller_id_wait) < rb_current_time())
 				{
 					if(p_or_n != NOTICE)
 						sendto_one_numeric(source_p, RPL_TARGNOTIFY,
@@ -676,10 +666,11 @@ msg_client(int p_or_n, const char *command,
 						   me.name, target_p->name, source_p->name,
 						   source_p->username, source_p->host);
 
-					target_p->localClient->last_caller_id_time = CurrentTime;
+					target_p->localClient->last_caller_id_time =
+						rb_current_time();
 				}
 				/* Only so opers can watch for floods */
-				(void) flood_attack_client(p_or_n, source_p, target_p);
+				(void)flood_attack_client(p_or_n, source_p, target_p);
 			}
 		}
 		else
@@ -691,8 +682,7 @@ msg_client(int p_or_n, const char *command,
 			 * and flooding    -- fl */
 			if(!MyClient(source_p) || IsOper(source_p) ||
 			   !flood_attack_client(p_or_n, source_p, target_p))
-				sendto_anywhere(target_p, source_p, command,
-						":%s", text);
+				sendto_anywhere(target_p, source_p, command, ":%s", text);
 		}
 	}
 	else if(!MyClient(source_p) || IsOper(source_p) ||
@@ -718,11 +708,12 @@ flood_attack_client(int p_or_n, struct Client *source_p, struct Client *target_p
 
 	if(GlobalSetOptions.floodcount && MyConnect(target_p) && IsClient(source_p))
 	{
-		if((target_p->localClient->first_received_message_time + 1) < CurrentTime)
+		if((target_p->localClient->first_received_message_time + 1) < rb_current_time())
 		{
-			delta = CurrentTime - target_p->localClient->first_received_message_time;
+			delta = rb_current_time() -
+				target_p->localClient->first_received_message_time;
 			target_p->localClient->received_number_of_privmsgs -= delta;
-			target_p->localClient->first_received_message_time = CurrentTime;
+			target_p->localClient->first_received_message_time = rb_current_time();
 			if(target_p->localClient->received_number_of_privmsgs <= 0)
 			{
 				target_p->localClient->received_number_of_privmsgs = 0;
@@ -745,9 +736,9 @@ flood_attack_client(int p_or_n, struct Client *source_p, struct Client *target_p
 				target_p->localClient->received_number_of_privmsgs += 2;
 			}
 			if(MyClient(source_p) && (p_or_n != NOTICE))
-				sendto_one(source_p,
-					   ":%s NOTICE %s :*** Message to %s throttled due to flooding",
-					   me.name, source_p->name, target_p->name);
+				sendto_one_notice(source_p,
+						  ":*** Message to %s throttled due to flooding",
+						  target_p->name);
 			return 1;
 		}
 		else
@@ -767,17 +758,17 @@ flood_attack_client(int p_or_n, struct Client *source_p, struct Client *target_p
  * side effects	- check for flood attack on target chptr
  */
 static int
-flood_attack_channel(int p_or_n, struct Client *source_p, struct Channel *chptr, char *chname)
+flood_attack_channel(int p_or_n, struct Client *source_p, struct Channel *chptr)
 {
 	int delta;
 
 	if(GlobalSetOptions.floodcount && MyClient(source_p))
 	{
-		if((chptr->first_received_message_time + 1) < CurrentTime)
+		if((chptr->first_received_message_time + 1) < rb_current_time())
 		{
-			delta = CurrentTime - chptr->first_received_message_time;
+			delta = rb_current_time() - chptr->first_received_message_time;
 			chptr->received_number_of_privmsgs -= delta;
-			chptr->first_received_message_time = CurrentTime;
+			chptr->first_received_message_time = rb_current_time();
 			if(chptr->received_number_of_privmsgs <= 0)
 			{
 				chptr->received_number_of_privmsgs = 0;
@@ -801,9 +792,9 @@ flood_attack_channel(int p_or_n, struct Client *source_p, struct Channel *chptr,
 				chptr->received_number_of_privmsgs += 2;
 			}
 			if(MyClient(source_p) && (p_or_n != NOTICE))
-				sendto_one(source_p,
-					   ":%s NOTICE %s :*** Message to %s throttled due to flooding",
-					   me.name, source_p->name, chptr->chname);
+				sendto_one_notice(source_p,
+						  ":*** Message to %s throttled due to flooding",
+						  chptr->chname);
 			return 1;
 		}
 		else
@@ -831,8 +822,8 @@ flood_attack_channel(int p_or_n, struct Client *source_p, struct Channel *chptr,
  *		  This disambiguates the syntax.
  */
 static void
-handle_special(int p_or_n, const char *command, struct Client *client_p,
-	     struct Client *source_p, const char *nick, const char *text)
+handle_special(const char *command, struct Client *client_p,
+	       struct Client *source_p, const char *nick, const char *text)
 {
 	struct Client *target_p;
 	char *host;
@@ -848,7 +839,7 @@ handle_special(int p_or_n, const char *command, struct Client *client_p,
 	{
 		if((target_p = find_server(source_p, server + 1)) == NULL)
 		{
-			sendto_one_numeric(source_p, ERR_NOSUCHSERVER, 
+			sendto_one_numeric(source_p, ERR_NOSUCHSERVER,
 					   form_str(ERR_NOSUCHSERVER), server + 1);
 			return;
 		}
@@ -859,7 +850,7 @@ handle_special(int p_or_n, const char *command, struct Client *client_p,
 		{
 			if(strchr(nick, '%') || (strncmp(nick, "opers", 5) == 0))
 			{
-				sendto_one_numeric(source_p, ERR_NOSUCHNICK, 
+				sendto_one_numeric(source_p, ERR_NOSUCHNICK,
 						   form_str(ERR_NOSUCHNICK), nick);
 				return;
 			}
@@ -868,9 +859,8 @@ handle_special(int p_or_n, const char *command, struct Client *client_p,
 		/* somewhere else.. */
 		if(!IsMe(target_p))
 		{
-			sendto_one(target_p, ":%s %s %s :%s", 
-				   get_id(source_p, target_p), 
-				   command, nick, text);
+			sendto_one(target_p, ":%s %s %s :%s",
+				   get_id(source_p, target_p), command, nick, text);
 			return;
 		}
 
@@ -902,12 +892,10 @@ handle_special(int p_or_n, const char *command, struct Client *client_p,
 				*--host = '%';
 
 			if(count == 1)
-				sendto_anywhere(target_p, source_p, command, 
-						":%s", text);
+				sendto_anywhere(target_p, source_p, command, ":%s", text);
 			else
 				sendto_one(source_p, form_str(ERR_TOOMANYTARGETS),
-					   get_id(&me, source_p),
-					   get_id(source_p, source_p), nick);
+					   get_id(&me, source_p), get_id(source_p, source_p), nick);
 		}
 	}
 
@@ -923,9 +911,9 @@ handle_special(int p_or_n, const char *command, struct Client *client_p,
 			nick++;
 		else if(MyOper(source_p))
 		{
-			sendto_one(source_p,
-				   ":%s NOTICE %s :The command %s %s is no longer supported, please use $%s",
-				   me.name, source_p->name, command, nick, nick);
+			sendto_one_notice(source_p,
+					  ":The command %s %s is no longer supported, please use $%s",
+					  command, nick, nick);
 			return;
 		}
 
@@ -935,7 +923,7 @@ handle_special(int p_or_n, const char *command, struct Client *client_p,
 					   form_str(ERR_NOTOPLEVEL), nick);
 			return;
 		}
-		while (*++s)
+		while(*++s)
 			if(*s == '.' || *s == '*' || *s == '?')
 				break;
 		if(*s == '*' || *s == '?')
@@ -969,17 +957,16 @@ find_userhost(const char *user, const char *host, int *count)
 	struct Client *c2ptr;
 	struct Client *res = NULL;
 	char *u = LOCAL_COPY(user);
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 	*count = 0;
 	if(collapse(u) != NULL)
 	{
-		DLINK_FOREACH(ptr, global_client_list.head)
+		RB_DLINK_FOREACH(ptr, global_client_list.head)
 		{
 			c2ptr = ptr->data;
 			if(!MyClient(c2ptr))	/* implies mine and an user */
 				continue;
-			if((!host || match(host, c2ptr->host)) &&
-			   irccmp(u, c2ptr->username) == 0)
+			if((!host || match(host, c2ptr->host)) && irccmp(u, c2ptr->username) == 0)
 			{
 				(*count)++;
 				res = c2ptr;
