@@ -18,19 +18,18 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
  *  USA
  *
- *  $Id: m_mode.c 23253 2006-10-29 12:28:38Z leeh $
+ *  $Id: m_mode.c 26094 2008-09-19 15:33:46Z androsyn $
  */
 
 #include "stdinc.h"
-#include "tools.h"
-#include "balloc.h"
+#include "struct.h"
 #include "channel.h"
 #include "client.h"
 #include "hash.h"
-#include "irc_string.h"
+#include "match.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "s_user.h"
@@ -38,11 +37,8 @@
 #include "s_serv.h"
 #include "s_log.h"
 #include "send.h"
-#include "msg.h"
 #include "parse.h"
 #include "modules.h"
-#include "packet.h"
-#include "sprintf_irc.h"
 #include "s_newconf.h"
 
 static int m_mode(struct Client *, struct Client *, int, const char **);
@@ -54,32 +50,37 @@ struct Message mode_msgtab = {
 	"MODE", 0, 0, 0, MFLG_SLOW,
 	{mg_unreg, {m_mode, 2}, {m_mode, 3}, {ms_mode, 3}, mg_ignore, {m_mode, 2}}
 };
+
 struct Message tmode_msgtab = {
 	"TMODE", 0, 0, 0, MFLG_SLOW,
 	{mg_ignore, mg_ignore, {ms_tmode, 4}, {ms_tmode, 4}, mg_ignore, mg_ignore}
 };
+
 struct Message bmask_msgtab = {
 	"BMASK", 0, 0, 0, MFLG_SLOW,
 	{mg_ignore, mg_ignore, mg_ignore, {ms_bmask, 5}, mg_ignore, mg_ignore}
 };
 
 mapi_clist_av1 mode_clist[] = { &mode_msgtab, &tmode_msgtab, &bmask_msgtab, NULL };
-DECLARE_MODULE_AV1(mode, NULL, NULL, mode_clist, NULL, NULL, "$Revision: 23253 $");
+
+DECLARE_MODULE_AV1(mode, NULL, NULL, mode_clist, NULL, NULL, "$Revision: 26094 $");
 
 /* bitmasks for error returns, so we send once per call */
+#define SM_ERR_NOTS             0x00000001	/* No TS on channel */
 #define SM_ERR_NOOPS            0x00000002	/* No chan ops */
 #define SM_ERR_UNKNOWN          0x00000004
+#define SM_ERR_RPL_C            0x00000008
 #define SM_ERR_RPL_B            0x00000010
 #define SM_ERR_RPL_E            0x00000020
 #define SM_ERR_NOTONCHANNEL     0x00000040	/* Not on channel */
 #define SM_ERR_RPL_I            0x00000100
+#define SM_ERR_RPL_D            0x00000200
 
 static void set_channel_mode(struct Client *, struct Client *,
-			     struct Channel *, struct membership *,
-			     int, const char **);
+			     struct Channel *, struct membership *, int, const char **);
 
-static int add_id(struct Client *source_p, struct Channel *chptr, 
-		  const char *banid, dlink_list *list, long mode_type);
+static int add_id(struct Client *source_p, struct Channel *chptr,
+		  const char *banid, rb_dlink_list *list, long mode_type);
 
 static struct ChModeChange mode_changes[BUFSIZE];
 static int mode_count;
@@ -110,7 +111,7 @@ m_mode(struct Client *client_p, struct Client *source_p, int parc, const char *p
 		if(EmptyString(dest))
 		{
 			sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS),
-					me.name, source_p->name, "MODE");
+				   me.name, source_p->name, "MODE");
 			return 0;
 		}
 	}
@@ -125,8 +126,7 @@ m_mode(struct Client *client_p, struct Client *source_p, int parc, const char *p
 
 	if(!check_channel_name(dest))
 	{
-		sendto_one_numeric(source_p, ERR_BADCHANNAME,
-				   form_str(ERR_BADCHANNAME), parv[1]);
+		sendto_one_numeric(source_p, ERR_BADCHANNAME, form_str(ERR_BADCHANNAME), parv[1]);
 		return 0;
 	}
 
@@ -147,8 +147,7 @@ m_mode(struct Client *client_p, struct Client *source_p, int parc, const char *p
 
 		sendto_one(source_p, form_str(RPL_CHANNELMODEIS),
 			   me.name, source_p->name, parv[1],
-			   operspy ? channel_modes(chptr, &me) :
-			    channel_modes(chptr, source_p));
+			   operspy ? channel_modes(chptr, &me) : channel_modes(chptr, source_p));
 
 		sendto_one(source_p, form_str(RPL_CREATIONTIME),
 			   me.name, source_p->name, parv[1], chptr->channelts);
@@ -167,8 +166,7 @@ m_mode(struct Client *client_p, struct Client *source_p, int parc, const char *p
 				flood_endgrace(source_p);
 		}
 
-		set_channel_mode(client_p, source_p, chptr, msptr, 
-			         parc - n, parv + n);
+		set_channel_mode(client_p, source_p, chptr, msptr, parc - n, parv + n);
 	}
 
 	return 0;
@@ -188,12 +186,11 @@ ms_mode(struct Client *client_p, struct Client *source_p, int parc, const char *
 		return 0;
 	}
 
-	set_channel_mode(client_p, source_p, chptr, NULL,
-			 parc - 2, parv + 2);
+	set_channel_mode(client_p, source_p, chptr, NULL, parc - 2, parv + 2);
 
 	return 0;
 }
-	
+
 static int
 ms_tmode(struct Client *client_p, struct Client *source_p, int parc, const char *parv[])
 {
@@ -203,9 +200,7 @@ ms_tmode(struct Client *client_p, struct Client *source_p, int parc, const char
 	/* Now, try to find the channel in question */
 	if(!IsChanPrefix(parv[2][0]) || !check_channel_name(parv[2]))
 	{
-		sendto_one_numeric(source_p, ERR_BADCHANNAME,
-				   form_str(ERR_BADCHANNAME), 
-				   parv[2]);
+		sendto_one_numeric(source_p, ERR_BADCHANNAME, form_str(ERR_BADCHANNAME), parv[2]);
 		return 0;
 	}
 
@@ -224,8 +219,7 @@ ms_tmode(struct Client *client_p, struct Client *source_p, int parc, const char
 
 	if(IsServer(source_p))
 	{
-		set_channel_mode(client_p, source_p, chptr, NULL,
-				 parc - 3, parv + 3);
+		set_channel_mode(client_p, source_p, chptr, NULL, parc - 3, parv + 3);
 	}
 	else
 	{
@@ -235,8 +229,7 @@ ms_tmode(struct Client *client_p, struct Client *source_p, int parc, const char
 		if(is_deop(msptr))
 			return 0;
 
-		set_channel_mode(client_p, source_p, chptr, msptr, 
-			         parc - 3, parv + 3);
+		set_channel_mode(client_p, source_p, chptr, msptr, parc - 3, parv + 3);
 	}
 
 	return 0;
@@ -248,7 +241,7 @@ ms_bmask(struct Client *client_p, struct Client *source_p, int parc, const char
 	static char modebuf[BUFSIZE];
 	static char parabuf[BUFSIZE];
 	struct Channel *chptr;
-	dlink_list *banlist;
+	rb_dlink_list *banlist;
 	const char *s;
 	char *t;
 	char *mbuf;
@@ -272,50 +265,49 @@ ms_bmask(struct Client *client_p, struct Client *source_p, int parc, const char
 	if(atol(parv[1]) > chptr->channelts)
 		return 0;
 
-	switch(parv[3][0])
+	switch (parv[3][0])
 	{
-		case 'b':
-			banlist = &chptr->banlist;
-			mode_type = CHFL_BAN;
-			mems = ALL_MEMBERS;
-			break;
+	case 'b':
+		banlist = &chptr->banlist;
+		mode_type = CHFL_BAN;
+		mems = ALL_MEMBERS;
+		break;
 
-		case 'e':
-			banlist = &chptr->exceptlist;
-			mode_type = CHFL_EXCEPTION;
-			needcap = CAP_EX;
-			mems = ONLY_CHANOPS;
-			break;
+	case 'e':
+		banlist = &chptr->exceptlist;
+		mode_type = CHFL_EXCEPTION;
+		needcap = CAP_EX;
+		mems = ONLY_CHANOPS;
+		break;
 
-		case 'I':
-			banlist = &chptr->invexlist;
-			mode_type = CHFL_INVEX;
-			needcap = CAP_IE;
-			mems = ONLY_CHANOPS;
-			break;
+	case 'I':
+		banlist = &chptr->invexlist;
+		mode_type = CHFL_INVEX;
+		needcap = CAP_IE;
+		mems = ONLY_CHANOPS;
+		break;
 
 		/* maybe we should just blindly propagate this? */
-		default:
-			return 0;
+	default:
+		return 0;
 	}
 
 	parabuf[0] = '\0';
 	s = LOCAL_COPY(parv[4]);
 
-	mlen = ircsprintf(modebuf, ":%s MODE %s +",
-			  source_p->name, chptr->chname);
+	mlen = rb_sprintf(modebuf, ":%s MODE %s +", source_p->name, chptr->chname);
 	mbuf = modebuf + mlen;
 	pbuf = parabuf;
 
 	while(*s == ' ')
 		s++;
 
-	/* next char isnt a space, point t to the next one */
+	/* next char isnt a space, point t to next one */
 	if((t = strchr(s, ' ')) != NULL)
 	{
 		*t++ = '\0';
 
-		/* double spaces will break the parser */
+		/* double spaces break parser */
 		while(*t == ' ')
 			t++;
 	}
@@ -341,8 +333,7 @@ ms_bmask(struct Client *client_p, struct Client *source_p, int parc, const char
 			{
 				*mbuf = '\0';
 				*(pbuf - 1) = '\0';
-				sendto_channel_local(mems, chptr, "%s %s",
-						     modebuf, parabuf);
+				sendto_channel_local(mems, chptr, "%s %s", modebuf, parabuf);
 				sendto_server(client_p, chptr, needcap, CAP_TS6,
 					      "%s %s", modebuf, parabuf);
 
@@ -352,13 +343,13 @@ ms_bmask(struct Client *client_p, struct Client *source_p, int parc, const char
 			}
 
 			*mbuf++ = parv[3][0];
-			arglen = ircsprintf(pbuf, "%s ", s);
+			arglen = rb_sprintf(pbuf, "%s ", s);
 			pbuf += arglen;
 			plen += arglen;
 			modecount++;
 		}
 
-nextban:
+	      nextban:
 		s = t;
 
 		if(s != NULL)
@@ -377,14 +368,12 @@ nextban:
 	{
 		*mbuf = '\0';
 		*(pbuf - 1) = '\0';
-		sendto_channel_local(mems, chptr, "%s %s",
-				     modebuf, parabuf);
-		sendto_server(client_p, chptr, needcap, CAP_TS6,
-			      "%s %s", modebuf, parabuf);
+		sendto_channel_local(mems, chptr, "%s %s", modebuf, parabuf);
+		sendto_server(client_p, chptr, needcap, CAP_TS6, "%s %s", modebuf, parabuf);
 	}
 
-	sendto_server(client_p, chptr, CAP_TS6|needcap, NOCAPS, ":%s BMASK %ld %s %s :%s",
-		      source_p->id, (long) chptr->channelts, chptr->chname, parv[3], parv[4]);
+	sendto_server(client_p, chptr, CAP_TS6 | needcap, NOCAPS, ":%s BMASK %ld %s %s :%s",
+		      source_p->id, (long)chptr->channelts, chptr->chname, parv[3], parv[4]);
 	return 0;
 }
 
@@ -396,28 +385,29 @@ nextban:
  */
 static int
 add_id(struct Client *source_p, struct Channel *chptr, const char *banid,
-       dlink_list *list, long mode_type)
+       rb_dlink_list *list, long mode_type)
 {
 	struct Ban *actualBan;
 	static char who[BANLEN];
 	char *realban = LOCAL_COPY(banid);
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 
 	/* dont let local clients overflow the banlist, or set redundant
 	 * bans
 	 */
 	if(MyClient(source_p))
 	{
-		if((dlink_list_length(&chptr->banlist) + dlink_list_length(&chptr->exceptlist) + dlink_list_length(&chptr->invexlist)) >= ConfigChannel.max_bans)
+		if((rb_dlink_list_length(&chptr->banlist) +
+		    rb_dlink_list_length(&chptr->exceptlist) +
+		    rb_dlink_list_length(&chptr->invexlist)) >=
+		   (unsigned long)ConfigChannel.max_bans)
 		{
 			sendto_one(source_p, form_str(ERR_BANLISTFULL),
 				   me.name, source_p->name, chptr->chname, realban);
 			return 0;
 		}
 
-		collapse(realban);
-
-		DLINK_FOREACH(ptr, list->head)
+		RB_DLINK_FOREACH(ptr, list->head)
 		{
 			actualBan = ptr->data;
 			if(match(actualBan->banstr, realban))
@@ -427,7 +417,7 @@ add_id(struct Client *source_p, struct Channel *chptr, const char *banid,
 	/* dont let remotes set duplicates */
 	else
 	{
-		DLINK_FOREACH(ptr, list->head)
+		RB_DLINK_FOREACH(ptr, list->head)
 		{
 			actualBan = ptr->data;
 			if(!irccmp(actualBan->banstr, realban))
@@ -436,20 +426,19 @@ add_id(struct Client *source_p, struct Channel *chptr, const char *banid,
 	}
 
 
-	if(IsPerson(source_p))
-		ircsprintf(who, "%s!%s@%s",
-			   source_p->name, source_p->username, source_p->host);
+	if(IsClient(source_p))
+		rb_sprintf(who, "%s!%s@%s", source_p->name, source_p->username, source_p->host);
 	else
-		strlcpy(who, source_p->name, sizeof(who));
+		rb_strlcpy(who, source_p->name, sizeof(who));
 
 	actualBan = allocate_ban(realban, who);
-	actualBan->when = CurrentTime;
+	actualBan->when = rb_current_time();
 
-	dlinkAdd(actualBan, &actualBan->node, list);
+	rb_dlinkAdd(actualBan, &actualBan->node, list);
 
 	/* invalidate the can_send() cache */
 	if(mode_type == CHFL_BAN || mode_type == CHFL_EXCEPTION)
-		chptr->bants++;
+		chptr->ban_serial++;
 
 	return 1;
 }
@@ -461,27 +450,26 @@ add_id(struct Client *source_p, struct Channel *chptr, const char *banid,
  * side effects - given id is removed from the appropriate list
  */
 static int
-del_id(struct Channel *chptr, const char *banid, dlink_list *list,
-       long mode_type)
+del_id(struct Channel *chptr, const char *banid, rb_dlink_list *list, long mode_type)
 {
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 	struct Ban *banptr;
 
 	if(EmptyString(banid))
 		return 0;
 
-	DLINK_FOREACH(ptr, list->head)
+	RB_DLINK_FOREACH(ptr, list->head)
 	{
 		banptr = ptr->data;
 
 		if(irccmp(banid, banptr->banstr) == 0)
 		{
-			dlinkDelete(&banptr->node, list);
+			rb_dlinkDelete(&banptr->node, list);
 			free_ban(banptr);
 
 			/* invalidate the can_send() cache */
 			if(mode_type == CHFL_BAN || mode_type == CHFL_EXCEPTION)
-				chptr->bants++;
+				chptr->ban_serial++;
 
 			return 1;
 		}
@@ -504,7 +492,7 @@ check_string(char *s)
 	if(!(s && *s))
 		return splat;
 
-	for (; *s; ++s)
+	for(; *s; ++s)
 	{
 		if(IsSpace(*s))
 		{
@@ -608,7 +596,7 @@ pretty_mask(const char *idmask)
 		host[HOSTLEN] = '\0';
 	}
 
-	mask_pos += ircsprintf(mask_buf + mask_pos, "%s!%s@%s", nick, user, host) + 1;
+	mask_pos += rb_sprintf(mask_buf + mask_pos, "%s!%s@%s", nick, user, host) + 1;
 
 	/* restore mask, since we may need to use it again later */
 	if(at)
@@ -637,7 +625,7 @@ fix_key(char *arg)
 {
 	u_char *s, *t, c;
 
-	for (s = t = (u_char *) arg; (c = *s); s++)
+	for(s = t = (u_char *) arg; (c = *s); s++)
 	{
 		c &= 0x7f;
 		if(c != ':' && c != ',' && c > ' ')
@@ -660,7 +648,7 @@ fix_key_remote(char *arg)
 {
 	u_char *s, *t, c;
 
-	for (s = t = (u_char *) arg; (c = *s); s++)
+	for(s = t = (u_char *) arg; (c = *s); s++)
 	{
 		c &= 0x7f;
 		if((c != 0x0a) && (c != ':') && (c != ',') && (c != 0x0d) && (c != ' '))
@@ -676,7 +664,7 @@ fix_key_remote(char *arg)
  * The handlers for each specific mode.
  */
 static void
-chm_nosuch(struct Client *source_p, struct Channel *chptr, 
+chm_nosuch(struct Client *source_p, struct Channel *chptr,
 	   int alevel, int parc, int *parn,
 	   const char **parv, int *errors, int dir, char c, long mode_type)
 {
@@ -687,7 +675,7 @@ chm_nosuch(struct Client *source_p, struct Channel *chptr,
 }
 
 static void
-chm_simple(struct Client *source_p, struct Channel *chptr, 
+chm_simple(struct Client *source_p, struct Channel *chptr,
 	   int alevel, int parc, int *parn,
 	   const char **parv, int *errors, int dir, char c, long mode_type)
 {
@@ -732,14 +720,14 @@ chm_simple(struct Client *source_p, struct Channel *chptr,
 }
 
 static void
-chm_ban(struct Client *source_p, struct Channel *chptr, 
+chm_ban(struct Client *source_p, struct Channel *chptr,
 	int alevel, int parc, int *parn,
 	const char **parv, int *errors, int dir, char c, long mode_type)
 {
 	const char *mask;
 	const char *raw_mask;
-	dlink_list *list;
-	dlink_node *ptr;
+	rb_dlink_list *list;
+	rb_dlink_node *ptr;
 	struct Ban *banptr;
 	int errorval;
 	int rpl_list;
@@ -747,58 +735,57 @@ chm_ban(struct Client *source_p, struct Channel *chptr,
 	int caps;
 	int mems;
 
-	switch(mode_type)
+	switch (mode_type)
 	{
-		case CHFL_BAN:
-			list = &chptr->banlist;
-			errorval = SM_ERR_RPL_B;
-			rpl_list = RPL_BANLIST;
-			rpl_endlist = RPL_ENDOFBANLIST;
-			mems = ALL_MEMBERS;
-			caps = 0;
-			break;
+	case CHFL_BAN:
+		list = &chptr->banlist;
+		errorval = SM_ERR_RPL_B;
+		rpl_list = RPL_BANLIST;
+		rpl_endlist = RPL_ENDOFBANLIST;
+		mems = ALL_MEMBERS;
+		caps = 0;
+		break;
 
-		case CHFL_EXCEPTION:
-			/* if +e is disabled, allow all but +e locally */
-			if(!ConfigChannel.use_except && MyClient(source_p) &&
-			   ((dir == MODE_ADD) && (parc > *parn)))
-				return;
-
-			list = &chptr->exceptlist;
-			errorval = SM_ERR_RPL_E;
-			rpl_list = RPL_EXCEPTLIST;
-			rpl_endlist = RPL_ENDOFEXCEPTLIST;
-			caps = CAP_EX;
-			
-			if(ConfigChannel.use_except || (dir == MODE_DEL))
-				mems = ONLY_CHANOPS;
-			else
-				mems = ONLY_SERVERS;
-			break;
-
-		case CHFL_INVEX:
-			/* if +I is disabled, allow all but +I locally */
-			if(!ConfigChannel.use_invex && MyClient(source_p) &&
-					(dir == MODE_ADD) && (parc > *parn))
-				return;
-
-			list = &chptr->invexlist;
-			errorval = SM_ERR_RPL_I;
-			rpl_list = RPL_INVITELIST;
-			rpl_endlist = RPL_ENDOFINVITELIST;
-			caps = CAP_IE;
-
-			if(ConfigChannel.use_invex || (dir == MODE_DEL))
-				mems = ONLY_CHANOPS;
-			else
-				mems = ONLY_SERVERS;
-			break;
-
-		default:
-			sendto_realops_flags(UMODE_ALL, L_ALL,
-					     "chm_ban() called with unknown type!");
+	case CHFL_EXCEPTION:
+		/* if +e is disabled, allow all but +e locally */
+		if(!ConfigChannel.use_except && MyClient(source_p) &&
+		   ((dir == MODE_ADD) && (parc > *parn)))
 			return;
-			break;
+
+		list = &chptr->exceptlist;
+		errorval = SM_ERR_RPL_E;
+		rpl_list = RPL_EXCEPTLIST;
+		rpl_endlist = RPL_ENDOFEXCEPTLIST;
+		caps = CAP_EX;
+
+		if(ConfigChannel.use_except || (dir == MODE_DEL))
+			mems = ONLY_CHANOPS;
+		else
+			mems = ONLY_SERVERS;
+		break;
+
+	case CHFL_INVEX:
+		/* if +I is disabled, allow all but +I locally */
+		if(!ConfigChannel.use_invex && MyClient(source_p) &&
+		   (dir == MODE_ADD) && (parc > *parn))
+			return;
+
+		list = &chptr->invexlist;
+		errorval = SM_ERR_RPL_I;
+		rpl_list = RPL_INVITELIST;
+		rpl_endlist = RPL_ENDOFINVITELIST;
+		caps = CAP_IE;
+
+		if(ConfigChannel.use_invex || (dir == MODE_DEL))
+			mems = ONLY_CHANOPS;
+		else
+			mems = ONLY_SERVERS;
+		break;
+
+	default:
+		sendto_realops_flags(UMODE_ALL, L_ALL, "chm_ban() called with unknown type!");
+		return;
+		break;
 	}
 
 	if(dir == 0 || parc <= *parn)
@@ -817,15 +804,14 @@ chm_ban(struct Client *source_p, struct Channel *chptr,
 			return;
 		}
 
-		DLINK_FOREACH(ptr, list->head)
+		RB_DLINK_FOREACH(ptr, list->head)
 		{
 			banptr = ptr->data;
 			sendto_one(source_p, form_str(rpl_list),
 				   me.name, source_p->name, chptr->chname,
 				   banptr->banstr, banptr->who, banptr->when);
 		}
-		sendto_one(source_p, form_str(rpl_endlist),
-			   me.name, source_p->name, chptr->chname);
+		sendto_one(source_p, form_str(rpl_endlist), me.name, source_p->name, chptr->chname);
 		return;
 	}
 
@@ -899,7 +885,7 @@ chm_ban(struct Client *source_p, struct Channel *chptr,
 }
 
 static void
-chm_op(struct Client *source_p, struct Channel *chptr, 
+chm_op(struct Client *source_p, struct Channel *chptr,
        int alevel, int parc, int *parn,
        const char **parv, int *errors, int dir, char c, long mode_type)
 {
@@ -925,8 +911,7 @@ chm_op(struct Client *source_p, struct Channel *chptr,
 	/* empty nick */
 	if(EmptyString(opnick))
 	{
-		sendto_one_numeric(source_p, ERR_NOSUCHNICK,
-				   form_str(ERR_NOSUCHNICK), "*");
+		sendto_one_numeric(source_p, ERR_NOSUCHNICK, form_str(ERR_NOSUCHNICK), "*");
 		return;
 	}
 
@@ -972,8 +957,7 @@ chm_op(struct Client *source_p, struct Channel *chptr,
 		if(MyClient(source_p) && IsService(targ_p))
 		{
 			sendto_one(source_p, form_str(ERR_ISCHANSERVICE),
-					me.name, source_p->name, targ_p->name,
-					chptr->chname);
+				   me.name, source_p->name, targ_p->name, chptr->chname);
 			return;
 		}
 #endif
@@ -1018,8 +1002,7 @@ chm_voice(struct Client *source_p, struct Channel *chptr,
 	/* empty nick */
 	if(EmptyString(opnick))
 	{
-		sendto_one_numeric(source_p, ERR_NOSUCHNICK, 
-				   form_str(ERR_NOSUCHNICK), "*");
+		sendto_one_numeric(source_p, ERR_NOSUCHNICK, form_str(ERR_NOSUCHNICK), "*");
 		return;
 	}
 
@@ -1099,7 +1082,7 @@ chm_limit(struct Client *source_p, struct Channel *chptr,
 		if(EmptyString(lstr) || (limit = atoi(lstr)) <= 0)
 			return;
 
-		ircsprintf(limitstr, "%d", limit);
+		rb_sprintf(limitstr, "%d", limit);
 
 		mode_changes[mode_count].letter = c;
 		mode_changes[mode_count].dir = MODE_ADD;
@@ -1161,7 +1144,7 @@ chm_key(struct Client *source_p, struct Channel *chptr,
 			return;
 
 		s_assert(key[0] != ' ');
-		strlcpy(chptr->mode.key, key, sizeof(chptr->mode.key));
+		rb_strlcpy(chptr->mode.key, key, sizeof(chptr->mode.key));
 
 		mode_changes[mode_count].letter = c;
 		mode_changes[mode_count].dir = MODE_ADD;
@@ -1176,7 +1159,7 @@ chm_key(struct Client *source_p, struct Channel *chptr,
 		static char splat[] = "*";
 		int i;
 
-		if (parc > *parn)
+		if(parc > *parn)
 			(*parn)++;
 
 		if(!(*chptr->mode.key))
@@ -1207,8 +1190,8 @@ chm_key(struct Client *source_p, struct Channel *chptr,
 #ifdef ENABLE_SERVICES
 static void
 chm_regonly(struct Client *source_p, struct Channel *chptr,
-		int alevel, int parc, int *parn,
-		const char **parv, int *errors, int dir, char c, long mode_type)
+	    int alevel, int parc, int *parn,
+	    const char **parv, int *errors, int dir, char c, long mode_type)
 {
 	if(alevel != CHFL_CHANOP)
 	{
@@ -1241,12 +1224,54 @@ chm_regonly(struct Client *source_p, struct Channel *chptr,
 }
 #endif
 
+
+static void
+chm_sslonly(struct Client *source_p, struct Channel *chptr,
+	    int alevel, int parc, int *parn,
+	    const char **parv, int *errors, int dir, char c, long mode_type)
+{
+	if(alevel != CHFL_CHANOP)
+	{
+		if(!(*errors & SM_ERR_NOOPS))
+			sendto_one(source_p, form_str(ERR_CHANOPRIVSNEEDED),
+				   me.name, source_p->name, chptr->chname);
+		*errors |= SM_ERR_NOOPS;
+		return;
+	}
+
+	if(dir == MODE_QUERY)
+		return;
+
+	if(((dir == MODE_ADD) && (chptr->mode.mode & MODE_SSLONLY)) ||
+	   ((dir == MODE_DEL) && !(chptr->mode.mode & MODE_SSLONLY)))
+		return;
+
+	/* do not allow our clients to set use_sslonly if it is disabled
+	 * we do however allow them to remove it if it gets set 
+	 */
+	if(dir == MODE_ADD && MyClient(source_p) && ConfigChannel.use_sslonly == FALSE)
+		return;
+
+	if(dir == MODE_ADD)
+		chptr->mode.mode |= MODE_SSLONLY;
+	else
+		chptr->mode.mode &= ~MODE_SSLONLY;
+
+	mode_changes[mode_count].letter = c;
+	mode_changes[mode_count].dir = dir;
+	mode_changes[mode_count].caps = 0;
+	mode_changes[mode_count].nocaps = 0;
+	mode_changes[mode_count].mems = ALL_MEMBERS;
+	mode_changes[mode_count].id = NULL;
+	mode_changes[mode_count++].arg = NULL;
+}
+
+
 struct ChannelMode
 {
-	void (*func) (struct Client *source_p, struct Channel *chptr,
-		      int alevel, int parc, int *parn, 
-		      const char **parv, int *errors, int dir, 
-		      char c, long mode_type);
+	void (*func) (struct Client * source_p, struct Channel * chptr,
+		      int alevel, int parc, int *parn,
+		      const char **parv, int *errors, int dir, char c, long mode_type);
 	long mode_type;
 };
 
@@ -1272,7 +1297,7 @@ static struct ChannelMode ModeTable[255] =
   {chm_nosuch,	0 },			/* P */
   {chm_nosuch,	0 },			/* Q */
   {chm_nosuch,	0 },			/* R */
-  {chm_nosuch,	0 },			/* S */
+  {chm_sslonly,  MODE_SSLONLY },         /* S */
   {chm_nosuch,	0 },			/* T */
   {chm_nosuch,	0 },			/* U */
   {chm_nosuch,	0 },			/* V */
@@ -1336,8 +1361,7 @@ get_channel_access(struct Client *source_p, struct membership *msptr)
  */
 void
 set_channel_mode(struct Client *client_p, struct Client *source_p,
-		 struct Channel *chptr, struct membership *msptr,
-		 int parc, const char *parv[])
+		 struct Channel *chptr, struct membership *msptr, int parc, const char *parv[])
 {
 	static char modebuf[BUFSIZE];
 	static char parabuf[BUFSIZE];
@@ -1359,7 +1383,7 @@ set_channel_mode(struct Client *client_p, struct Client *source_p,
 
 	alevel = get_channel_access(source_p, msptr);
 
-	for (; (c = *ml) != 0; ml++)
+	for(; (c = *ml) != 0; ml++)
 	{
 		switch (c)
 		{
@@ -1378,8 +1402,8 @@ set_channel_mode(struct Client *client_p, struct Client *source_p,
 			else
 				table_position = c - 'A' + 1;
 			ModeTable[table_position].func(source_p, chptr, alevel,
-						       parc, &parn, parv, 
-						       &errors, dir, c, 
+						       parc, &parn, parv,
+						       &errors, dir, c,
 						       ModeTable[table_position].mode_type);
 			break;
 		}
@@ -1390,11 +1414,10 @@ set_channel_mode(struct Client *client_p, struct Client *source_p,
 		return;
 
 	if(IsServer(source_p))
-		mlen = ircsprintf(modebuf, ":%s MODE %s ", 
-				  source_p->name, chptr->chname);
+		mlen = rb_sprintf(modebuf, ":%s MODE %s ", source_p->name, chptr->chname);
 	else
-		mlen = ircsprintf(modebuf, ":%s!%s@%s MODE %s ",
-				  source_p->name, source_p->username, 
+		mlen = rb_sprintf(modebuf, ":%s!%s@%s MODE %s ",
+				  source_p->name, source_p->username,
 				  source_p->host, chptr->chname);
 
 	for(j = 0, flags = ALL_MEMBERS; j < 2; j++, flags = ONLY_CHANOPS)
@@ -1406,17 +1429,16 @@ set_channel_mode(struct Client *client_p, struct Client *source_p,
 		paracount = paralen = 0;
 		dir = MODE_QUERY;
 
-		for (i = 0; i < mode_count; i++)
+		for(i = 0; i < mode_count; i++)
 		{
-			if(mode_changes[i].letter == 0 ||
-			   mode_changes[i].mems != flags)
+			if(mode_changes[i].letter == 0 || mode_changes[i].mems != flags)
 				continue;
 
 			if(mode_changes[i].arg != NULL)
 			{
 				arglen = strlen(mode_changes[i].arg);
 
-				if(arglen > MODEBUFLEN-5)
+				if(arglen > MODEBUFLEN - 5)
 					continue;
 			}
 			else
@@ -1427,12 +1449,13 @@ set_channel_mode(struct Client *client_p, struct Client *source_p,
 			 */
 			if(mode_changes[i].arg != NULL &&
 			   ((paracount == MAXMODEPARAMSSERV) ||
-			    ((cur_len + paralen + arglen + 4) > (BUFSIZE-3))))
+			    ((cur_len + paralen + arglen + 4) > (BUFSIZE - 3))))
 			{
 				*mbuf = '\0';
 
 				if(cur_len > mlen)
-					sendto_channel_local(flags, chptr, "%s %s", modebuf, parabuf);
+					sendto_channel_local(flags, chptr, "%s %s", modebuf,
+							     parabuf);
 				else
 					continue;
 
@@ -1453,11 +1476,11 @@ set_channel_mode(struct Client *client_p, struct Client *source_p,
 
 			*mbuf++ = mode_changes[i].letter;
 			cur_len++;
-			
+
 			if(mode_changes[i].arg != NULL)
 			{
 				paracount++;
-				len = ircsprintf(pbuf, "%s ", mode_changes[i].arg);
+				len = rb_sprintf(pbuf, "%s ", mode_changes[i].arg);
 				pbuf += len;
 				paralen += len;
 			}
@@ -1472,7 +1495,6 @@ set_channel_mode(struct Client *client_p, struct Client *source_p,
 	}
 
 	/* only propagate modes originating locally, or if we're hubbing */
-	if(MyClient(source_p) || dlink_list_length(&serv_list) > 1)
+	if(MyClient(source_p) || rb_dlink_list_length(&serv_list) > 1)
 		send_cap_mode_changes(client_p, source_p, chptr, mode_changes, mode_count);
 }
-
