@@ -18,40 +18,42 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
  *  USA
  *
- *  $Id: hash.c 22407 2006-05-07 15:39:43Z androsyn $
+ *  $Id: hash.c 26094 2008-09-19 15:33:46Z androsyn $
  */
 
 #include "stdinc.h"
-#include "ircd_defs.h"
-#include "tools.h"
+#include "struct.h"
 #include "s_conf.h"
 #include "channel.h"
 #include "client.h"
-#include "common.h"
 #include "hash.h"
-#include "irc_string.h"
+#include "match.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "send.h"
-#include "memory.h"
-#include "msg.h"
 #include "cache.h"
 #include "s_newconf.h"
+#include "s_log.h"
 
-static dlink_list *clientTable;
-static dlink_list *channelTable;
-static dlink_list *idTable;
-dlink_list *resvTable;
-static dlink_list *hostTable;
-static dlink_list *helpTable;
-dlink_list *ndTable;
+#define hash_nick(x) (fnv_hash_upper((const unsigned char *)(x), U_MAX_BITS, 0))
+#define hash_id(x) (fnv_hash((const unsigned char *)(x), U_MAX_BITS, 0))
+#define hash_channel(x) (fnv_hash_upper_len((const unsigned char *)(x), CH_MAX_BITS, 30))
+#define hash_hostname(x) (fnv_hash_upper_len((const unsigned char *)(x), HOST_MAX_BITS, 30))
+#define hash_resv(x) (fnv_hash_upper_len((const unsigned char *)(x), R_MAX_BITS, 30))
+#define hash_cli_fd(x)	(x % CLI_FD_MAX)
 
-/*
- * look in whowas.c for the missing ...[WW_MAX]; entry
- */
+
+static rb_dlink_list clientbyfdTable[U_MAX];
+static rb_dlink_list clientTable[U_MAX];
+static rb_dlink_list channelTable[CH_MAX];
+static rb_dlink_list idTable[U_MAX];
+rb_dlink_list resvTable[R_MAX];
+static rb_dlink_list hostTable[HOST_MAX];
+static rb_dlink_list helpTable[HELP_MAX];
+rb_dlink_list ndTable[U_MAX];
 
 /*
  * Hashing.
@@ -93,123 +95,68 @@ dlink_list *ndTable;
 void
 init_hash(void)
 {
-	clientTable = MyMalloc(sizeof(dlink_list) * U_MAX);
-	idTable = MyMalloc(sizeof(dlink_list) * U_MAX);
-	ndTable = MyMalloc(sizeof(dlink_list) * U_MAX);
-	channelTable = MyMalloc(sizeof(dlink_list) * CH_MAX);
-	hostTable = MyMalloc(sizeof(dlink_list) * HOST_MAX);
-	resvTable = MyMalloc(sizeof(dlink_list) * R_MAX);
-	helpTable = MyMalloc(sizeof(dlink_list) * HELP_MAX);
+	/* nothing to do here */
 }
 
 
 uint32_t
-fnv_hash_upper(const unsigned char *s, int bits)
+fnv_hash_upper(const unsigned char *s, unsigned int bits, unsigned int unused)
 {
- 	uint32_t h = FNV1_32_INIT;
-
-	while (*s)
+	uint32_t h = FNV1_32_INIT;
+	bits = 32 - bits;
+	while(*s)
 	{
-         	h ^= ToUpper(*s++);
-		h += (h<<1) + (h<<4) + (h<<7) + (h << 8) + (h << 24);
+		h ^= ToUpper(*s++);
+		h += (h << 1) + (h << 4) + (h << 7) + (h << 8) + (h << 24);
 	}
-        h = (h >> bits) ^ (h & ((2^bits)-1));
+	h = (h >> bits) ^ (h & ((2 ^ bits) - 1));
 	return h;
 }
 
 uint32_t
-fnv_hash(const unsigned char *s, int bits)
+fnv_hash(const unsigned char *s, unsigned int bits, unsigned int unused)
 {
- 	uint32_t h = FNV1_32_INIT;
-
-	while (*s)
+	uint32_t h = FNV1_32_INIT;
+	bits = 32 - bits;
+	while(*s)
 	{
 		h ^= *s++;
-		h += (h<<1) + (h<<4) + (h<<7) + (h << 8) + (h << 24);
+		h += (h << 1) + (h << 4) + (h << 7) + (h << 8) + (h << 24);
 	}
-        h = (h >> bits) ^ (h & ((2^bits)-1));
+	h = (h >> bits) ^ (h & ((2 ^ bits) - 1));
 	return h;
 }
 
 uint32_t
-fnv_hash_len(const unsigned char *s, int bits, int len)
+fnv_hash_len(const unsigned char *s, unsigned int bits, unsigned int len)
 {
- 	uint32_t h = FNV1_32_INIT;
+	uint32_t h = FNV1_32_INIT;
+	bits = 32 - bits;
 	const unsigned char *x = s + len;
-	while (*s && s < x)
+	while(*s && s < x)
 	{
 		h ^= *s++;
-		h += (h<<1) + (h<<4) + (h<<7) + (h << 8) + (h << 24);
+		h += (h << 1) + (h << 4) + (h << 7) + (h << 8) + (h << 24);
 	}
-        h = (h >> bits) ^ (h & ((2^bits)-1));
+	h = (h >> bits) ^ (h & ((2 ^ bits) - 1));
 	return h;
 }
 
 uint32_t
-fnv_hash_upper_len(const unsigned char *s, int bits, int len)
+fnv_hash_upper_len(const unsigned char *s, unsigned int bits, unsigned int len)
 {
- 	uint32_t h = FNV1_32_INIT;
+	uint32_t h = FNV1_32_INIT;
+	bits = 32 - bits;
 	const unsigned char *x = s + len;
-	while (*s && s < x)
+	while(*s && s < x)
 	{
-         	h ^= ToUpper(*s++);
-		h += (h<<1) + (h<<4) + (h<<7) + (h << 8) + (h << 24);
+		h ^= ToUpper(*s++);
+		h += (h << 1) + (h << 4) + (h << 7) + (h << 8) + (h << 24);
 	}
-        h = (h >> bits) ^ (h & ((2^bits)-1));
+	h = (h >> bits) ^ (h & ((2 ^ bits) - 1));
 	return h;
 }
 
-/* hash_nick()
- *
- * hashes a nickname, first converting to lowercase
- */
-static uint32_t
-hash_nick(const char *name)
-{
-	return fnv_hash_upper((const unsigned char *) name, U_MAX_BITS);
-}
-
-/* hash_id()
- *
- * hashes an id, case is kept
- */
-static uint32_t
-hash_id(const char *name)
-{
-	return fnv_hash((const unsigned char *) name, U_MAX_BITS);
-}
-
-/* hash_channel()
- *
- * hashes a channel name, based on first 30 chars only for efficiency
- */
-static uint32_t
-hash_channel(const char *name)
-{
-	return fnv_hash_upper_len((const unsigned char *) name, CH_MAX_BITS, 30);
-}
-
-/* hash_hostname()
- *
- * hashes a hostname, based on first 30 chars only, as thats likely to
- * be more dynamic than rest.
- */
-static uint32_t
-hash_hostname(const char *name)
-{
-	return fnv_hash_upper_len((const unsigned char *) name, HOST_MAX_BITS, 30);
-}
-
-/* hash_resv()
- *
- * hashes a resv channel name, based on first 30 chars only
- */
-static uint32_t
-hash_resv(const char *name)
-{
-	return fnv_hash_upper_len((const unsigned char *) name, R_MAX_BITS, 30);
-}
-
 static unsigned int
 hash_help(const char *name)
 {
@@ -217,80 +164,61 @@ hash_help(const char *name)
 
 	while(*name)
 	{
-		h += (unsigned int) (ToLower(*name++) & 0xDF);
+		h += (unsigned int)(ToLower(*name++) & 0xDF);
 	}
 
 	return (h % HELP_MAX);
 }
 
-/* add_to_id_hash()
- *
- * adds an entry to the id hash table
- */
-void
-add_to_id_hash(const char *name, struct Client *client_p)
+static struct _hash_function
 {
+	uint32_t (*func) (unsigned const char *, unsigned int, unsigned int);
+	rb_dlink_list *table;
+	unsigned int hashbits;
+	unsigned int hashlen;
+} hash_function[] =
+{
+	{
+	fnv_hash_upper, clientTable, U_MAX_BITS, 0},
+	{
+	fnv_hash, idTable, U_MAX_BITS, 0},
+	{
+	fnv_hash_upper_len, channelTable, CH_MAX_BITS, 30},
+	{
+	fnv_hash_upper_len, hostTable, HOST_MAX_BITS, 30},
+	{
+	fnv_hash_upper_len, resvTable, R_MAX_BITS, 30}
+};
+
+void
+add_to_hash(hash_type type, const char *hashindex, void *pointer)
+{
+	rb_dlink_list *table = hash_function[type].table;
 	unsigned int hashv;
 
-	if(EmptyString(name) || (client_p == NULL))
+	if(EmptyString(hashindex) || (pointer == NULL))
 		return;
 
-	hashv = hash_id(name);
-	dlinkAddAlloc(client_p, &idTable[hashv]);
+	hashv = (hash_function[type].func) ((const unsigned char *)hashindex,
+					    hash_function[type].hashbits,
+					    hash_function[type].hashlen);
+//      rb_dlinkAddAlloc(pointer, &hash_function[type].table[hashv]);
+	rb_dlinkAddAlloc(pointer, &table[hashv]);
 }
 
-/* add_to_client_hash()
- *
- * adds an entry (client/server) to the client hash table
- */
 void
-add_to_client_hash(const char *name, struct Client *client_p)
+del_from_hash(hash_type type, const char *hashindex, void *pointer)
 {
+	rb_dlink_list *table = hash_function[type].table;
 	unsigned int hashv;
 
-	s_assert(name != NULL);
-	s_assert(client_p != NULL);
-	if(EmptyString(name) || (client_p == NULL))
+	if(EmptyString(hashindex) || (pointer == NULL))
 		return;
 
-	hashv = hash_nick(name);
-	dlinkAddAlloc(client_p, &clientTable[hashv]);
-}
-
-/* add_to_hostname_hash()
- *
- * adds a client entry to the hostname hash table
- */
-void
-add_to_hostname_hash(const char *hostname, struct Client *client_p)
-{
-	unsigned int hashv;
-
-	s_assert(hostname != NULL);
-	s_assert(client_p != NULL);
-	if(EmptyString(hostname) || (client_p == NULL))
-		return;
-
-	hashv = hash_hostname(hostname);
-	dlinkAddAlloc(client_p, &hostTable[hashv]);
-}
-
-/* add_to_resv_hash()
- *
- * adds a resv channel entry to the resv hash table
- */
-void
-add_to_resv_hash(const char *name, struct ConfItem *aconf)
-{
-	unsigned int hashv;
-
-	s_assert(!EmptyString(name));
-	s_assert(aconf != NULL);
-	if(EmptyString(name) || aconf == NULL)
-		return;
-
-	hashv = hash_resv(name);
-	dlinkAddAlloc(aconf, &resvTable[hashv]);
+	hashv = (hash_function[type].func) ((const unsigned char *)hashindex,
+					    hash_function[type].hashbits,
+					    hash_function[type].hashlen);
+	rb_dlinkFindDestroy(pointer, &table[hashv]);
 }
 
 void
@@ -302,122 +230,32 @@ add_to_help_hash(const char *name, struct cachefile *hptr)
 		return;
 
 	hashv = hash_help(name);
-	dlinkAddAlloc(hptr, &helpTable[hashv]);
+	rb_dlinkAddAlloc(hptr, &helpTable[hashv]);
 }
 
 void
 add_to_nd_hash(const char *name, struct nd_entry *nd)
 {
 	nd->hashv = hash_nick(name);
-	dlinkAdd(nd, &nd->hnode, &ndTable[nd->hashv]);
-}
-
-/* del_from_id_hash()
- *
- * removes an id from the id hash table
- */
-void
-del_from_id_hash(const char *id, struct Client *client_p)
-{
-	unsigned int hashv;
-
-	s_assert(id != NULL);
-	s_assert(client_p != NULL);
-	if(EmptyString(id) || client_p == NULL)
-		return;
-
-	hashv = hash_id(id);
-	dlinkFindDestroy(client_p, &idTable[hashv]);
-}
-
-/* del_from_client_hash()
- *
- * removes a client/server from the client hash table
- */
-void
-del_from_client_hash(const char *name, struct Client *client_p)
-{
-	unsigned int hashv;
-
-	/* no s_asserts, this can happen when removing a client that
-	 * is unregistered.
-	 */
-	if(EmptyString(name) || client_p == NULL)
-		return;
-
-	hashv = hash_nick(name);
-	dlinkFindDestroy(client_p, &clientTable[hashv]);
-}
-
-/* del_from_channel_hash()
- * 
- * removes a channel from the channel hash table
- */
-void
-del_from_channel_hash(const char *name, struct Channel *chptr)
-{
-	unsigned int hashv;
-
-	s_assert(name != NULL);
-	s_assert(chptr != NULL);
-
-	if(EmptyString(name) || chptr == NULL)
-		return;
-
-	hashv = hash_channel(name);
-	dlinkFindDestroy(chptr, &channelTable[hashv]);
-}
-
-/* del_from_hostname_hash()
- *
- * removes a client entry from the hostname hash table
- */
-void
-del_from_hostname_hash(const char *hostname, struct Client *client_p)
-{
-	unsigned int hashv;
-
-	if(hostname == NULL || client_p == NULL)
-		return;
-
-	hashv = hash_hostname(hostname);
-
-	dlinkFindDestroy(client_p, &hostTable[hashv]);
-}
-
-/* del_from_resv_hash()
- *
- * removes a resv entry from the resv hash table
- */
-void
-del_from_resv_hash(const char *name, struct ConfItem *aconf)
-{
-	unsigned int hashv;
-
-	s_assert(name != NULL);
-	s_assert(aconf != NULL);
-	if(EmptyString(name) || aconf == NULL)
-		return;
-
-	hashv = hash_resv(name);
-
-	dlinkFindDestroy(aconf, &resvTable[hashv]);
+	rb_dlinkAdd(nd, &nd->hnode, &ndTable[nd->hashv]);
 }
 
 void
 clear_help_hash(void)
 {
-	dlink_node *ptr;
-	dlink_node *next_ptr;
+	rb_dlink_node *ptr;
+	rb_dlink_node *next_ptr;
 	int i;
 
 	HASH_WALK_SAFE(i, HELP_MAX, ptr, next_ptr, helpTable)
 	{
 		free_cachefile(ptr->data);
-		dlinkDestroy(ptr, &helpTable[i]);
+		rb_dlinkDestroy(ptr, &helpTable[i]);
 	}
-	HASH_WALK_END
-}
+HASH_WALK_END}
+
+
+
 
 /* find_id()
  *
@@ -427,7 +265,7 @@ struct Client *
 find_id(const char *name)
 {
 	struct Client *target_p;
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 	unsigned int hashv;
 
 	if(EmptyString(name))
@@ -435,7 +273,7 @@ find_id(const char *name)
 
 	hashv = hash_id(name);
 
-	DLINK_FOREACH(ptr, idTable[hashv].head)
+	RB_DLINK_FOREACH(ptr, idTable[hashv].head)
 	{
 		target_p = ptr->data;
 
@@ -469,9 +307,9 @@ hash_find_masked_server(struct Client *source_p, const char *name)
 		return NULL;
 
 	/* copy it across to give us a buffer to work on */
-	strlcpy(buf, name, sizeof(buf));
+	rb_strlcpy(buf, name, sizeof(buf));
 
-	while ((s = strchr(p, '.')) != 0)
+	while((s = strchr(p, '.')) != 0)
 	{
 		*--s = '*';
 		/*
@@ -494,7 +332,7 @@ struct Client *
 find_any_client(const char *name)
 {
 	struct Client *target_p;
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 	unsigned int hashv;
 
 	s_assert(name != NULL);
@@ -507,7 +345,7 @@ find_any_client(const char *name)
 
 	hashv = hash_nick(name);
 
-	DLINK_FOREACH(ptr, clientTable[hashv].head)
+	RB_DLINK_FOREACH(ptr, clientTable[hashv].head)
 	{
 		target_p = ptr->data;
 
@@ -527,7 +365,7 @@ struct Client *
 find_client(const char *name)
 {
 	struct Client *target_p;
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 	unsigned int hashv;
 
 	s_assert(name != NULL);
@@ -540,7 +378,7 @@ find_client(const char *name)
 
 	hashv = hash_nick(name);
 
-	DLINK_FOREACH(ptr, clientTable[hashv].head)
+	RB_DLINK_FOREACH(ptr, clientTable[hashv].head)
 	{
 		target_p = ptr->data;
 
@@ -559,7 +397,7 @@ struct Client *
 find_named_client(const char *name)
 {
 	struct Client *target_p;
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 	unsigned int hashv;
 
 	s_assert(name != NULL);
@@ -568,7 +406,7 @@ find_named_client(const char *name)
 
 	hashv = hash_nick(name);
 
-	DLINK_FOREACH(ptr, clientTable[hashv].head)
+	RB_DLINK_FOREACH(ptr, clientTable[hashv].head)
 	{
 		target_p = ptr->data;
 
@@ -587,28 +425,26 @@ struct Client *
 find_server(struct Client *source_p, const char *name)
 {
 	struct Client *target_p;
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 	unsigned int hashv;
-  
+
 	if(EmptyString(name))
 		return NULL;
 
-	if((source_p == NULL || !MyClient(source_p)) && 
-	   IsDigit(*name) && strlen(name) == 3)
+	if((source_p == NULL || !MyClient(source_p)) && IsDigit(*name) && strlen(name) == 3)
 	{
 		target_p = find_id(name);
-      		return(target_p);
+		return (target_p);
 	}
 
 	hashv = hash_nick(name);
 
-	DLINK_FOREACH(ptr, clientTable[hashv].head)
+	RB_DLINK_FOREACH(ptr, clientTable[hashv].head)
 	{
 		target_p = ptr->data;
 
-		if((IsServer(target_p) || IsMe(target_p)) &&
-		   irccmp(name, target_p->name) == 0)
-				return target_p;
+		if((IsServer(target_p) || IsMe(target_p)) && irccmp(name, target_p->name) == 0)
+			return target_p;
 	}
 
 	/* wasnt found, look for a masked server */
@@ -621,7 +457,7 @@ find_server(struct Client *source_p, const char *name)
  * we return the full dlink list, because you can have multiple
  * entries with the same hostname
  */
-dlink_node *
+rb_dlink_node *
 find_hostname(const char *hostname)
 {
 	unsigned int hashv;
@@ -642,7 +478,7 @@ struct Channel *
 find_channel(const char *name)
 {
 	struct Channel *chptr;
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 	unsigned int hashv;
 
 	s_assert(name != NULL);
@@ -651,7 +487,7 @@ find_channel(const char *name)
 
 	hashv = hash_channel(name);
 
-	DLINK_FOREACH(ptr, channelTable[hashv].head)
+	RB_DLINK_FOREACH(ptr, channelTable[hashv].head)
 	{
 		chptr = ptr->data;
 
@@ -677,7 +513,7 @@ struct Channel *
 get_or_create_channel(struct Client *client_p, const char *chname, int *isnew)
 {
 	struct Channel *chptr;
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 	unsigned int hashv;
 	int len;
 	const char *s = chname;
@@ -688,7 +524,6 @@ get_or_create_channel(struct Client *client_p, const char *chname, int *isnew)
 	len = strlen(s);
 	if(len > CHANNELLEN)
 	{
-		char *t;
 		if(IsServer(client_p))
 		{
 			sendto_realops_flags(UMODE_DEBUG, L_ALL,
@@ -696,14 +531,12 @@ get_or_create_channel(struct Client *client_p, const char *chname, int *isnew)
 					     client_p->name, len, CHANNELLEN, s);
 		}
 		len = CHANNELLEN;
-		t = LOCAL_COPY(s);
-		*(t + CHANNELLEN) = '\0';
-		s = t;
+		s = LOCAL_COPY_N(s, CHANNELLEN);
 	}
 
 	hashv = hash_channel(s);
 
-	DLINK_FOREACH(ptr, channelTable[hashv].head)
+	RB_DLINK_FOREACH(ptr, channelTable[hashv].head)
 	{
 		chptr = ptr->data;
 
@@ -720,11 +553,11 @@ get_or_create_channel(struct Client *client_p, const char *chname, int *isnew)
 
 	chptr = allocate_channel(s);
 
-	dlinkAdd(chptr, &chptr->node, &global_channel_list);
+	rb_dlinkAdd(chptr, &chptr->node, &global_channel_list);
 
-	chptr->channelts = CurrentTime;	/* doesn't hurt to set it here */
+	chptr->channelts = rb_current_time();	/* doesn't hurt to set it here */
 
-	dlinkAddAlloc(chptr, &channelTable[hashv]);
+	rb_dlinkAddAlloc(chptr, &channelTable[hashv]);
 
 	return chptr;
 }
@@ -737,7 +570,7 @@ struct ConfItem *
 hash_find_resv(const char *name)
 {
 	struct ConfItem *aconf;
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 	unsigned int hashv;
 
 	s_assert(name != NULL);
@@ -746,11 +579,11 @@ hash_find_resv(const char *name)
 
 	hashv = hash_resv(name);
 
-	DLINK_FOREACH(ptr, resvTable[hashv].head)
+	RB_DLINK_FOREACH(ptr, resvTable[hashv].head)
 	{
 		aconf = ptr->data;
 
-		if(!irccmp(name, aconf->name))
+		if(!irccmp(name, aconf->host))
 		{
 			aconf->port++;
 			return aconf;
@@ -764,7 +597,7 @@ struct cachefile *
 hash_find_help(const char *name, int flags)
 {
 	struct cachefile *hptr;
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 	unsigned int hashv;
 
 	if(EmptyString(name))
@@ -772,12 +605,11 @@ hash_find_help(const char *name, int flags)
 
 	hashv = hash_help(name);
 
-	DLINK_FOREACH(ptr, helpTable[hashv].head)
+	RB_DLINK_FOREACH(ptr, helpTable[hashv].head)
 	{
 		hptr = ptr->data;
 
-		if((irccmp(name, hptr->name) == 0) &&
-		   (hptr->flags & flags))
+		if((irccmp(name, hptr->name) == 0) && (hptr->flags & flags))
 			return hptr;
 	}
 
@@ -788,8 +620,8 @@ void
 clear_resv_hash(void)
 {
 	struct ConfItem *aconf;
-	dlink_node *ptr;
-	dlink_node *next_ptr;
+	rb_dlink_node *ptr;
+	rb_dlink_node *next_ptr;
 	int i;
 
 	HASH_WALK_SAFE(i, R_MAX, ptr, next_ptr, resvTable)
@@ -797,20 +629,19 @@ clear_resv_hash(void)
 		aconf = ptr->data;
 
 		/* skip temp resvs */
-		if(aconf->hold)
+		if(aconf->flags & CONF_FLAGS_TEMPORARY)
 			continue;
 
 		free_conf(ptr->data);
-		dlinkDestroy(ptr, &resvTable[i]);
+		rb_dlinkDestroy(ptr, &resvTable[i]);
 	}
-	HASH_WALK_END
-}
+HASH_WALK_END}
 
 struct nd_entry *
 hash_find_nd(const char *name)
 {
 	struct nd_entry *nd;
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 	unsigned int hashv;
 
 	if(EmptyString(name))
@@ -818,7 +649,7 @@ hash_find_nd(const char *name)
 
 	hashv = hash_nick(name);
 
-	DLINK_FOREACH(ptr, ndTable[hashv].head)
+	RB_DLINK_FOREACH(ptr, ndTable[hashv].head)
 	{
 		nd = ptr->data;
 
@@ -829,19 +660,56 @@ hash_find_nd(const char *name)
 	return NULL;
 }
 
+void
+add_to_cli_fd_hash(struct Client *client_p)
+{
+	rb_dlinkAddAlloc(client_p,
+			 &clientbyfdTable[hash_cli_fd(rb_get_fd(client_p->localClient->F))]);
+}
+
+
+void
+del_from_cli_fd_hash(struct Client *client_p)
+{
+	unsigned int hashv;
+	hashv = hash_cli_fd(rb_get_fd(client_p->localClient->F));
+	rb_dlinkFindDestroy(client_p, &clientbyfdTable[hashv]);
+}
+
+struct Client *
+find_cli_fd_hash(int fd)
+{
+	struct Client *target_p;
+	rb_dlink_node *ptr;
+	unsigned int hashv;
+	hashv = hash_cli_fd(fd);
+	RB_DLINK_FOREACH(ptr, clientbyfdTable[hashv].head)
+	{
+		target_p = ptr->data;
+		if(rb_get_fd(target_p->localClient->F) == fd)
+			return target_p;
+	}
+	return NULL;
+}
+
 static void
 output_hash(struct Client *source_p, const char *name, int length, int *counts, int deepest)
 {
 	unsigned long total = 0;
+	char buf[128];
 	int i;
 
-	sendto_one_numeric(source_p, RPL_STATSDEBUG,
-			"B :%s Hash Statistics", name);
+	sendto_one_numeric(source_p, RPL_STATSDEBUG, "B :%s Hash Statistics", name);
 
-	sendto_one_numeric(source_p, RPL_STATSDEBUG,
-			"B :Size: %d Empty: %d (%.3f%%)",
-			length, counts[0], 
-			(float) ((counts[0]*100) / (float) length));
+	/* rb_snprintf which sendto_one_* uses doesn't support float formats */
+#ifdef HAVE_SNPRINTF
+	snprintf(buf, sizeof(buf),
+#else
+	sprintf(buf,
+#endif
+		"%.3f%%", (float)((counts[0] * 100) / (float)length));
+	sendto_one_numeric(source_p, RPL_STATSDEBUG, "B :Size: %d Empty: %d (%s)",
+			   length, counts[0], buf);
 
 	for(i = 1; i < 11; i++)
 	{
@@ -850,38 +718,43 @@ output_hash(struct Client *source_p, const char *name, int length, int *counts,
 
 	/* dont want to divide by 0! --fl */
 	if(counts[0] != length)
+	{
+#ifdef HAVE_SNPRINTF
+		snprintf(buf, sizeof(buf),
+#else
+		sprintf(buf,
+#endif
+			"%.3f%%/%.3f%%", (float)(total / (length - counts[0])),
+			(float)(total / length));
 		sendto_one_numeric(source_p, RPL_STATSDEBUG,
-				"B :Average depth: %.3f/%.3f Highest depth: %d",
-				(float) (total / (length - counts[0])),
-				(float) (total / length), deepest);
-
+				   "B :Average depth: %s Highest depth: %d", buf, deepest);
+	}
 	for(i = 0; i < 11; i++)
 	{
 		sendto_one_numeric(source_p, RPL_STATSDEBUG,
-				"B :Nodes with %d entries: %d",
-				i, counts[i]);
+				   "B :Nodes with %d entries: %d", i, counts[i]);
 	}
 }
-	
+
 
 static void
-count_hash(struct Client *source_p, dlink_list *table, int length, const char *name)
+count_hash(struct Client *source_p, rb_dlink_list *table, int length, const char *name)
 {
 	int counts[11];
-	int deepest = 0;
+	unsigned long deepest = 0;
 	int i;
 
 	memset(counts, 0, sizeof(counts));
-	
+
 	for(i = 0; i < length; i++)
 	{
-		if(dlink_list_length(&table[i]) >= 10)
+		if(rb_dlink_list_length(&table[i]) >= 10)
 			counts[10]++;
 		else
-			counts[dlink_list_length(&table[i])]++;
+			counts[rb_dlink_list_length(&table[i])]++;
 
-		if(dlink_list_length(&table[i]) > deepest)
-			deepest = dlink_list_length(&table[i]);
+		if(rb_dlink_list_length(&table[i]) > deepest)
+			deepest = rb_dlink_list_length(&table[i]);
 	}
 
 	output_hash(source_p, name, length, counts, deepest);
@@ -897,4 +770,6 @@ hash_stats(struct Client *source_p)
 	count_hash(source_p, idTable, U_MAX, "ID");
 	sendto_one_numeric(source_p, RPL_STATSDEBUG, "B :--");
 	count_hash(source_p, hostTable, HOST_MAX, "Hostname");
-}	
+	sendto_one_numeric(source_p, RPL_STATSDEBUG, "B :--");
+	count_hash(source_p, clientbyfdTable, CLI_FD_MAX, "Client by FD");
+}
