@@ -18,36 +18,29 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
  *  USA
  *
- *  $Id: ircd.c 21852 2006-02-07 19:29:02Z leeh $
+ *  $Id: ircd.c 26304 2008-12-16 03:08:43Z androsyn $
  */
 
-#include "stdinc.h"
+#include "ratbox_lib.h"
 #include "setup.h"
 #include "config.h"
-
-#include "tools.h"
+#include "stdinc.h"
+#include "struct.h"
 #include "ircd.h"
 #include "channel.h"
 #include "class.h"
 #include "client.h"
-#include "common.h"
-#include "event.h"
 #include "hash.h"
-#include "irc_string.h"
+#include "match.h"
 #include "ircd_signal.h"
-#include "sprintf_irc.h"
-#include "s_gline.h"
-#include "msg.h"		/* msgtab */
 #include "hostmask.h"
 #include "numeric.h"
 #include "parse.h"
-#include "res.h"
 #include "restart.h"
 #include "s_auth.h"
-#include "commio.h"
 #include "s_conf.h"
 #include "s_log.h"
 #include "s_serv.h"		/* try_connections */
@@ -55,19 +48,19 @@
 #include "scache.h"
 #include "send.h"
 #include "whowas.h"
-#include "modules.h"
-#include "memory.h"
 #include "hook.h"
+#include "modules.h"
 #include "ircd_getopt.h"
-#include "balloc.h"
 #include "newconf.h"
-#include "patricia.h"
 #include "reject.h"
 #include "s_conf.h"
 #include "s_newconf.h"
 #include "cache.h"
 #include "monitor.h"
-
+#include "dns.h"
+#include "bandbi.h"
+#include "sslproc.h"
+#include "supported.h"
 /*
  * Try and find the correct name to use with getrlimit() for setting the max.
  * number of files allowed to be open by this process.
@@ -84,20 +77,20 @@ struct server_info ServerInfo;
 struct admin_info AdminInfo;
 
 struct Counter Count;
+struct ServerStatistics ServerStats;
 
-struct timeval SystemTime;
-int ServerRunning;		/* GLOBAL - server execution state */
+int maxconnections;
 struct Client me;		/* That's me */
 struct LocalUser meLocalUser;	/* That's also part of me */
 
-dlink_list global_client_list;
+rb_dlink_list global_client_list;
 
 /* unknown/client pointer lists */
-dlink_list unknown_list;	/* unknown clients ON this server only */
-dlink_list lclient_list;	/* local clients only ON this server */
-dlink_list serv_list;		/* local servers to this server ONLY */
-dlink_list global_serv_list;	/* global servers on the network */
-dlink_list oper_list;		/* our opers, duplicated in lclient_list */
+rb_dlink_list unknown_list;	/* unknown clients ON this server only */
+rb_dlink_list lclient_list;	/* local clients only ON this server */
+rb_dlink_list serv_list;	/* local servers to this server ONLY */
+rb_dlink_list global_serv_list;	/* global servers on the network */
+rb_dlink_list oper_list;	/* our opers, duplicated in lclient_list */
 
 static unsigned long initialVMTop = 0;	/* top of virtual memory at init */
 const char *logFileName = LPATH;
@@ -109,10 +102,12 @@ int dorehashbans = 0;
 int doremotd = 0;
 int kline_queued = 0;
 int server_state_foreground = 0;
+int printVersion = 0;
+int ircd_ssl_ok = 0;
+int zlib_ok = 1;
 
 int testing_conf = 0;
 int conf_parse_failure = 0;
-
 time_t startup_time;
 
 /* Set to zero because it should be initialized later using
@@ -126,29 +121,25 @@ int split_users;
 int split_servers;
 int eob_count;
 
-int maxconnections;
-
 void
 ircd_shutdown(const char *reason)
 {
 	struct Client *target_p;
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 
-	DLINK_FOREACH(ptr, lclient_list.head)
+	RB_DLINK_FOREACH(ptr, lclient_list.head)
 	{
 		target_p = ptr->data;
 
-		sendto_one(target_p,
-				":%s NOTICE %s :Server Terminating. %s",
-				me.name, target_p->name, reason);
+		sendto_one(target_p, ":%s NOTICE %s :Server Terminating. %s",
+			   me.name, target_p->name, reason);
 	}
 
-	DLINK_FOREACH(ptr, serv_list.head)
+	RB_DLINK_FOREACH(ptr, serv_list.head)
 	{
 		target_p = ptr->data;
 
-		sendto_one(target_p, ":%s ERROR :Server Terminating. %s",
-				me.name, reason);
+		sendto_one(target_p, ":%s ERROR :Terminated by %s", me.name, reason);
 	}
 
 	ilog(L_MAIN, "Server Terminating. %s", reason);
@@ -174,8 +165,12 @@ get_vm_top(void)
 	 * offset from 0 (NULL), so the result of sbrk is cast to a size_t and 
 	 * returned. We really shouldn't be using it here but...
 	 */
+#ifndef _WIN32
 	void *vptr = sbrk(0);
-	return (unsigned long) vptr;
+	return (unsigned long)vptr;
+#else
+	return -1;
+#endif
 }
 
 /*
@@ -194,14 +189,13 @@ static void
 print_startup(int pid)
 {
 	printf("ircd: version %s\n", ircd_version);
+	printf("ircd: %s\n", rb_lib_version());
 	printf("ircd: pid %d\n", pid);
 #ifndef RATBOX_PROFILE
 	printf("ircd: running in %s mode from %s\n",
-	       !server_state_foreground ? "background" : "foreground",
-	        ConfigFileEntry.dpath);
+	       !server_state_foreground ? "background" : "foreground", ConfigFileEntry.dpath);
 #else
-	printf("ircd: running in foreground mode from %s for profiling\n",
-		ConfigFileEntry.dpath);
+	printf("ircd: running in foreground mode from %s for profiling\n", ConfigFileEntry.dpath);
 #endif
 }
 
@@ -216,21 +210,21 @@ static void
 init_sys(void)
 {
 #if defined(RLIMIT_NOFILE) && defined(HAVE_SYS_RESOURCE_H)
-        struct rlimit limit;
+	struct rlimit limit;
 
-        if(!getrlimit(RLIMIT_NOFILE, &limit))
-        {
+	if(!getrlimit(RLIMIT_NOFILE, &limit))
+	{
 		maxconnections = limit.rlim_cur;
-
 		if(maxconnections <= MAX_BUFFER)
 		{
 			fprintf(stderr, "ERROR: Shell FD limits are too low.\n");
-			fprintf(stderr, "ERROR: ircd-ratbox reserves %d FDs, shell limits must be above this\n", MAX_BUFFER);
+			fprintf(stderr,
+				"ERROR: ircd-ratbox reserves %d FDs, shell limits must be above this\n",
+				MAX_BUFFER);
 			exit(EXIT_FAILURE);
 		}
-
-                return;
-        }
+		return;
+	}
 #endif /* RLIMIT_FD_MAX */
 	maxconnections = MAXCONNECTIONS;
 }
@@ -238,7 +232,8 @@ init_sys(void)
 static int
 make_daemon(void)
 {
-	int pid;
+#ifndef _WIN32
+	int pid, fd;
 
 	if((pid = fork()) < 0)
 	{
@@ -252,30 +247,29 @@ make_daemon(void)
 	}
 
 	setsid();
-	/*  fclose(stdin);
-	   fclose(stdout);
-	   fclose(stderr); */
 
+	fd = open("/dev/null", O_RDWR);
+	/* dup2 should close the target fd if its open */
+	dup2(fd, 0);
+	dup2(fd, 1);
+	dup2(fd, 2);
+	close(fd);
+#endif
 	return 0;
 }
 
-static int printVersion = 0;
+static const char *basedir = DPATH;
+static const char *configfile = CPATH;
 
 struct lgetopt myopts[] = {
-	{"dlinefile", &ConfigFileEntry.dlinefile,
-	 STRING, "File to use for dlines.conf"},
-	{"configfile", &ConfigFileEntry.configfile,
-	 STRING, "File to use for ircd.conf"},
-	{"klinefile", &ConfigFileEntry.klinefile,
-	 STRING, "File to use for klines.conf"},
-	{"xlinefile", &ConfigFileEntry.xlinefile,
-	 STRING, "File to use for xlines.conf"},
-	{"resvfile", &ConfigFileEntry.resvfile,
-	 STRING, "File to use for resv.conf"},
+	{"basedir", &basedir,
+	 ISTRING, "Base directory to run ircd from"},
+	{"configfile", &configfile,
+	 ISTRING, "File to use for ircd.conf"},
 	{"logfile", &logFileName,
-	 STRING, "File to use for ircd.log"},
+	 ISTRING, "File to use for ircd.log"},
 	{"pidfile", &pidFileName,
-	 STRING, "File to use for process ID"},
+	 ISTRING, "File to use for process ID"},
 	{"foreground", &server_state_foreground,
 	 YESNO, "Run in foreground (don't detach)"},
 	{"version", &printVersion,
@@ -283,72 +277,33 @@ struct lgetopt myopts[] = {
 	{"conftest", &testing_conf,
 	 YESNO, "Test the configuration files and exit"},
 	{"help", NULL, USAGE, "Print this text"},
-	{NULL, NULL, STRING, NULL},
+	{NULL, NULL, ISTRING, NULL},
 };
 
-void
-set_time(void)
-{
-	struct timeval newtime;
-	newtime.tv_sec = 0;
-	newtime.tv_usec = 0;
-#ifdef HAVE_GETTIMEOFDAY
-	if(gettimeofday(&newtime, NULL) == -1)
-	{
-		ilog(L_MAIN, "Clock Failure (%d)", errno);
-		restart("Clock Failure");
-	}
-#else
-	newtime.tv_sec = time(NULL);
-	
-#endif
-	if(newtime.tv_sec < CurrentTime)
-		set_back_events(CurrentTime - newtime.tv_sec);
-
-	SystemTime.tv_sec = newtime.tv_sec;
-	SystemTime.tv_usec = newtime.tv_usec;
-}
-
 static void
-io_loop(void)
+check_rehash(void *unusued)
 {
-	time_t delay;
-
-	while (ServerRunning)
-
+	/*
+	 * Check to see whether we have to rehash the configuration ..
+	 */
+	if(dorehash)
 	{
-		/* Run pending events, then get the number of seconds to the next
-		 * event
-		 */
+		rehash(1);
+		dorehash = 0;
+	}
 
-		delay = eventNextTime();
-		if(delay <= CurrentTime)
-			eventRun();
+	if(dorehashbans)
+	{
+		rehash_bans(1);
+		dorehashbans = 0;
+	}
 
-
-		comm_select(250);
-
-		/*
-		 * Check to see whether we have to rehash the configuration ..
-		 */
-		if(dorehash)
-		{
-			rehash(1);
-			dorehash = 0;
-		}
-		if(dorehashbans)
-		{
-			rehash_bans(1);
-			dorehashbans = 0;
-		}
-		if(doremotd)
-		{
-			sendto_realops_flags(UMODE_ALL, L_ALL,
-					     "Got signal SIGUSR1, reloading ircd motd file");
-			free_cachefile(user_motd);
-			user_motd = cache_file(MPATH, "ircd.motd", 0);
-			doremotd = 0;
-		}
+	if(doremotd)
+	{
+		sendto_realops_flags(UMODE_ALL, L_ALL,
+				     "Got signal SIGUSR1, reloading ircd motd file");
+		cache_user_motd();
+		doremotd = 0;
 	}
 }
 
@@ -365,12 +320,14 @@ initialize_global_set_options(void)
 	memset(&GlobalSetOptions, 0, sizeof(GlobalSetOptions));
 	/* memset( &ConfigFileEntry, 0, sizeof(ConfigFileEntry)); */
 
-	GlobalSetOptions.maxclients = ServerInfo.default_max_clients - MAX_BUFFER;
-	GlobalSetOptions.autoconn = 1;
+	GlobalSetOptions.maxclients = ServerInfo.default_max_clients;
 
-	if(GlobalSetOptions.maxclients > (maxconnections - MAX_BUFFER))
+	if(GlobalSetOptions.maxclients > (maxconnections - MAX_BUFFER)
+	   || (GlobalSetOptions.maxclients <= 0))
 		GlobalSetOptions.maxclients = maxconnections - MAX_BUFFER;
 
+	GlobalSetOptions.autoconn = 1;
+
 	GlobalSetOptions.spam_time = MIN_JOIN_LEAVE_TIME;
 	GlobalSetOptions.spam_num = MAX_JOIN_LEAVE_COUNT;
 
@@ -391,12 +348,10 @@ initialize_global_set_options(void)
 
 	GlobalSetOptions.ident_timeout = IDENT_TIMEOUT;
 
-	strlcpy(GlobalSetOptions.operstring,
-		ConfigFileEntry.default_operstring,
-		sizeof(GlobalSetOptions.operstring));
-	strlcpy(GlobalSetOptions.adminstring,
-		ConfigFileEntry.default_adminstring,
-		sizeof(GlobalSetOptions.adminstring));
+	rb_strlcpy(GlobalSetOptions.operstring,
+		   ConfigFileEntry.default_operstring, sizeof(GlobalSetOptions.operstring));
+	rb_strlcpy(GlobalSetOptions.adminstring,
+		   ConfigFileEntry.default_adminstring, sizeof(GlobalSetOptions.adminstring));
 
 	/* memset( &ConfigChannel, 0, sizeof(ConfigChannel)); */
 
@@ -431,9 +386,9 @@ write_pidfile(const char *filename)
 	char buff[32];
 	if((fb = fopen(filename, "w")))
 	{
-		unsigned int pid = (unsigned int) getpid();
+		unsigned int pid = (unsigned int)getpid();
 
-		ircsnprintf(buff, sizeof(buff), "%u\n", pid);
+		rb_snprintf(buff, sizeof(buff), "%u\n", pid);
 		if((fputs(buff, fb) == -1))
 		{
 			ilog(L_MAIN, "Error writing %u to pid file %s (%s)",
@@ -470,7 +425,7 @@ check_pidfile(const char *filename)
 		if(fgets(buff, 20, fb) != NULL)
 		{
 			pidfromfile = atoi(buff);
-			if(!kill(pidfromfile, 0))
+			if(!rb_kill(pidfromfile, 0))
 			{
 				printf("ircd: daemon is already running\n");
 				exit(-1);
@@ -503,104 +458,138 @@ setup_corefile(void)
 #endif
 }
 
-int
-main(int argc, char *argv[])
+static void
+ilogcb(const char *buf)
 {
-	static const char ircd_default_name[] = "ircd.ratbox";
+	ilog(L_MAIN, "libratbox reports: %s", buf);
+}
 
+static void
+restartcb(const char *buf)
+{
+	ilog(L_MAIN, "libratbox has called the restart callback: %s", buf);
+	restart(buf);
+}
+
+static void
+diecb(const char *buf)
+{
+	if(buf != NULL)
+		ilog(L_MAIN, "libratbox has called the die callback..aborting: %s", buf);
+	else
+		ilog(L_MAIN, "libratbox has called the die callback..aborting");
+	abort();
+}
+
+#ifndef _WIN32
+static int
+seed_with_urandom(void)
+{
+	unsigned int seed;
+	int fd;
+
+	fd = open("/dev/urandom", O_RDONLY);
+	if(fd >= 0)
+	{
+		if(read(fd, &seed, sizeof(seed)) == sizeof(seed))
+		{
+			close(fd);
+			srand(seed);
+			return 1;
+		}
+	}
+	return 0;
+}
+#endif
+
+static void
+seed_with_clock(void)
+{
+	const struct timeval *tv;
+	rb_set_time();
+	tv = rb_current_time_tv();
+	srand(tv->tv_sec ^ (tv->tv_usec | (getpid() << 20)));
+}
+
+static void
+seed_random(void *unused)
+{
+	unsigned int seed;
+	if(rb_get_random(&seed, sizeof(seed)) == -1)
+	{
+#ifndef _WIN32
+		if(!seed_with_urandom())
+#endif
+			seed_with_clock();
+		return;
+	}
+	srand(seed);
+}
+
+int
+ratbox_main(int argc, char *argv[])
+{
+	char emptyname[] = "";
+	int r;
 	/* Check to see if the user is running us as root, which is a nono */
+#ifndef _WIN32
 	if(geteuid() == 0)
 	{
 		fprintf(stderr, "Don't run ircd as root!!!\n");
-		return -1;
+		exit(EXIT_FAILURE);
 	}
+#endif
+	init_sys();
 
-	/*
-	 * save server boot time right away, so getrusage works correctly
-	 */
-	set_time();
-	/*
-	 * Setup corefile size immediately after boot -kre
-	 */
-	setup_corefile();
 
-	/*
-	 * set initialVMTop before we allocate any memory
-	 */
-	initialVMTop = get_vm_top();
 
-	ServerRunning = 0;
-	/* It ain't random, but it ought to be a little harder to guess */
-	srand(SystemTime.tv_sec ^ (SystemTime.tv_usec | (getpid() << 20)));
-	memset(&me, 0, sizeof(me));
-	memset(&meLocalUser, 0, sizeof(meLocalUser));
-	me.localClient = &meLocalUser;
-
-	/* some parts of startup will try to send out messages to opers,
-	 * which will include me.name, which at this point is NULL.  Set it
-	 * to something to prevent a core. --fl
-	 */
-	me.name = ircd_default_name;
-
-	/* Make sure all lists are zeroed */
-	memset(&unknown_list, 0, sizeof(unknown_list));
-	memset(&lclient_list, 0, sizeof(lclient_list));
-	memset(&serv_list, 0, sizeof(serv_list));
-	memset(&global_serv_list, 0, sizeof(global_serv_list));
-	memset(&oper_list, 0, sizeof(oper_list));
-
-	dlinkAddTail(&me, &me.node, &global_client_list);
-
-	memset((void *) &Count, 0, sizeof(Count));
-	memset((void *) &ServerInfo, 0, sizeof(ServerInfo));
-	memset((void *) &AdminInfo, 0, sizeof(AdminInfo));
-
-	/* Initialise the channel capability usage counts... */
-	init_chcap_usage_counts();
-
-	ConfigFileEntry.dpath = DPATH;
-	ConfigFileEntry.configfile = CPATH;	/* Server configuration file */
-	ConfigFileEntry.klinefile = KPATH;	/* Server kline file */
-	ConfigFileEntry.dlinefile = DLPATH;	/* dline file */
-	ConfigFileEntry.xlinefile = XPATH;
-	ConfigFileEntry.resvfile = RESVPATH;
-	ConfigFileEntry.connect_timeout = 30;	/* Default to 30 */
 	myargv = argv;
-	umask(077);		/* better safe than sorry --SRB */
-
 	parseargs(&argc, &argv, myopts);
 
 	if(printVersion)
 	{
 		printf("ircd: version %s\n", ircd_version);
+		printf("ircd: %s\n", rb_lib_version());
 		printf("ircd: configure options\n");
-		printf("%s\n", RATBOX_CONFIGURE_OPTS);
+		puts(RATBOX_CONFIGURE_OPTS);
 		exit(EXIT_SUCCESS);
 	}
 
-	if(chdir(ConfigFileEntry.dpath))
+	if(chdir(basedir))
 	{
-		fprintf(stderr, "Unable to chdir to %s: %s\n", ConfigFileEntry.dpath, strerror(errno));
+		fprintf(stderr, "Unable to chdir to %s: %s\n", basedir, strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
-	setup_signals();
+	add_all_conf_settings();
 
-#if defined(__CYGWIN__) || defined(RATBOX_PROFILE)
-	server_state_foreground = 1;
-#endif
-
-	if (testing_conf)
-		server_state_foreground = 1;
-
-	/* We need this to initialise the fd array before anything else */
-	init_sys();
-	fdlist_init();
-	if(!server_state_foreground)
+	r = read_config_file(configfile);
+	if(r > 0)
 	{
-		comm_close_all();
+		fprintf(stderr,
+			"ERROR: Unable to start ircd: found %d syntax error(s) whlist loading config file.  Try running with -conftest\n",
+			r);
+		return 1;
 	}
 
+	if(testing_conf)
+		fprintf(stderr, "Syntax OK, doing second pass...\n");
+
+
+	r = check_valid_entries();
+	if(r > 0)
+	{
+		fprintf(stderr,
+			"ERROR: Unable to start ircd: found %d error(s) whilst loading config file.  Try running with -conftest\n",
+			r);
+		return 1;
+	}
+
+	if(testing_conf)
+		fprintf(stderr, "Second pass reports OK\n");
+
+	ConfigFileEntry.dpath = basedir;
+	ConfigFileEntry.configfile = configfile;	/* Server configuration file */
 
 	/* Check if there is pidfile and daemon already running */
 	if(!testing_conf)
@@ -613,57 +602,113 @@ main(int argc, char *argv[])
 			print_startup(getpid());
 	}
 
-	init_netio();		/* This needs to be setup early ! -- adrian */
+	/* This must be after we daemonize.. */
+	rb_lib_init(ilogcb, restartcb, diecb, 1, maxconnections, DNODE_HEAP_SIZE, FD_HEAP_SIZE);
+	rb_linebuf_init(LINEBUF_HEAP_SIZE);
 
-	/* Init the event subsystem */
-	eventInit();
+	set_default_conf();
+	rb_set_time();
+	setup_corefile();
+	initialVMTop = get_vm_top();
 
-	init_main_logfile();
-	initBlockHeap();
-	init_dlink_nodes();
-	init_patricia();
-	newconf_init();
+	memset(&me, 0, sizeof(me));
+	me.name = emptyname;
+	memset(&meLocalUser, 0, sizeof(meLocalUser));
+	me.localClient = &meLocalUser;
+
+	/* Make sure all lists are zeroed */
+	memset(&unknown_list, 0, sizeof(unknown_list));
+	memset(&lclient_list, 0, sizeof(lclient_list));
+	memset(&serv_list, 0, sizeof(serv_list));
+	memset(&global_serv_list, 0, sizeof(global_serv_list));
+	memset(&oper_list, 0, sizeof(oper_list));
+
+	rb_dlinkAddTail(&me, &me.node, &global_client_list);
+
+	memset(&Count, 0, sizeof(Count));
+	memset(&ServerInfo, 0, sizeof(ServerInfo));
+	memset(&AdminInfo, 0, sizeof(AdminInfo));
+	memset(&ServerStats, 0, sizeof(struct ServerStatistics));
+
+
+	/* Initialise the channel capability usage counts... */
+	init_chcap_usage_counts();
+
+	ConfigFileEntry.connect_timeout = 30;	/* Default to 30 */
+
+	umask(077);		/* better safe than sorry --SRB */
+
+
+	setup_signals();
 	init_s_conf();
 	init_s_newconf();
-	linebuf_init();		/* set up some linebuf stuff to control paging */
+
+
+
+#if defined(__CYGWIN__) || defined(_WIN32) || defined(RATBOX_PROFILE)
+	server_state_foreground = 1;
+#endif
+
+	if(ConfigServerHide.links_delay > 0)
+		rb_event_add("cache_links", cache_links, NULL, ConfigServerHide.links_delay);
+	else
+		ConfigServerHide.links_disabled = 1;
+
+	if(ConfigFileEntry.use_egd && (ConfigFileEntry.egdpool_path != NULL))
+	{
+		rb_init_prng(ConfigFileEntry.egdpool_path, RB_PRNG_EGD);
+	}
+	else
+		rb_init_prng(NULL, RB_PRNG_DEFAULT);
+	seed_random(NULL);
+
+	init_main_logfile(logFileName);
 	init_hash();
-	clear_scache_hash_table();	/* server cache name table */
 	init_host_hash();
 	clear_hash_parse();
 	init_client();
 	init_channels();
 	initclass();
 	initwhowas();
-	init_stats();
 	init_hook();
 	init_reject();
 	init_cache();
 	init_monitor();
+	init_isupport();
+#ifdef STATIC_MODULES
+	load_static_modules();
+#else
 	load_all_modules(1);
-#ifndef STATIC_MODULES
 	load_core_modules(1);
 #endif
-	init_auth();		/* Initialise the auth code */
 	init_resolver();	/* Needs to be setup before the io loop */
+	init_ssld();
 
-	read_conf_files(YES);	/* cold start init conf files */
+	load_conf_settings();
+	if(ServerInfo.bandb_path == NULL)
+	        ServerInfo.bandb_path = rb_strdup(DBPATH);
+
+	init_bandb();
 	rehash_bans(0);
-#ifndef STATIC_MODULES
 
-	mod_add_path(MODULE_DIR); 
-	mod_add_path(MODULE_DIR "/autoload"); 
+#ifndef STATIC_MODULES
+	mod_add_path(MODULE_DIR);
+	mod_add_path(MODULE_DIR "/autoload");
 #endif
 
 	initialize_server_capabs();	/* Set up default_server_capabs */
 	initialize_global_set_options();
 
+	init_auth();		/* Initialise the auth code - depends on global set options */
+	rehash_dns_vhost();	/* load any vhost dns binds now */
+
 	if(ServerInfo.name == NULL)
 	{
 		fprintf(stderr, "ERROR: No server name specified in serverinfo block.\n");
 		ilog(L_MAIN, "No server name specified in serverinfo block.");
 		exit(EXIT_FAILURE);
 	}
-	me.name = find_or_add(ServerInfo.name);
+	me.name = ServerInfo.name;
 
 	if(ServerInfo.sid[0] == '\0')
 	{
@@ -681,11 +726,23 @@ main(int argc, char *argv[])
 		ilog(L_MAIN, "ERROR: No server description specified in serverinfo block.");
 		exit(EXIT_FAILURE);
 	}
-	strlcpy(me.info, ServerInfo.description, sizeof(me.info));
+	rb_strlcpy(me.info, ServerInfo.description, sizeof(me.info));
 
-	if (testing_conf)
+	if(ServerInfo.ssl_cert != NULL && ServerInfo.ssl_private_key != NULL)
+	{
+		/* just do the rb_setup_ssl_server to validate the config */
+		if(!rb_setup_ssl_server
+		   (ServerInfo.ssl_cert, ServerInfo.ssl_private_key, ServerInfo.ssl_dh_params))
+		{
+			ilog(L_MAIN, "WARNING: Unable to setup SSL.");
+			ircd_ssl_ok = 0;
+		}
+		else
+			ircd_ssl_ok = 1;
+	}
+
+	if(testing_conf)
 	{
-		fflush(stderr);
 		exit(conf_parse_failure ? 1 : 0);
 	}
 
@@ -693,45 +750,32 @@ main(int argc, char *argv[])
 	me.servptr = &me;
 	SetMe(&me);
 	make_server(&me);
-	startup_time = CurrentTime;
-	add_to_client_hash(me.name, &me);
-	add_to_id_hash(me.id, &me);
+	startup_time = rb_current_time();
+	add_to_hash(HASH_CLIENT, me.name, &me);
+	add_to_hash(HASH_ID, me.id, &me);
 
-	dlinkAddAlloc(&me, &global_serv_list);
+	rb_dlinkAddAlloc(&me, &global_serv_list);
 
 	check_class();
 	write_pidfile(pidFileName);
 	load_help();
-	open_logfiles();
+	open_logfiles(logFileName);
 
 	ilog(L_MAIN, "Server Ready");
 
-	eventAddIsh("cleanup_glines", cleanup_glines, NULL, CLEANUP_GLINES_TIME);
-
 	/* We want try_connections to be called as soon as possible now! -- adrian */
 	/* No, 'cause after a restart it would cause all sorts of nick collides */
 	/* um.  by waiting even longer, that just means we have even *more*
 	 * nick collisions.  what a stupid idea. set an event for the IO loop --fl
 	 */
-	eventAddIsh("try_connections", try_connections, NULL, STARTUP_CONNECTIONS_TIME);
-	eventAddOnce("try_connections_startup", try_connections, NULL, 0);
-
-	eventAddIsh("collect_zipstats", collect_zipstats, NULL, ZIPSTATS_TIME);
-
-	/* Setup the timeout check. I'll shift it later :)  -- adrian */
-	eventAddIsh("comm_checktimeouts", comm_checktimeouts, NULL, 1);
-
-	if(ConfigServerHide.links_delay > 0)
-		eventAdd("cache_links", cache_links, NULL,
-			    ConfigServerHide.links_delay);
-	else
-		ConfigServerHide.links_disabled = 1;
+	rb_event_addish("try_connections", try_connections, NULL, STARTUP_CONNECTIONS_TIME);
+	rb_event_addonce("try_connections_startup", try_connections, NULL, 2);
+	rb_event_add("check_rehash", check_rehash, NULL, 3);
+	rb_event_addish("reseed_srand", seed_random, NULL, 300);	/* reseed every 10 minutes */
 
 	if(splitmode)
-		eventAdd("check_splitmode", check_splitmode, NULL, 2);
+		rb_event_add("check_splitmode", check_splitmode, NULL, 5);
 
-	ServerRunning = 1;
-
-	io_loop();
+	rb_lib_loop(0);		/* we'll never return from here */
 	return 0;
 }
