@@ -16,14 +16,18 @@
  *   along with this program; if not, write to the Free Software
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: match.c 24900 2008-01-11 19:02:50Z androsyn $
+ * $Id: match.c 26094 2008-09-19 15:33:46Z androsyn $
  *
  */
 #include "stdinc.h"
-#include "config.h"
-#include "client.h"
+#include "ratbox_lib.h"
 #include "ircd.h"
-#include "irc_string.h"
+#include "struct.h"
+#include "match.h"
+#include "s_conf.h"
+#include "s_log.h"
+#include "send.h"
+#include "client.h"
 
 /* match()
  * 
@@ -41,10 +45,10 @@
 int
 match(const char *mask, const char *name)
 {
-	const unsigned char *m = (const unsigned char *) mask;
-	const unsigned char *n = (const unsigned char *) name;
-	const unsigned char *ma = (const unsigned char *) mask;
-	const unsigned char *na = (const unsigned char *) name;
+	const unsigned char *m = (const unsigned char *)mask;
+	const unsigned char *n = (const unsigned char *)name;
+	const unsigned char *ma = (const unsigned char *)mask;
+	const unsigned char *na = (const unsigned char *)name;
 	int wild = 0;
 	int calls = 0;
 
@@ -55,10 +59,10 @@ match(const char *mask, const char *name)
 		return 0;
 
 	/* if the mask is "*", it matches everything */
-	if((*m == '*') && (*(m+1) == '\0'))
+	if((*m == '*') && (*(m + 1) == '\0'))
 		return 1;
 
-	while (calls++ < MATCH_MAX_CALLS)
+	while(calls++ < MATCH_MAX_CALLS)
 	{
 		if(*m == '*')
 		{
@@ -66,7 +70,7 @@ match(const char *mask, const char *name)
 			 * XXX - shouldn't need to spin here, the mask should have been
 			 * collapsed before match is called
 			 */
-			while (*m == '*')
+			while(*m == '*')
 				m++;
 			wild = 1;
 			ma = m;
@@ -77,9 +81,9 @@ match(const char *mask, const char *name)
 		{
 			if(!*n)
 				return 1;
-			for (m--; (m > (const unsigned char *) mask) && (*m == '?'); m--)
+			for(m--; (m > (const unsigned char *)mask) && (*m == '?'); m--)
 				;
-			if(*m == '*' && (m > (const unsigned char *) mask))
+			if(*m == '*' && (m > (const unsigned char *)mask))
 				return 1;
 			if(!wild)
 				return 0;
@@ -92,7 +96,7 @@ match(const char *mask, const char *name)
 			 * XXX - shouldn't need to spin here, the mask should have been
 			 * collapsed before match is called
 			 */
-			while (*m == '*')
+			while(*m == '*')
 				m++;
 			return (*m == 0);
 		}
@@ -122,10 +126,10 @@ match(const char *mask, const char *name)
 int
 match_esc(const char *mask, const char *name)
 {
-	const unsigned char *m = (const unsigned char *) mask;
-	const unsigned char *n = (const unsigned char *) name;
-	const unsigned char *ma = (const unsigned char *) mask;
-	const unsigned char *na = (const unsigned char *) name;
+	const unsigned char *m = (const unsigned char *)mask;
+	const unsigned char *n = (const unsigned char *)name;
+	const unsigned char *ma = (const unsigned char *)mask;
+	const unsigned char *na = (const unsigned char *)name;
 	int wild = 0;
 	int calls = 0;
 	int quote = 0;
@@ -138,10 +142,10 @@ match_esc(const char *mask, const char *name)
 		return 0;
 
 	/* if the mask is "*", it matches everything */
-	if((*m == '*') && (*(m+1) == '\0'))
+	if((*m == '*') && (*(m + 1) == '\0'))
 		return 1;
 
-	while (calls++ < MATCH_MAX_CALLS)
+	while(calls++ < MATCH_MAX_CALLS)
 	{
 		if(quote)
 			quote++;
@@ -159,7 +163,7 @@ match_esc(const char *mask, const char *name)
 			 * XXX - shouldn't need to spin here, the mask should have been
 			 * collapsed before match is called
 			 */
-			while (*m == '*')
+			while(*m == '*')
 				m++;
 
 			wild = 1;
@@ -175,7 +179,6 @@ match_esc(const char *mask, const char *name)
 				quote++;
 				continue;
 			}
-
 		}
 
 		if(!*m)
@@ -184,9 +187,9 @@ match_esc(const char *mask, const char *name)
 				return 1;
 			if(quote)
 				return 0;
-			for (m--; (m > (const unsigned char *) mask) && (*m == '?'); m--);;
-				
-			if(*m == '*' && (m > (const unsigned char *) mask))
+			for(m--; (m > (const unsigned char *)mask) && (*m == '?'); m--);;
+
+			if(*m == '*' && (m > (const unsigned char *)mask))
 				return 1;
 			if(!wild)
 				return 0;
@@ -201,24 +204,24 @@ match_esc(const char *mask, const char *name)
 			 */
 			if(quote)
 				return 0;
-			while (*m == '*')
+			while(*m == '*')
 				m++;
 			return (*m == 0);
 		}
 
-		if (quote)
+		if(quote)
 			match1 = *m == 's' ? *n == ' ' : ToLower(*m) == ToLower(*n);
-		else if (*m == '?')
+		else if(*m == '?')
 			match1 = 1;
-		else if (*m == '@')
+		else if(*m == '@')
 			match1 = IsLetter(*n);
-		else if (*m == '#')
+		else if(*m == '#')
 			match1 = IsDigit(*n);
 		else
 			match1 = ToLower(*m) == ToLower(*n);
-		if (match1)
+		if(match1)
 		{
-			if(*m)  
+			if(*m)
 				m++;
 			if(*n)
 				n++;
@@ -235,41 +238,39 @@ match_esc(const char *mask, const char *name)
 }
 
 int
-comp_with_mask(void *addr, void *dest, uint mask)
+comp_with_mask(void *addr, void *dest, unsigned int mask)
 {
 	if(memcmp(addr, dest, mask / 8) == 0)
 	{
 		int n = mask / 8;
 		int m = ((-1) << (8 - (mask % 8)));
-		if(mask % 8 == 0 || (((u_char *) addr)[n] & m) == (((u_char *) dest)[n] & m))
-		{
-			return (1);
-		}
+		if(mask % 8 == 0 || (((uint8_t *)addr)[n] & m) == (((uint8_t *)dest)[n] & m))
+			return 1;
 	}
-	return (0);
+	return 0;
 }
 
 int
-comp_with_mask_sock(struct sockaddr *addr, struct sockaddr *dest, uint mask)
+comp_with_mask_sock(struct sockaddr *addr, struct sockaddr *dest, unsigned int mask)
 {
 	void *iaddr = NULL;
 	void *idest = NULL;
-	
+
 	if(addr->sa_family == AF_INET)
 	{
-		iaddr = &((struct sockaddr_in *)addr)->sin_addr;
-		idest = &((struct sockaddr_in *)dest)->sin_addr;
-	} 
-#ifdef IPV6
+		iaddr = &((struct sockaddr_in *)(void *)addr)->sin_addr;
+		idest = &((struct sockaddr_in *)(void *)dest)->sin_addr;
+	}
+#ifdef RB_IPV6
 	else
 	{
-		iaddr = &((struct sockaddr_in6 *)addr)->sin6_addr;
-		idest = &((struct sockaddr_in6 *)dest)->sin6_addr;
-	
-	} 
+		iaddr = &((struct sockaddr_in6 *)(void *)addr)->sin6_addr;
+		idest = &((struct sockaddr_in6 *)(void *)dest)->sin6_addr;
+
+	}
 #endif
 
-	return(comp_with_mask(iaddr, idest, mask));
+	return (comp_with_mask(iaddr, idest, mask));
 }
 
 /*
@@ -280,15 +281,15 @@ comp_with_mask_sock(struct sockaddr *addr, struct sockaddr *dest, uint mask)
 int
 match_ips(const char *s1, const char *s2)
 {
-	struct irc_sockaddr_storage ipaddr, maskaddr;
+	struct rb_sockaddr_storage ipaddr, maskaddr;
 	char mask[BUFSIZE];
 	char address[HOSTLEN + 1];
 	char *len;
 	void *ipptr, *maskptr;
 	int cidrlen, aftype;
 
-	strcpy(mask, s1);
-	strcpy(address, s2);
+	rb_strlcpy(mask, s1, sizeof(mask));
+	rb_strlcpy(address, s2, sizeof(address));
 
 	len = strrchr(mask, '/');
 	if(len == NULL)
@@ -300,7 +301,7 @@ match_ips(const char *s1, const char *s2)
 	if(cidrlen == 0)
 		return 0;
 
-#ifdef IPV6
+#ifdef RB_IPV6
 	if(strchr(mask, ':') && strchr(address, ':'))
 	{
 		aftype = AF_INET6;
@@ -318,14 +319,14 @@ match_ips(const char *s1, const char *s2)
 	else
 		return 0;
 
-	inetpton(aftype, address, ipptr);
-	inetpton(aftype, mask, maskptr);
+	rb_inet_pton(aftype, address, ipptr);
+	rb_inet_pton(aftype, mask, maskptr);
 	if(comp_with_mask(ipptr, maskptr, cidrlen))
 		return 1;
 	else
 		return 0;
 }
- 
+
 
 /* match_cidr()
  * Matches cidr channel bans in *@host form
@@ -336,7 +337,7 @@ match_ips(const char *s1, const char *s2)
 int
 match_cidr(const char *s1, const char *s2)
 {
-	struct irc_sockaddr_storage ipaddr, maskaddr;
+	struct rb_sockaddr_storage ipaddr, maskaddr;
 	char mask[BUFSIZE];
 	char address[NICKLEN + USERLEN + HOSTLEN + 6];
 	char *ipmask;
@@ -345,8 +346,8 @@ match_cidr(const char *s1, const char *s2)
 	void *ipptr, *maskptr;
 	int cidrlen, aftype;
 
-	strcpy(mask, s1);
-	strcpy(address, s2);
+	rb_strlcpy(mask, s1, sizeof(mask));
+	rb_strlcpy(address, s2, sizeof(address));
 
 	ipmask = strrchr(mask, '@');
 	if(ipmask == NULL)
@@ -370,7 +371,7 @@ match_cidr(const char *s1, const char *s2)
 	if(cidrlen == 0)
 		return 0;
 
-#ifdef IPV6
+#ifdef RB_IPV6
 	if(strchr(ip, ':') && strchr(ipmask, ':'))
 	{
 		aftype = AF_INET6;
@@ -388,8 +389,8 @@ match_cidr(const char *s1, const char *s2)
 	else
 		return 0;
 
-	inetpton(aftype, ip, ipptr);
-	inetpton(aftype, ipmask, maskptr);
+	rb_inet_pton(aftype, ip, ipptr);
+	rb_inet_pton(aftype, ipmask, maskptr);
 	if(comp_with_mask(ipptr, maskptr, cidrlen) && match(mask, address))
 		return 1;
 	else
@@ -410,7 +411,7 @@ collapse(char *pattern)
 	if(p == NULL)
 		return NULL;
 
-	while ((c = *p++))
+	while((c = *p++))
 	{
 		if(c == '*')
 		{
@@ -443,7 +444,7 @@ collapse_esc(char *pattern)
 	if(p == NULL)
 		return NULL;
 
-	while ((c = *p++))
+	while((c = *p++))
 	{
 		if(!(f & 2) && c == '*')
 		{
@@ -476,14 +477,14 @@ collapse_esc(char *pattern)
 int
 irccmp(const char *s1, const char *s2)
 {
-	const unsigned char *str1 = (const unsigned char *) s1;
-	const unsigned char *str2 = (const unsigned char *) s2;
+	const unsigned char *str1 = (const unsigned char *)s1;
+	const unsigned char *str2 = (const unsigned char *)s2;
 	int res;
 
 	s_assert(s1 != NULL);
 	s_assert(s2 != NULL);
 
-	while ((res = ToUpper(*str1) - ToUpper(*str2)) == 0)
+	while((res = ToUpper(*str1) - ToUpper(*str2)) == 0)
 	{
 		if(*str1 == '\0')
 			return 0;
@@ -496,13 +497,13 @@ irccmp(const char *s1, const char *s2)
 int
 ircncmp(const char *s1, const char *s2, int n)
 {
-	const unsigned char *str1 = (const unsigned char *) s1;
-	const unsigned char *str2 = (const unsigned char *) s2;
+	const unsigned char *str1 = (const unsigned char *)s1;
+	const unsigned char *str2 = (const unsigned char *)s2;
 	int res;
 	s_assert(s1 != NULL);
 	s_assert(s2 != NULL);
 
-	while ((res = ToUpper(*str1) - ToUpper(*str2)) == 0)
+	while((res = ToUpper(*str1) - ToUpper(*str2)) == 0)
 	{
 		str1++;
 		str2++;
@@ -513,6 +514,119 @@ ircncmp(const char *s1, const char *s2, int n)
 	return (res);
 }
 
+
+/* 
+ * valid_hostname - check hostname for validity
+ *
+ * Inputs       - pointer to user
+ * Output       - YES if valid, NO if not
+ * Side effects - NONE
+ *
+ * NOTE: this doesn't allow a hostname to begin with a dot and
+ * will not allow more dots than chars.
+ */
+int
+valid_hostname(const char *hostname)
+{
+	const char *p = hostname;
+	int found_sep = 0;
+
+	s_assert(NULL != p);
+
+	if(hostname == NULL)
+		return NO;
+
+	if('.' == *p || ':' == *p)
+		return NO;
+
+	while(*p)
+	{
+		if(!IsHostChar(*p))
+			return NO;
+		if(*p == '.' || *p == ':')
+			found_sep++;
+		p++;
+	}
+
+	if(found_sep == 0)
+		return (NO);
+
+	return (YES);
+}
+
+/* 
+ * valid_username - check username for validity
+ *
+ * Inputs       - pointer to user
+ * Output       - YES if valid, NO if not
+ * Side effects - NONE
+ * 
+ * Absolutely always reject any '*' '!' '?' '@' in an user name
+ * reject any odd control characters names.
+ * Allow '.' in username to allow for "first.last"
+ * style of username
+ */
+int
+valid_username(const char *username)
+{
+	int dots = 0;
+	const char *p = username;
+
+	s_assert(NULL != p);
+
+	if(username == NULL)
+		return NO;
+
+	if('~' == *p)
+		++p;
+
+	/* reject usernames that don't start with an alphanum
+	 * i.e. reject jokers who have '-@somehost' or '.@somehost'
+	 * or "-hi-@somehost", "h-----@somehost" would still be accepted.
+	 */
+	if(!IsAlNum(*p))
+		return NO;
+
+	while(*++p)
+	{
+		if((*p == '.') && ConfigFileEntry.dots_in_ident)
+		{
+			dots++;
+			if(dots > ConfigFileEntry.dots_in_ident)
+				return NO;
+			if(!IsUserChar(p[1]))
+				return NO;
+		}
+		else if(!IsUserChar(*p))
+			return NO;
+	}
+	return YES;
+}
+
+int
+valid_servername(const char *servername)
+{
+	const char *s;
+	int dots = 0;
+
+	for(s = servername; *s != '\0'; s++)
+	{
+		if(!IsServChar(*s))
+		{
+			return NO;
+		}
+		if(*s == '.')
+			dots++;
+	}
+	if(dots == 0)
+	{
+		return NO;
+	}
+	return YES;
+}
+
+
+
 const unsigned char ToLowerTab[] = {
 	0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xa,
 	0xb, 0xc, 0xd, 0xe, 0xf, 0x10, 0x11, 0x12, 0x13, 0x14,
