@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
  *  Copyright (C) 1996-2002 Hybrid Development Team
- *  Copyright (C) 2002-2005 ircd-ratbox development team
+ *  Copyright (C) 2002-2004 ircd-ratbox development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,15 +18,15 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
  *  USA
  *
- *  $Id: modules.c 21713 2006-01-18 22:56:08Z jilles $
+ *  $Id: modules.c 26172 2008-10-26 21:03:00Z androsyn $
  */
 
 #include "stdinc.h"
-
-
+#include "struct.h"
+#include "hook.h"
 #include "modules.h"
 #include "s_log.h"
 #include "ircd.h"
@@ -36,29 +36,14 @@
 #include "s_newconf.h"
 #include "numeric.h"
 #include "parse.h"
-#include "ircd_defs.h"
-#include "irc_string.h"
-#include "memory.h"
-#include "tools.h"
-#include "sprintf_irc.h"
+#include "match.h"
 
 
-
-/* -TimeMr14C:
- * I have moved the dl* function definitions and
- * the two functions (load_a_module / unload_a_module) to the
- * file dynlink.c 
- * And also made the necessary changes to those functions
- * to comply with shl_load and friends.
- * In this file, to keep consistency with the makefile, 
- * I added the ability to load *.sl files, too.
- * 27/02/2002
- */
-
 #ifndef STATIC_MODULES
 
+#include "ltdl.h"
 struct module **modlist = NULL;
-
+static char unknown_ver[] = "<unknown>";
 static const char *core_module_table[] = {
 	"m_die",
 	"m_error",
@@ -71,7 +56,6 @@ static const char *core_module_table[] = {
 	"m_part",
 	"m_quit",
 	"m_server",
-	"m_sjoin",
 	"m_squit",
 	NULL
 };
@@ -80,10 +64,11 @@ static const char *core_module_table[] = {
 int num_mods = 0;
 int max_mods = MODS_INCREMENT;
 
-static dlink_list mod_paths;
+static rb_dlink_list mod_paths;
+
+static void increase_modlist(void);
 
 static int mo_modload(struct Client *, struct Client *, int, const char **);
-static int mo_modlist(struct Client *, struct Client *, int, const char **);
 static int mo_modreload(struct Client *, struct Client *, int, const char **);
 static int mo_modunload(struct Client *, struct Client *, int, const char **);
 static int mo_modrestart(struct Client *, struct Client *, int, const char **);
@@ -103,28 +88,45 @@ struct Message modreload_msgtab = {
 	{mg_unreg, mg_not_oper, mg_ignore, mg_ignore, mg_ignore, {mo_modreload, 2}}
 };
 
-struct Message modlist_msgtab = {
-	"MODLIST", 0, 0, 0, MFLG_SLOW,
-	{mg_unreg, mg_not_oper, mg_ignore, mg_ignore, mg_ignore, {mo_modlist, 0}}
-};
 
 struct Message modrestart_msgtab = {
 	"MODRESTART", 0, 0, 0, MFLG_SLOW,
 	{mg_unreg, mg_not_oper, mg_ignore, mg_ignore, mg_ignore, {mo_modrestart, 0}}
 };
 
+
+static int mo_modlist(struct Client *, struct Client *, int, const char **);
+
+struct Message modlist_msgtab = {
+	"MODLIST", 0, 0, 0, MFLG_SLOW,
+	{mg_unreg, mg_not_oper, mg_ignore, mg_ignore, mg_ignore, {mo_modlist, 0}}
+};
+
+
 extern struct Message error_msgtab;
+#endif /* !STATIC_MODULES */
 
 void
 modules_init(void)
 {
+#ifndef STATIC_MODULES
+	if(lt_dlinit())
+	{
+		ilog(L_MAIN, "lt_dlinit failed");
+		exit(0);
+	}
+
+	modlist = rb_malloc(sizeof(struct module) * (MODS_INCREMENT));
 	mod_add_cmd(&modload_msgtab);
 	mod_add_cmd(&modunload_msgtab);
 	mod_add_cmd(&modreload_msgtab);
-	mod_add_cmd(&modlist_msgtab);
 	mod_add_cmd(&modrestart_msgtab);
+	mod_add_cmd(&modlist_msgtab);	/* have modlist if we are static or not */
+#endif
+
 }
 
+#ifndef STATIC_MODULES
 /* mod_find_path()
  *
  * input	- path
@@ -134,10 +136,10 @@ modules_init(void)
 static struct module_path *
 mod_find_path(const char *path)
 {
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 	struct module_path *mpath;
 
-	DLINK_FOREACH(ptr, mod_paths.head)
+	RB_DLINK_FOREACH(ptr, mod_paths.head)
 	{
 		mpath = ptr->data;
 
@@ -162,10 +164,10 @@ mod_add_path(const char *path)
 	if(mod_find_path(path))
 		return;
 
-	pathst = MyMalloc(sizeof(struct module_path));
+	pathst = rb_malloc(sizeof(struct module_path));
 
 	strcpy(pathst->path, path);
-	dlinkAddAlloc(pathst, &mod_paths);
+	rb_dlinkAddAlloc(pathst, &mod_paths);
 }
 
 /* mod_clear_paths()
@@ -177,39 +179,18 @@ mod_add_path(const char *path)
 void
 mod_clear_paths(void)
 {
-	dlink_node *ptr, *next_ptr;
+	rb_dlink_node *ptr, *next_ptr;
 
-	DLINK_FOREACH_SAFE(ptr, next_ptr, mod_paths.head)
+	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, mod_paths.head)
 	{
-		MyFree(ptr->data);
-		free_dlink_node(ptr);
+		rb_free(ptr->data);
+		rb_free_rb_dlink_node(ptr);
 	}
 
 	mod_paths.head = mod_paths.tail = NULL;
 	mod_paths.length = 0;
 }
 
-/* irc_basename
- *
- * input	-
- * output	-
- * side effects -
- */
-char *
-irc_basename(const char *path)
-{
-	char *mod_basename = MyMalloc(strlen(path) + 1);
-	const char *s;
-
-	if(!(s = strrchr(path, '/')))
-		s = path;
-	else
-		s++;
-
-	(void) strcpy(mod_basename, s);
-	return mod_basename;
-}
-
 /* findmodule_byname
  *
  * input        -
@@ -222,7 +203,7 @@ findmodule_byname(const char *name)
 {
 	int i;
 
-	for (i = 0; i < num_mods; i++)
+	for(i = 0; i < num_mods; i++)
 	{
 		if(!irccmp(modlist[i]->name, name))
 			return i;
@@ -239,18 +220,26 @@ findmodule_byname(const char *name)
 void
 load_all_modules(int warn)
 {
+	static const char *shext = SHLIBEXT;
 	DIR *system_module_dir = NULL;
 	struct dirent *ldirent = NULL;
 	char module_fq_name[PATH_MAX + 1];
+	char module_dir_name[PATH_MAX + 1];
 	int len;
-
+	int ext_len = strlen(SHLIBEXT);
 	modules_init();
 
-	modlist = (struct module **) MyMalloc(sizeof(struct module) * (MODS_INCREMENT));
 
 	max_mods = MODS_INCREMENT;
+	rb_strlcpy(module_dir_name, AUTOMODPATH, sizeof(module_dir_name));
+	system_module_dir = opendir(module_dir_name);
 
-	system_module_dir = opendir(AUTOMODPATH);
+	if(system_module_dir == NULL)
+	{
+		rb_strlcpy(module_dir_name, ConfigFileEntry.dpath, sizeof(module_dir_name));
+		rb_strlcat(module_dir_name, "/modules/autoload", sizeof(module_dir_name));
+		system_module_dir = opendir(module_dir_name);
+	}
 
 	if(system_module_dir == NULL)
 	{
@@ -258,17 +247,18 @@ load_all_modules(int warn)
 		return;
 	}
 
-	while ((ldirent = readdir(system_module_dir)) != NULL)
+	while((ldirent = readdir(system_module_dir)) != NULL)
 	{
-        	len = strlen(ldirent->d_name);
-		if((len > 3) && !strcmp(ldirent->d_name+len-3, SHARED_SUFFIX))
-                {
-		 	(void) ircsnprintf(module_fq_name, sizeof(module_fq_name), "%s/%s", AUTOMODPATH, ldirent->d_name);
-		 	(void) load_a_module(module_fq_name, warn, 0);
-                }
+		len = strlen(ldirent->d_name);
 
+		if((len > ext_len) && !strcmp(ldirent->d_name + len - ext_len, shext))
+		{
+			(void)rb_snprintf(module_fq_name, sizeof(module_fq_name), "%s/%s",
+					  module_dir_name, ldirent->d_name);
+			(void)load_a_module(module_fq_name, warn, 0);
+		}
 	}
-	(void) closedir(system_module_dir);
+	(void)closedir(system_module_dir);
 }
 
 /* load_core_modules()
@@ -280,23 +270,47 @@ load_all_modules(int warn)
 void
 load_core_modules(int warn)
 {
-	char module_name[MAXPATHLEN];
+	char module_name[PATH_MAX + 1];
+	char dir_name[PATH_MAX + 1];
+	DIR *core_dir;
 	int i;
 
-
-	for (i = 0; core_module_table[i]; i++)
+	core_dir = opendir(MODPATH);
+	if(core_dir == NULL)
 	{
-		ircsnprintf(module_name, sizeof(module_name), "%s/%s%s", MODPATH,
-			    core_module_table[i], SHARED_SUFFIX);
+		rb_snprintf(dir_name, sizeof(dir_name), "%s/modules", ConfigFileEntry.dpath);
+		core_dir = opendir(dir_name);
+	}
+	else
+	{
+		rb_strlcpy(dir_name, MODPATH, sizeof(dir_name));
+	}
+
+
+	if(core_dir == NULL)
+	{
+		ilog(L_MAIN,
+		     "Cannot find where core modules are located(tried %s and %s): terminating ircd",
+		     MODPATH, dir_name);
+		exit(0);
+	}
+
+
+	for(i = 0; core_module_table[i]; i++)
+	{
+
+		rb_snprintf(module_name, sizeof(module_name), "%s/%s%s", dir_name,
+			    core_module_table[i], SHLIBEXT);
 
 		if(load_a_module(module_name, warn, 1) == -1)
 		{
 			ilog(L_MAIN,
 			     "Error loading core module %s%s: terminating ircd",
-			     core_module_table[i], SHARED_SUFFIX);
+			     core_module_table[i], SHLIBEXT);
 			exit(0);
 		}
 	}
+	closedir(core_dir);
 }
 
 /* load_one_module()
@@ -309,16 +323,16 @@ int
 load_one_module(const char *path, int coremodule)
 {
 	char modpath[MAXPATHLEN];
-	dlink_node *pathst;
+	rb_dlink_node *pathst;
 	struct module_path *mpath;
 
 	struct stat statbuf;
 
-	DLINK_FOREACH(pathst, mod_paths.head)
+	RB_DLINK_FOREACH(pathst, mod_paths.head)
 	{
 		mpath = pathst->data;
 
-		ircsnprintf(modpath, sizeof(modpath), "%s/%s", mpath->path, path);
+		rb_snprintf(modpath, sizeof(modpath), "%s/%s", mpath->path, path);
 		if((strstr(modpath, "../") == NULL) && (strstr(modpath, "/..") == NULL))
 		{
 			if(stat(modpath, &statbuf) == 0)
@@ -350,25 +364,22 @@ mo_modload(struct Client *client_p, struct Client *source_p, int parc, const cha
 
 	if(!IsOperAdmin(source_p))
 	{
-		sendto_one(source_p, form_str(ERR_NOPRIVS),
-			   me.name, source_p->name, "admin");
+		sendto_one(source_p, form_str(ERR_NOPRIVS), me.name, source_p->name, "admin");
 		return 0;
 	}
 
-	m_bn = irc_basename(parv[1]);
+	m_bn = rb_basename(parv[1]);
 
 	if(findmodule_byname(m_bn) != -1)
 	{
-		sendto_one(source_p,
-			   ":%s NOTICE %s :Module %s is already loaded",
-			   me.name, source_p->name, m_bn);
-		MyFree(m_bn);
+		sendto_one_notice(source_p, ":Module %s is already loaded", m_bn);
+		rb_free(m_bn);
 		return 0;
 	}
 
 	load_one_module(parv[1], 0);
 
-	MyFree(m_bn);
+	rb_free(m_bn);
 
 	return 0;
 }
@@ -383,36 +394,32 @@ mo_modunload(struct Client *client_p, struct Client *source_p, int parc, const c
 
 	if(!IsOperAdmin(source_p))
 	{
-		sendto_one(source_p, form_str(ERR_NOPRIVS),
-			   me.name, source_p->name, "admin");
+		sendto_one(source_p, form_str(ERR_NOPRIVS), me.name, source_p->name, "admin");
 		return 0;
 	}
 
-	m_bn = irc_basename(parv[1]);
+	m_bn = rb_basename(parv[1]);
 
 	if((modindex = findmodule_byname(m_bn)) == -1)
 	{
-		sendto_one(source_p,
-			   ":%s NOTICE %s :Module %s is not loaded", me.name, source_p->name, m_bn);
-		MyFree(m_bn);
+		sendto_one_notice(source_p, ":Module %s is not loaded", m_bn);
+		rb_free(m_bn);
 		return 0;
 	}
 
 	if(modlist[modindex]->core == 1)
 	{
-		sendto_one(source_p,
-			   ":%s NOTICE %s :Module %s is a core module and may not be unloaded",
-			   me.name, source_p->name, m_bn);
-		MyFree(m_bn);
+		sendto_one_notice(source_p,
+				  ":Module %s is a core module and may not be unloaded", m_bn);
+		rb_free(m_bn);
 		return 0;
 	}
 
 	if(unload_one_module(m_bn, 1) == -1)
 	{
-		sendto_one(source_p,
-			   ":%s NOTICE %s :Module %s is not loaded", me.name, source_p->name, m_bn);
+		sendto_one_notice(source_p, ":Module %s is not loaded", m_bn);
 	}
-	MyFree(m_bn);
+	rb_free(m_bn);
 	return 0;
 }
 
@@ -426,18 +433,16 @@ mo_modreload(struct Client *client_p, struct Client *source_p, int parc, const c
 
 	if(!IsOperAdmin(source_p))
 	{
-		sendto_one(source_p, form_str(ERR_NOPRIVS),
-			   me.name, source_p->name, "admin");
+		sendto_one(source_p, form_str(ERR_NOPRIVS), me.name, source_p->name, "admin");
 		return 0;
 	}
 
-	m_bn = irc_basename(parv[1]);
+	m_bn = rb_basename(parv[1]);
 
 	if((modindex = findmodule_byname(m_bn)) == -1)
 	{
-		sendto_one(source_p,
-			   ":%s NOTICE %s :Module %s is not loaded", me.name, source_p->name, m_bn);
-		MyFree(m_bn);
+		sendto_one_notice(source_p, ":Module %s is not loaded", m_bn);
+		rb_free(m_bn);
 		return 0;
 	}
 
@@ -445,9 +450,8 @@ mo_modreload(struct Client *client_p, struct Client *source_p, int parc, const c
 
 	if(unload_one_module(m_bn, 1) == -1)
 	{
-		sendto_one(source_p,
-			   ":%s NOTICE %s :Module %s is not loaded", me.name, source_p->name, m_bn);
-		MyFree(m_bn);
+		sendto_one_notice(source_p, ":Module %s is not loaded", m_bn);
+		rb_free(m_bn);
 		return 0;
 	}
 
@@ -459,7 +463,37 @@ mo_modreload(struct Client *client_p, struct Client *source_p, int parc, const c
 		exit(0);
 	}
 
-	MyFree(m_bn);
+	rb_free(m_bn);
+	return 0;
+}
+
+
+/* unload and reload all modules */
+static int
+mo_modrestart(struct Client *client_p, struct Client *source_p, int parc, const char **parv)
+{
+	int modnum;
+
+	if(!IsOperAdmin(source_p))
+	{
+		sendto_one(source_p, form_str(ERR_NOPRIVS), me.name, source_p->name, "admin");
+		return 0;
+	}
+
+	sendto_one_notice(source_p, ":Reloading all modules");
+
+	modnum = num_mods;
+	while(num_mods)
+		unload_one_module(modlist[0]->name, 0);
+
+	load_all_modules(0);
+	load_core_modules(0);
+	rehash(0);
+
+	sendto_realops_flags(UMODE_ALL, L_ALL,
+			     "Module Restart: %d modules unloaded, %d modules loaded",
+			     modnum, num_mods);
+	ilog(L_MAIN, "Module Restart: %d modules unloaded, %d modules loaded", modnum, num_mods);
 	return 0;
 }
 
@@ -471,12 +505,11 @@ mo_modlist(struct Client *client_p, struct Client *source_p, int parc, const cha
 
 	if(!IsOperAdmin(source_p))
 	{
-		sendto_one(source_p, form_str(ERR_NOPRIVS),
-			   me.name, source_p->name, "admin");
+		sendto_one(source_p, form_str(ERR_NOPRIVS), me.name, source_p->name, "admin");
 		return 0;
 	}
-
-	for (i = 0; i < num_mods; i++)
+	SetCork(source_p);
+	for(i = 0; i < num_mods; i++)
 	{
 		if(parc > 1)
 		{
@@ -497,199 +530,11 @@ mo_modlist(struct Client *client_p, struct Client *source_p, int parc, const cha
 				   modlist[i]->core ? "(core)" : "");
 		}
 	}
-
+	ClearCork(source_p);
 	sendto_one(source_p, form_str(RPL_ENDOFMODLIST), me.name, source_p->name);
 	return 0;
 }
 
-/* unload and reload all modules */
-static int
-mo_modrestart(struct Client *client_p, struct Client *source_p, int parc, const char **parv)
-{
-	int modnum;
-
-	if(!IsOperAdmin(source_p))
-	{
-		sendto_one(source_p, form_str(ERR_NOPRIVS),
-			   me.name, source_p->name, "admin");
-		return 0;
-	}
-
-	sendto_one(source_p, ":%s NOTICE %s :Reloading all modules", me.name, parv[0]);
-
-	modnum = num_mods;
-	while (num_mods)
-		unload_one_module(modlist[0]->name, 0);
-
-	load_all_modules(0);
-	load_core_modules(0);
-	rehash(0);
-
-	sendto_realops_flags(UMODE_ALL, L_ALL,
-			     "Module Restart: %d modules unloaded, %d modules loaded",
-			     modnum, num_mods);
-	ilog(L_MAIN, "Module Restart: %d modules unloaded, %d modules loaded", modnum, num_mods);
-	return 0;
-}
-
-
-
-#ifndef RTLD_NOW
-#define RTLD_NOW RTLD_LAZY	/* openbsd deficiency */
-#endif
-
-static void increase_modlist(void);
-
-#define MODS_INCREMENT 10
-
-static char unknown_ver[] = "<unknown>";
-
-/* This file contains the core functions to use dynamic libraries.
- * -TimeMr14C
- */
-
-
-#ifdef HAVE_MACH_O_DYLD_H
-/*
-** jmallett's dl*(3) shims for NSModule(3) systems.
-*/
-#include <mach-o/dyld.h>
-
-#ifndef HAVE_DLOPEN
-#ifndef	RTLD_LAZY
-#define RTLD_LAZY 2185		/* built-in dl*(3) don't care */
-#endif
-
-void undefinedErrorHandler(const char *);
-NSModule multipleErrorHandler(NSSymbol, NSModule, NSModule);
-void linkEditErrorHandler(NSLinkEditErrors, int, const char *, const char *);
-char *dlerror(void);
-void *dlopen(char *, int);
-int dlclose(void *);
-void *dlsym(void *, char *);
-
-static int firstLoad = TRUE;
-static int myDlError;
-static char *myErrorTable[] = { "Loading file as object failed\n",
-	"Loading file as object succeeded\n",
-	"Not a valid shared object\n",
-	"Architecture of object invalid on this architecture\n",
-	"Invalid or corrupt image\n",
-	"Could not access object\n",
-	"NSCreateObjectFileImageFromFile failed\n",
-	NULL
-};
-
-void
-undefinedErrorHandler(const char *symbolName)
-{
-	sendto_realops_flags(UMODE_ALL, L_ALL, "Undefined symbol: %s", symbolName);
-	ilog(L_MAIN, "Undefined symbol: %s", symbolName);
-	return;
-}
-
-NSModule
-multipleErrorHandler(NSSymbol s, NSModule old, NSModule new)
-{
-	/* XXX
-	 ** This results in substantial leaking of memory... Should free one
-	 ** module, maybe?
-	 */
-	sendto_realops_flags(UMODE_ALL, L_ALL,
-			     "Symbol `%s' found in `%s' and `%s'",
-			     NSNameOfSymbol(s), NSNameOfModule(old), NSNameOfModule(new));
-	ilog(L_MAIN, "Symbol `%s' found in `%s' and `%s'",
-	     NSNameOfSymbol(s), NSNameOfModule(old), NSNameOfModule(new));
-	/* We return which module should be considered valid, I believe */
-	return new;
-}
-
-void
-linkEditErrorHandler(NSLinkEditErrors errorClass, int errnum,
-		     const char *fileName, const char *errorString)
-{
-	sendto_realops_flags(UMODE_ALL, L_ALL,
-			     "Link editor error: %s for %s", errorString, fileName);
-	ilog(L_MAIN, "Link editor error: %s for %s", errorString, fileName);
-	return;
-}
-
-char *
-dlerror(void)
-{
-	return myDlError == NSObjectFileImageSuccess ? NULL : myErrorTable[myDlError % 7];
-}
-
-void *
-dlopen(char *filename, int unused)
-{
-	NSObjectFileImage myImage;
-	NSModule myModule;
-
-	if(firstLoad)
-	{
-		/*
-		 ** If we are loading our first symbol (huzzah!) we should go ahead
-		 ** and install link editor error handling!
-		 */
-		NSLinkEditErrorHandlers linkEditorErrorHandlers;
-
-		linkEditorErrorHandlers.undefined = undefinedErrorHandler;
-		linkEditorErrorHandlers.multiple = multipleErrorHandler;
-		linkEditorErrorHandlers.linkEdit = linkEditErrorHandler;
-		NSInstallLinkEditErrorHandlers(&linkEditorErrorHandlers);
-		firstLoad = FALSE;
-	}
-	myDlError = NSCreateObjectFileImageFromFile(filename, &myImage);
-	if(myDlError != NSObjectFileImageSuccess)
-	{
-		return NULL;
-	}
-	myModule = NSLinkModule(myImage, filename, NSLINKMODULE_OPTION_PRIVATE);
-	return (void *) myModule;
-}
-
-int
-dlclose(void *myModule)
-{
-	NSUnLinkModule(myModule, FALSE);
-	return 0;
-}
-
-void *
-dlsym(void *myModule, char *mySymbolName)
-{
-	NSSymbol mySymbol;
-
-	mySymbol = NSLookupSymbolInModule((NSModule) myModule, mySymbolName);
-	return NSAddressOfSymbol(mySymbol);
-}
-#endif
-#endif
-
-
-/*
- * HPUX dl compat functions
- */
-#if defined(HAVE_SHL_LOAD) && !defined(HAVE_DLOPEN)
-#define RTLD_LAZY BIND_DEFERRED
-#define RTLD_GLOBAL DYNAMIC_PATH
-#define dlopen(file,mode) (void *)shl_load((file), (mode), (long) 0)
-#define dlclose(handle) shl_unload((shl_t)(handle))
-#define dlsym(handle,name) hpux_dlsym(handle,name)
-#define dlerror() strerror(errno)
-
-static void *
-hpux_dlsym(void *handle, char *name)
-{
-	void *sym_addr;
-	if(!shl_findsym((shl_t *) & handle, name, TYPE_UNDEFINED, &sym_addr))
-		return sym_addr;
-	return NULL;
-}
-
-#endif
-
 /* unload_one_module()
  *
  * inputs	- name of module to unload
@@ -705,17 +550,6 @@ unload_one_module(const char *name, int warn)
 	if((modindex = findmodule_byname(name)) == -1)
 		return -1;
 
-	/*
-	 ** XXX - The type system in C does not allow direct conversion between
-	 ** data and function pointers, but as it happens, most C compilers will
-	 ** safely do this, however it is a theoretical overlow to cast as we 
-	 ** must do here.  I have library functions to take care of this, but 
-	 ** despite being more "correct" for the C language, this is more 
-	 ** practical.  Removing the abuse of the ability to cast ANY pointer
-	 ** to and from an integer value here will break some compilers.
-	 **          -jmallett
-	 */
-	/* Left the comment in but the code isn't here any more         -larne */
 	switch (modlist[modindex]->mapi_version)
 	{
 	case 1:
@@ -724,7 +558,7 @@ unload_one_module(const char *name, int warn)
 			if(mheader->mapi_command_list)
 			{
 				struct Message **m;
-				for (m = mheader->mapi_command_list; *m; ++m)
+				for(m = mheader->mapi_command_list; *m; ++m)
 					mod_del_cmd(*m);
 			}
 
@@ -734,8 +568,8 @@ unload_one_module(const char *name, int warn)
 			if(mheader->mapi_hfn_list)
 			{
 				mapi_hfn_list_av1 *m;
-				for (m = mheader->mapi_hfn_list; m->hapi_name; ++m)
-					remove_hook(m->hapi_name, m->fn);
+				for(m = mheader->mapi_hfn_list; m->hapi_name; ++m)
+					remove_hook(m->hapi_name, m->hookfn);
 			}
 
 			if(mheader->mapi_unregister)
@@ -751,9 +585,9 @@ unload_one_module(const char *name, int warn)
 		break;
 	}
 
-	dlclose(modlist[modindex]->address);
+	lt_dlclose(modlist[modindex]->address);
 
-	MyFree(modlist[modindex]->name);
+	rb_free(modlist[modindex]->name);
 	memcpy(&modlist[modindex], &modlist[modindex + 1],
 	       sizeof(struct module) * ((num_mods - 1) - modindex));
 
@@ -780,46 +614,51 @@ unload_one_module(const char *name, int warn)
 int
 load_a_module(const char *path, int warn, int core)
 {
-	void *tmpptr = NULL;
+	lt_dlhandle tmpptr = NULL;
 
 	char *mod_basename;
 	const char *ver;
 
+	void *mapi_base;
 	int *mapi_version;
 
-	mod_basename = irc_basename(path);
+	mod_basename = rb_basename(path);
 
-	tmpptr = dlopen(path, RTLD_NOW);
+	tmpptr = lt_dlopen(path);
 
 	if(tmpptr == NULL)
 	{
-		const char *err = dlerror();
+		const char *err = lt_dlerror();
 
 		sendto_realops_flags(UMODE_ALL, L_ALL,
 				     "Error loading module %s: %s", mod_basename, err);
 		ilog(L_MAIN, "Error loading module %s: %s", mod_basename, err);
-		MyFree(mod_basename);
+		rb_free(mod_basename);
 		return -1;
 	}
 
-
 	/*
 	 * _mheader is actually a struct mapi_mheader_*, but mapi_version
 	 * is always the first member of this structure, so we treate it
 	 * as a single int in order to determine the API version.
 	 *      -larne.
 	 */
-	mapi_version = (int *) (uintptr_t) dlsym(tmpptr, "_mheader");
-	if((mapi_version == NULL
-	    && (mapi_version = (int *) (uintptr_t) dlsym(tmpptr, "__mheader")) == NULL)
-	   || MAPI_MAGIC(*mapi_version) != MAPI_MAGIC_HDR)
+	mapi_base = lt_dlsym(tmpptr, "_mheader");
+	if(mapi_base == NULL)
+	{
+		mapi_base = lt_dlsym(tmpptr, "__mheader");
+	}
+
+	mapi_version = (int *)mapi_base;
+
+	if(mapi_base == NULL || (MAPI_MAGIC(*mapi_version) != MAPI_MAGIC_HDR))
 	{
 		sendto_realops_flags(UMODE_ALL, L_ALL,
 				     "Data format error: module %s has no MAPI header.",
 				     mod_basename);
 		ilog(L_MAIN, "Data format error: module %s has no MAPI header.", mod_basename);
-		(void) dlclose(tmpptr);
-		MyFree(mod_basename);
+		lt_dlclose(tmpptr);
+		rb_free(mod_basename);
 		return -1;
 	}
 
@@ -827,7 +666,7 @@ load_a_module(const char *path, int warn, int core)
 	{
 	case 1:
 		{
-			struct mapi_mheader_av1 *mheader = (struct mapi_mheader_av1 *) mapi_version;	/* see above */
+			struct mapi_mheader_av1 *mheader = mapi_base;	/* see above */
 			if(mheader->mapi_register && (mheader->mapi_register() == -1))
 			{
 				ilog(L_MAIN, "Module %s indicated failure during load.",
@@ -835,29 +674,29 @@ load_a_module(const char *path, int warn, int core)
 				sendto_realops_flags(UMODE_ALL, L_ALL,
 						     "Module %s indicated failure during load.",
 						     mod_basename);
-				dlclose(tmpptr);
-				MyFree(mod_basename);
+				lt_dlclose(tmpptr);
+				rb_free(mod_basename);
 				return -1;
 			}
 			if(mheader->mapi_command_list)
 			{
 				struct Message **m;
-				for (m = mheader->mapi_command_list; *m; ++m)
+				for(m = mheader->mapi_command_list; *m; ++m)
 					mod_add_cmd(*m);
 			}
 
 			if(mheader->mapi_hook_list)
 			{
 				mapi_hlist_av1 *m;
-				for (m = mheader->mapi_hook_list; m->hapi_name; ++m)
+				for(m = mheader->mapi_hook_list; m->hapi_name; ++m)
 					*m->hapi_id = register_hook(m->hapi_name);
 			}
 
 			if(mheader->mapi_hfn_list)
 			{
 				mapi_hfn_list_av1 *m;
-				for (m = mheader->mapi_hfn_list; m->hapi_name; ++m)
-					add_hook(m->hapi_name, m->fn);
+				for(m = mheader->mapi_hfn_list; m->hapi_name; ++m)
+					add_hook(m->hapi_name, m->hookfn);
 			}
 
 			ver = mheader->mapi_module_version;
@@ -870,8 +709,8 @@ load_a_module(const char *path, int warn, int core)
 		sendto_realops_flags(UMODE_ALL, L_ALL,
 				     "Module %s has unknown/unsupported MAPI version %d.",
 				     mod_basename, *mapi_version);
-		dlclose(tmpptr);
-		MyFree(mod_basename);
+		lt_dlclose(tmpptr);
+		rb_free(mod_basename);
 		return -1;
 	}
 
@@ -880,11 +719,11 @@ load_a_module(const char *path, int warn, int core)
 
 	increase_modlist();
 
-	modlist[num_mods] = MyMalloc(sizeof(struct module));
+	modlist[num_mods] = rb_malloc(sizeof(struct module));
 	modlist[num_mods]->address = tmpptr;
 	modlist[num_mods]->version = ver;
 	modlist[num_mods]->core = core;
-	DupString(modlist[num_mods]->name, mod_basename);
+	modlist[num_mods]->name = rb_strdup(mod_basename);
 	modlist[num_mods]->mapi_header = mapi_version;
 	modlist[num_mods]->mapi_version = MAPI_VERSION(*mapi_version);
 	num_mods++;
@@ -892,13 +731,12 @@ load_a_module(const char *path, int warn, int core)
 	if(warn == 1)
 	{
 		sendto_realops_flags(UMODE_ALL, L_ALL,
-				     "Module %s [version: %s; MAPI version: %d] loaded at 0x%lx",
-				     mod_basename, ver, MAPI_VERSION(*mapi_version),
-				     (unsigned long) tmpptr);
-		ilog(L_MAIN, "Module %s [version: %s; MAPI version: %d] loaded at 0x%lx",
-		     mod_basename, ver, MAPI_VERSION(*mapi_version), (unsigned long) tmpptr);
+				     "Module %s [version: %s; MAPI version: %d] loaded at 0x%p",
+				     mod_basename, ver, MAPI_VERSION(*mapi_version), (void *)tmpptr);
+		ilog(L_MAIN, "Module %s [version: %s; MAPI version: %d] loaded at 0x%p",
+		     mod_basename, ver, MAPI_VERSION(*mapi_version), (void *)tmpptr);
 	}
-	MyFree(mod_basename);
+	rb_free(mod_basename);
 	return 0;
 }
 
@@ -912,32 +750,76 @@ load_a_module(const char *path, int warn, int core)
 static void
 increase_modlist(void)
 {
-	struct module **new_modlist = NULL;
-
 	if((num_mods + 1) < max_mods)
 		return;
 
-	new_modlist = (struct module **) MyMalloc(sizeof(struct module) *
-						  (max_mods + MODS_INCREMENT));
-	memcpy((void *) new_modlist, (void *) modlist, sizeof(struct module) * num_mods);
-
-	MyFree(modlist);
-	modlist = new_modlist;
+	modlist = rb_realloc(modlist, sizeof(struct module) * (max_mods + MODS_INCREMENT));
 	max_mods += MODS_INCREMENT;
 }
+#endif /* !STATIC_MODULES */
 
-#else /* STATIC_MODULES */
 
-/* load_all_modules()
- *
- * input        -
- * output       -
- * side effects - all the msgtabs are added for static modules
- */
+#ifdef STATIC_MODULES
+extern const struct mapi_header_av1 *static_mapi_headers[];
 void
-load_all_modules(int warn)
+load_static_modules(void)
 {
-	load_static_modules();
-}
+	int x;
+	const int *mapi_version;
 
-#endif /* STATIC_MODULES */
+	modules_init();
+	for(x = 0; static_mapi_headers[x] != NULL; x++)
+	{
+		mapi_version = (const int *)static_mapi_headers[x];
+		if(MAPI_MAGIC(*mapi_version) != MAPI_MAGIC_HDR)
+		{
+			ilog(L_MAIN, "Error: linked in module without a MAPI header..giving up");
+			exit(70);
+		}
+		switch (MAPI_VERSION(*mapi_version))
+		{
+		case 1:
+			{
+				const struct mapi_mheader_av1 *mheader =
+					(const struct mapi_mheader_av1 *)mapi_version;
+				if(mheader->mapi_register && (mheader->mapi_register() == -1))
+				{
+					ilog(L_MAIN,
+					     "Error: linked in module failed loading..giving up");
+					exit(70);
+				}
+
+				if(mheader->mapi_command_list)
+				{
+					struct Message **m;
+					for(m = mheader->mapi_command_list; *m; ++m)
+						mod_add_cmd(*m);
+				}
+
+				if(mheader->mapi_hook_list)
+				{
+					mapi_hlist_av1 *m;
+					for(m = mheader->mapi_hook_list; m->hapi_name; ++m)
+						*m->hapi_id = register_hook(m->hapi_name);
+				}
+
+				if(mheader->mapi_hfn_list)
+				{
+					mapi_hfn_list_av1 *m;
+					for(m = mheader->mapi_hfn_list; m->hapi_name; ++m)
+						add_hook(m->hapi_name, m->hookfn);
+
+				}
+				break;
+			}
+		default:
+			{
+				ilog(L_MAIN,
+				     "Error: Unknown MAPI version (%d)in linked in module..giving up",
+				     MAPI_VERSION(*mapi_version));
+				exit(70);
+			}
+		}
+	}
+}
+#endif
