@@ -29,91 +29,96 @@
  * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: s_newconf.c 24906 2008-01-11 20:04:29Z androsyn $
+ * $Id: s_newconf.c 26094 2008-09-19 15:33:46Z androsyn $
  */
 
 #include "stdinc.h"
-#include "ircd_defs.h"
-#include "common.h"
+#include "struct.h"
+#include "dns.h"
 #include "s_conf.h"
 #include "s_newconf.h"
-#include "tools.h"
 #include "client.h"
-#include "memory.h"
 #include "s_serv.h"
 #include "send.h"
 #include "hostmask.h"
 #include "newconf.h"
 #include "hash.h"
-#include "balloc.h"
-#include "event.h"
-#include "sprintf_irc.h"
+#include "ratbox_lib.h"
+#include "match.h"
+#include "ircd.h"
+#include "class.h"
+#include "s_gline.h"
+#include "s_log.h"
 
-dlink_list shared_conf_list;
-dlink_list cluster_conf_list;
-dlink_list oper_conf_list;
-dlink_list hubleaf_conf_list;
-dlink_list server_conf_list;
-dlink_list xline_conf_list;
-dlink_list resv_conf_list;	/* nicks only! */
-static dlink_list nd_list;	/* nick delay */
-dlink_list tgchange_list;
+rb_dlink_list shared_conf_list;
+rb_dlink_list cluster_conf_list;
+rb_dlink_list oper_conf_list;
+rb_dlink_list hubleaf_conf_list;
+rb_dlink_list server_conf_list;
+rb_dlink_list xline_conf_list;
+rb_dlink_list resv_conf_list;	/* nicks only! */
+rb_dlink_list pending_glines;
+rb_dlink_list glines;
+static rb_dlink_list nd_list;	/* nick delay */
+rb_dlink_list tgchange_list;
 
-patricia_tree_t *tgchange_tree;
+rb_patricia_tree_t *tgchange_tree;
 
-static BlockHeap *nd_heap = NULL;
+static rb_bh *nd_heap = NULL;
 
 static void expire_temp_rxlines(void *unused);
 static void expire_nd_entries(void *unused);
+static void expire_glines(void *unused);
 
 void
 init_s_newconf(void)
 {
-	tgchange_tree = New_Patricia(PATRICIA_BITS);
-	nd_heap = BlockHeapCreate(sizeof(struct nd_entry), ND_HEAP_SIZE);
-	eventAddIsh("expire_nd_entries", expire_nd_entries, NULL, 30);
-	eventAddIsh("expire_temp_rxlines", expire_temp_rxlines, NULL, 60);
+	tgchange_tree = rb_new_patricia(PATRICIA_BITS);
+	nd_heap = rb_bh_create(sizeof(struct nd_entry), ND_HEAP_SIZE, "nd_heap");
+	rb_event_addish("expire_nd_entries", expire_nd_entries, NULL, 30);
+	rb_event_addish("expire_temp_rxlines", expire_temp_rxlines, NULL, 60);
+	rb_event_addish("expire_glines", expire_glines, NULL, CLEANUP_GLINES_TIME);
 }
 
 void
 clear_s_newconf(void)
 {
 	struct server_conf *server_p;
-	dlink_node *ptr;
-	dlink_node *next_ptr;
+	rb_dlink_node *ptr;
+	rb_dlink_node *next_ptr;
 
-	DLINK_FOREACH_SAFE(ptr, next_ptr, shared_conf_list.head)
+	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, shared_conf_list.head)
 	{
 		/* ptr here is ptr->data->node */
-		dlinkDelete(ptr, &shared_conf_list);
+		rb_dlinkDelete(ptr, &shared_conf_list);
 		free_remote_conf(ptr->data);
 	}
 
-	DLINK_FOREACH_SAFE(ptr, next_ptr, cluster_conf_list.head)
+	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, cluster_conf_list.head)
 	{
-		dlinkDelete(ptr, &cluster_conf_list);
+		rb_dlinkDelete(ptr, &cluster_conf_list);
 		free_remote_conf(ptr->data);
 	}
 
-	DLINK_FOREACH_SAFE(ptr, next_ptr, hubleaf_conf_list.head)
+	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, hubleaf_conf_list.head)
 	{
-		dlinkDelete(ptr, &hubleaf_conf_list);
+		rb_dlinkDelete(ptr, &hubleaf_conf_list);
 		free_remote_conf(ptr->data);
 	}
 
-	DLINK_FOREACH_SAFE(ptr, next_ptr, oper_conf_list.head)
+	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, oper_conf_list.head)
 	{
 		free_oper_conf(ptr->data);
-		dlinkDestroy(ptr, &oper_conf_list);
+		rb_dlinkDestroy(ptr, &oper_conf_list);
 	}
 
-	DLINK_FOREACH_SAFE(ptr, next_ptr, server_conf_list.head)
+	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, server_conf_list.head)
 	{
 		server_p = ptr->data;
 
 		if(!server_p->servers)
 		{
-			dlinkDelete(ptr, &server_conf_list);
+			rb_dlinkDelete(ptr, &server_conf_list);
 			free_server_conf(ptr->data);
 		}
 		else
@@ -125,29 +130,29 @@ void
 clear_s_newconf_bans(void)
 {
 	struct ConfItem *aconf;
-	dlink_node *ptr, *next_ptr;
+	rb_dlink_node *ptr, *next_ptr;
 
-	DLINK_FOREACH_SAFE(ptr, next_ptr, xline_conf_list.head)
+	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, xline_conf_list.head)
 	{
 		aconf = ptr->data;
 
-		if(aconf->hold)
+		if(aconf->flags & CONF_FLAGS_TEMPORARY)
 			continue;
 
 		free_conf(aconf);
-		dlinkDestroy(ptr, &xline_conf_list);
+		rb_dlinkDestroy(ptr, &xline_conf_list);
 	}
 
-	DLINK_FOREACH_SAFE(ptr, next_ptr, resv_conf_list.head)
+	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, resv_conf_list.head)
 	{
 		aconf = ptr->data;
 
 		/* temporary resv */
-		if(aconf->hold)
+		if(aconf->flags & CONF_FLAGS_TEMPORARY)
 			continue;
 
 		free_conf(aconf);
-		dlinkDestroy(ptr, &resv_conf_list);
+		rb_dlinkDestroy(ptr, &resv_conf_list);
 	}
 
 	clear_resv_hash();
@@ -156,7 +161,7 @@ clear_s_newconf_bans(void)
 struct remote_conf *
 make_remote_conf(void)
 {
-	struct remote_conf *remote_p = MyMalloc(sizeof(struct remote_conf));
+	struct remote_conf *remote_p = rb_malloc(sizeof(struct remote_conf));
 	return remote_p;
 }
 
@@ -167,26 +172,24 @@ free_remote_conf(struct remote_conf *remote_p)
 	if(remote_p == NULL)
 		return;
 
-	MyFree(remote_p->username);
-	MyFree(remote_p->host);
-	MyFree(remote_p->server);
-	MyFree(remote_p);
+	rb_free(remote_p->username);
+	rb_free(remote_p->host);
+	rb_free(remote_p->server);
+	rb_free(remote_p);
 }
 
 int
-find_shared_conf(const char *username, const char *host, 
-		const char *server, int flags)
+find_shared_conf(const char *username, const char *host, const char *server, int flags)
 {
 	struct remote_conf *shared_p;
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 
-	DLINK_FOREACH(ptr, shared_conf_list.head)
+	RB_DLINK_FOREACH(ptr, shared_conf_list.head)
 	{
 		shared_p = ptr->data;
 
 		if(match(shared_p->username, username) &&
-		   match(shared_p->host, host) &&
-		   match(shared_p->server, server))
+		   match(shared_p->host, host) && match(shared_p->server, server))
 		{
 			if(shared_p->flags & flags)
 				return YES;
@@ -199,57 +202,52 @@ find_shared_conf(const char *username, const char *host,
 }
 
 void
-propagate_generic(struct Client *source_p, const char *command,
-		const char *target, int cap, const char *format, ...)
-{
-	char buffer[BUFSIZE];
-	va_list args;
-
-	va_start(args, format);
-	ircvsnprintf(buffer, sizeof(buffer), format, args);
-	va_end(args);
-
-	sendto_match_servs(source_p, target, cap, NOCAPS,
-			"%s %s %s",
-			command, target, buffer);
-	sendto_match_servs(source_p, target, CAP_ENCAP, cap,
-			"ENCAP %s %s %s",
-			target, command, buffer);
-}
-			
-void
-cluster_generic(struct Client *source_p, const char *command,
-		int cltype, int cap, const char *format, ...)
+cluster_generic(struct Client *source_p, const char *command, int cltype, const char *format, ...)
 {
 	char buffer[BUFSIZE];
 	struct remote_conf *shared_p;
 	va_list args;
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 
 	va_start(args, format);
-	ircvsnprintf(buffer, sizeof(buffer), format, args);
+	rb_vsnprintf(buffer, sizeof(buffer), format, args);
 	va_end(args);
 
-	DLINK_FOREACH(ptr, cluster_conf_list.head)
+	RB_DLINK_FOREACH(ptr, cluster_conf_list.head)
 	{
 		shared_p = ptr->data;
 
 		if(!(shared_p->flags & cltype))
 			continue;
 
-		sendto_match_servs(source_p, shared_p->server, cap, NOCAPS,
-				"%s %s %s",
-				command, shared_p->server, buffer);
-		sendto_match_servs(source_p, shared_p->server, CAP_ENCAP, cap,
-				"ENCAP %s %s %s",
-				shared_p->server, command, buffer);
+		sendto_match_servs(source_p, shared_p->server, CAP_ENCAP, NOCAPS,
+				   "ENCAP %s %s %s", shared_p->server, command, buffer);
+	}
+}
+
+static void
+expire_glines(void *unused)
+{
+	struct ConfItem *aconf;
+	rb_dlink_node *ptr, *next_ptr;
+
+	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, glines.head)
+	{
+		aconf = ptr->data;
+
+		/* if gline_time changes, these could end up out of order */
+		if(aconf->hold > rb_current_time())
+			continue;
+
+		delete_one_address_conf(aconf->host, aconf);
+		rb_dlinkDestroy(ptr, &glines);
 	}
 }
 
 struct oper_conf *
 make_oper_conf(void)
 {
-	struct oper_conf *oper_p = MyMalloc(sizeof(struct oper_conf));
+	struct oper_conf *oper_p = rb_malloc(sizeof(struct oper_conf));
 	return oper_p;
 }
 
@@ -260,38 +258,38 @@ free_oper_conf(struct oper_conf *oper_p)
 	if(oper_p == NULL)
 		return;
 
-	MyFree(oper_p->username);
-	MyFree(oper_p->host);
-	MyFree(oper_p->name);
+	rb_free(oper_p->username);
+	rb_free(oper_p->host);
+	rb_free(oper_p->name);
 
 	if(oper_p->passwd)
 	{
 		memset(oper_p->passwd, 0, strlen(oper_p->passwd));
-		MyFree(oper_p->passwd);
+		rb_free(oper_p->passwd);
 	}
 
-#ifdef HAVE_LIBCRYPTO
-	MyFree(oper_p->rsa_pubkey_file);
+#ifdef USE_CHALLENGE
+	rb_free(oper_p->rsa_pubkey_file);
 
 	if(oper_p->rsa_pubkey)
 		RSA_free(oper_p->rsa_pubkey);
 #endif
 
-	MyFree(oper_p);
+	rb_free(oper_p);
 }
 
 struct oper_conf *
 find_oper_conf(const char *username, const char *host, const char *locip, const char *name)
 {
 	struct oper_conf *oper_p;
-	struct irc_sockaddr_storage ip, cip;
-	char addr[HOSTLEN+1];
+	struct rb_sockaddr_storage ip, cip;
+	char addr[HOSTLEN + 1];
 	int bits, cbits;
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 
 	parse_netmask(locip, (struct sockaddr *)&cip, &cbits);
 
-	DLINK_FOREACH(ptr, oper_conf_list.head)
+	RB_DLINK_FOREACH(ptr, oper_conf_list.head)
 	{
 		oper_p = ptr->data;
 
@@ -299,12 +297,13 @@ find_oper_conf(const char *username, const char *host, const char *locip, const
 		if(irccmp(oper_p->name, name) || !match(oper_p->username, username))
 			continue;
 
-		strlcpy(addr, oper_p->host, sizeof(addr));
+		rb_strlcpy(addr, oper_p->host, sizeof(addr));
 
 		if(parse_netmask(addr, (struct sockaddr *)&ip, &bits) != HM_HOST)
 		{
-			if(ip.ss_family == cip.ss_family &&
-			   comp_with_mask_sock((struct sockaddr *)&ip, (struct sockaddr *)&cip, bits))
+			if(GET_SS_FAMILY(&ip) == GET_SS_FAMILY(&cip) &&
+			   comp_with_mask_sock((struct sockaddr *)&ip, (struct sockaddr *)&cip,
+					       bits))
 				return oper_p;
 		}
 
@@ -325,24 +324,24 @@ struct oper_flags
 	char has;
 	char hasnt;
 };
-static struct oper_flags oper_flagtable[] =
-{
-	{ OPER_GLINE,		'G', 'g' },
-	{ OPER_KLINE,		'K', 'k' },
-	{ OPER_XLINE,		'X', 'x' },
-	{ OPER_GLOBKILL,	'O', 'o' },
-	{ OPER_LOCKILL,		'C', 'c' },
-	{ OPER_REMOTE,		'R', 'r' },
-	{ OPER_UNKLINE,		'U', 'u' },
-	{ OPER_REHASH,		'H', 'h' },
-	{ OPER_DIE,		'D', 'd' },
-	{ OPER_ADMIN,		'A', 'a' },
-	{ OPER_NICKS,		'N', 'n' },
-	{ OPER_OPERWALL,	'L', 'l' },
-	{ OPER_SPY,		'S', 's' },
-	{ OPER_INVIS,		'P', 'p' },
-	{ OPER_REMOTEBAN,	'B', 'b' },
-	{ 0,			'\0', '\0' }
+static struct oper_flags oper_flagtable[] = {
+	{OPER_GLINE, 'G', 'g'},
+	{OPER_KLINE, 'K', 'k'},
+	{OPER_XLINE, 'X', 'x'},
+	{OPER_RESV, 'Q', 'q'},
+	{OPER_GLOBKILL, 'O', 'o'},
+	{OPER_LOCKILL, 'C', 'c'},
+	{OPER_REMOTE, 'R', 'r'},
+	{OPER_UNKLINE, 'U', 'u'},
+	{OPER_REHASH, 'H', 'h'},
+	{OPER_DIE, 'D', 'd'},
+	{OPER_ADMIN, 'A', 'a'},
+	{OPER_NICKS, 'N', 'n'},
+	{OPER_OPERWALL, 'L', 'l'},
+	{OPER_SPY, 'S', 's'},
+	{OPER_INVIS, 'P', 'p'},
+	{OPER_REMOTEBAN, 'B', 'b'},
+	{0, '\0', '\0'}
 };
 
 const char *
@@ -370,8 +369,8 @@ get_oper_privs(int flags)
 struct server_conf *
 make_server_conf(void)
 {
-	struct server_conf *server_p = MyMalloc(sizeof(struct server_conf));
-	server_p->ipnum.ss_family = AF_INET;
+	struct server_conf *server_p = rb_malloc(sizeof(struct server_conf));
+	SET_SS_FAMILY(&server_p->ipnum, AF_INET);
 	return server_p;
 }
 
@@ -385,21 +384,21 @@ free_server_conf(struct server_conf *server_p)
 	if(!EmptyString(server_p->passwd))
 	{
 		memset(server_p->passwd, 0, strlen(server_p->passwd));
-		MyFree(server_p->passwd);
+		rb_free(server_p->passwd);
 	}
 
 	if(!EmptyString(server_p->spasswd))
 	{
 		memset(server_p->spasswd, 0, strlen(server_p->spasswd));
-		MyFree(server_p->spasswd);
+		rb_free(server_p->spasswd);
 	}
 
-	delete_adns_queries(server_p->dns_query);
+	cancel_lookup(server_p->dns_query);
 
-	MyFree(server_p->name);
-	MyFree(server_p->host);
-	MyFree(server_p->class_name);
-	MyFree(server_p);
+	rb_free(server_p->name);
+	rb_free(server_p->host);
+	rb_free(server_p->class_name);
+	rb_free(server_p);
 }
 
 /*
@@ -413,35 +412,14 @@ free_server_conf(struct server_conf *server_p)
  * if successful save hp in the conf item it was called with
  */
 static void
-conf_dns_callback(void *vptr, adns_answer * reply)
+conf_dns_callback(const char *result, int status, int aftype, void *data)
 {
-	struct server_conf *server_p = (struct server_conf *) vptr;
+	struct server_conf *server_p = data;
 
-	if(reply && reply->status == adns_s_ok)
-	{
-#ifdef IPV6
-		if(reply->type ==  adns_r_addr6)
-		{
-			struct sockaddr_in6 *in6 = (struct sockaddr_in6 *)&server_p->ipnum;
-			SET_SS_LEN(server_p->ipnum, sizeof(struct sockaddr_in6));
-			in6->sin6_family = AF_INET6;
-			in6->sin6_port = 0;
-			memcpy(&in6->sin6_addr, &reply->rrs.addr->addr.inet6.sin6_addr, sizeof(struct in6_addr));
-		}
-		else
-#endif
-		{
-			struct sockaddr_in *in = (struct sockaddr_in *)&server_p->ipnum;
-			SET_SS_LEN(server_p->ipnum, sizeof(struct sockaddr_in));
-			in->sin_family = AF_INET;
-			in->sin_port = 0;
-			in->sin_addr.s_addr = reply->rrs.addr->addr.inet.sin_addr.s_addr;
-		}
-		MyFree(reply);
-	}
+	if(status == 1)
+		rb_inet_pton_sock(result, (struct sockaddr *)&server_p->ipnum);
 
-	MyFree(server_p->dns_query);
-	server_p->dns_query = NULL;
+	server_p->dns_query = 0;
 }
 
 
@@ -455,32 +433,30 @@ add_server_conf(struct server_conf *server_p)
 
 	if(server_p->class == default_class)
 	{
-		conf_report_error("Warning connect::class invalid for %s",
-				server_p->name);
+		conf_report_error("Warning connect::class invalid for %s", server_p->name);
 
-		MyFree(server_p->class_name);
-		DupString(server_p->class_name, "default");
+		rb_free(server_p->class_name);
+		server_p->class_name = rb_strdup("default");
 	}
 
-	if(strchr(server_p->host, '*') || strchr(server_p->host, '?'))
+	if(strpbrk(server_p->host, "?*") != NULL)
 		return;
 
-	if(inetpton_sock(server_p->host, (struct sockaddr *)&server_p->ipnum) > 0)
+	if(rb_inet_pton_sock(server_p->host, (struct sockaddr *)&server_p->ipnum) > 0)
 		return;
 
-	server_p->dns_query = MyMalloc(sizeof(struct DNSQuery));
-	server_p->dns_query->ptr = server_p;
-	server_p->dns_query->callback = conf_dns_callback;
-	adns_gethost(server_p->host, server_p->ipnum.ss_family, server_p->dns_query);
+	server_p->dns_query =
+		lookup_hostname(server_p->host, GET_SS_FAMILY(&server_p->ipnum), conf_dns_callback,
+				server_p);
 }
 
 struct server_conf *
 find_server_conf(const char *name)
 {
 	struct server_conf *server_p;
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 
-	DLINK_FOREACH(ptr, server_conf_list.head)
+	RB_DLINK_FOREACH(ptr, server_conf_list.head)
 	{
 		server_p = ptr->data;
 
@@ -531,7 +507,7 @@ detach_server_conf(struct Client *client_p)
 		if(MaxUsers(server_p->class) < 0 && CurrUsers(server_p->class) <= 0)
 			free_class(server_p->class);
 
-		dlinkDelete(&server_p->node, &server_conf_list);
+		rb_dlinkDelete(&server_p->node, &server_conf_list);
 		free_server_conf(server_p);
 	}
 }
@@ -549,25 +525,24 @@ set_server_conf_autoconn(struct Client *source_p, char *name, int newval)
 			server_p->flags &= ~SERVER_AUTOCONN;
 
 		sendto_realops_flags(UMODE_ALL, L_ALL,
-				"%s has changed AUTOCONN for %s to %i",
-				get_oper_name(source_p), name, newval);
+				     "%s has changed AUTOCONN for %s to %i",
+				     get_oper_name(source_p), name, newval);
 	}
 	else
-		sendto_one(source_p, ":%s NOTICE %s :Can't find %s",
-				me.name, source_p->name, name);
+		sendto_one_notice(source_p, ":Can't find %s", name);
 }
 
 struct ConfItem *
 find_xline(const char *gecos, int counter)
 {
 	struct ConfItem *aconf;
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 
-	DLINK_FOREACH(ptr, xline_conf_list.head)
+	RB_DLINK_FOREACH(ptr, xline_conf_list.head)
 	{
 		aconf = ptr->data;
 
-		if(match_esc(aconf->name, gecos))
+		if(match_esc(aconf->host, gecos))
 		{
 			if(counter)
 				aconf->port++;
@@ -582,13 +557,13 @@ struct ConfItem *
 find_xline_mask(const char *gecos)
 {
 	struct ConfItem *aconf;
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 
-	DLINK_FOREACH(ptr, xline_conf_list.head)
+	RB_DLINK_FOREACH(ptr, xline_conf_list.head)
 	{
 		aconf = ptr->data;
 
-		if(!irccmp(aconf->name, gecos))
+		if(!irccmp(aconf->host, gecos))
 			return aconf;
 	}
 
@@ -599,13 +574,13 @@ struct ConfItem *
 find_nick_resv(const char *name)
 {
 	struct ConfItem *aconf;
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 
-	DLINK_FOREACH(ptr, resv_conf_list.head)
+	RB_DLINK_FOREACH(ptr, resv_conf_list.head)
 	{
 		aconf = ptr->data;
 
-		if(match_esc(aconf->name, name))
+		if(match_esc(aconf->host, name))
 		{
 			aconf->port++;
 			return aconf;
@@ -619,13 +594,13 @@ struct ConfItem *
 find_nick_resv_mask(const char *name)
 {
 	struct ConfItem *aconf;
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 
-	DLINK_FOREACH(ptr, resv_conf_list.head)
+	RB_DLINK_FOREACH(ptr, resv_conf_list.head)
 	{
 		aconf = ptr->data;
 
-		if(!irccmp(aconf->name, name))
+		if(!irccmp(aconf->host, name))
 			return aconf;
 	}
 
@@ -649,7 +624,7 @@ clean_resv_nick(const char *nick)
 	if(*nick == '-' || IsDigit(*nick))
 		return 0;
 
-	while ((tmpch = *nick++))
+	while((tmpch = *nick++))
 	{
 		if(tmpch == '?' || tmpch == '@' || tmpch == '#')
 			q++;
@@ -725,61 +700,59 @@ valid_temp_time(const char *p)
 	if(result > (60 * 24 * 7 * 52))
 		result = (60 * 24 * 7 * 52);
 
-	return(result * 60);
+	return (result * 60);
 }
 
 static void
 expire_temp_rxlines(void *unused)
 {
 	struct ConfItem *aconf;
-	dlink_node *ptr;
-	dlink_node *next_ptr;
+	rb_dlink_node *ptr;
+	rb_dlink_node *next_ptr;
 	int i;
 
 	HASH_WALK_SAFE(i, R_MAX, ptr, next_ptr, resvTable)
 	{
 		aconf = ptr->data;
 
-		if(aconf->hold && aconf->hold <= CurrentTime)
+		if((aconf->flags & CONF_FLAGS_TEMPORARY) && aconf->hold <= rb_current_time())
 		{
 			if(ConfigFileEntry.tkline_expire_notices)
 				sendto_realops_flags(UMODE_ALL, L_ALL,
-						"Temporary RESV for [%s] expired",
-						aconf->name);
+						     "Temporary RESV for [%s] expired",
+						     aconf->host);
 
 			free_conf(aconf);
-			dlinkDestroy(ptr, &resvTable[i]);
+			rb_dlinkDestroy(ptr, &resvTable[i]);
 		}
 	}
-	HASH_WALK_END
-
-	DLINK_FOREACH_SAFE(ptr, next_ptr, resv_conf_list.head)
+	HASH_WALK_END RB_DLINK_FOREACH_SAFE(ptr, next_ptr, resv_conf_list.head)
 	{
 		aconf = ptr->data;
 
-		if(aconf->hold && aconf->hold <= CurrentTime)
+		if((aconf->flags & CONF_FLAGS_TEMPORARY) && aconf->hold <= rb_current_time())
 		{
 			if(ConfigFileEntry.tkline_expire_notices)
 				sendto_realops_flags(UMODE_ALL, L_ALL,
-						"Temporary RESV for [%s] expired",
-						aconf->name);
+						     "Temporary RESV for [%s] expired",
+						     aconf->host);
 			free_conf(aconf);
-			dlinkDestroy(ptr, &resv_conf_list);
+			rb_dlinkDestroy(ptr, &resv_conf_list);
 		}
 	}
 
-	DLINK_FOREACH_SAFE(ptr, next_ptr, xline_conf_list.head)
+	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, xline_conf_list.head)
 	{
 		aconf = ptr->data;
 
-		if(aconf->hold && aconf->hold <= CurrentTime)
+		if((aconf->flags & CONF_FLAGS_TEMPORARY) && aconf->hold <= rb_current_time())
 		{
 			if(ConfigFileEntry.tkline_expire_notices)
 				sendto_realops_flags(UMODE_ALL, L_ALL,
-						"Temporary X-line for [%s] expired",
-						aconf->name);
+						     "Temporary X-line for [%s] expired",
+						     aconf->host);
 			free_conf(aconf);
-			dlinkDestroy(ptr, &xline_conf_list);
+			rb_dlinkDestroy(ptr, &xline_conf_list);
 		}
 	}
 }
@@ -787,7 +760,7 @@ expire_temp_rxlines(void *unused)
 unsigned long
 get_nd_count(void)
 {
-	return(dlink_list_length(&nd_list));
+	return (rb_dlink_list_length(&nd_list));
 }
 
 
@@ -799,39 +772,39 @@ add_nd_entry(const char *name)
 	if(hash_find_nd(name) != NULL)
 		return;
 
-	nd = BlockHeapAlloc(nd_heap);
-	
-	strlcpy(nd->name, name, sizeof(nd->name));
-	nd->expire = CurrentTime + ConfigFileEntry.nick_delay;
+	nd = rb_bh_alloc(nd_heap);
+
+	rb_strlcpy(nd->name, name, sizeof(nd->name));
+	nd->expire = rb_current_time() + ConfigFileEntry.nick_delay;
 
 	/* this list is ordered */
-	dlinkAddTail(nd, &nd->lnode, &nd_list);
+	rb_dlinkAddTail(nd, &nd->lnode, &nd_list);
 	add_to_nd_hash(name, nd);
 }
 
 void
 free_nd_entry(struct nd_entry *nd)
 {
-	dlinkDelete(&nd->lnode, &nd_list);
-	dlinkDelete(&nd->hnode, &ndTable[nd->hashv]);
-	BlockHeapFree(nd_heap, nd);
+	rb_dlinkDelete(&nd->lnode, &nd_list);
+	rb_dlinkDelete(&nd->hnode, &ndTable[nd->hashv]);
+	rb_bh_free(nd_heap, nd);
 }
 
 void
 expire_nd_entries(void *unused)
 {
 	struct nd_entry *nd;
-	dlink_node *ptr;
-	dlink_node *next_ptr;
+	rb_dlink_node *ptr;
+	rb_dlink_node *next_ptr;
 
-	DLINK_FOREACH_SAFE(ptr, next_ptr, nd_list.head)
+	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, nd_list.head)
 	{
 		nd = ptr->data;
 
 		/* this list is ordered - we can stop when we hit the first
 		 * entry that doesnt expire..
 		 */
-		if(nd->expire > CurrentTime)
+		if(nd->expire > rb_current_time())
 			return;
 
 		free_nd_entry(nd);
@@ -842,31 +815,30 @@ void
 add_tgchange(const char *host)
 {
 	tgchange *target;
-	patricia_node_t *pnode;
+	rb_patricia_node_t *pnode;
 
 	if(find_tgchange(host))
 		return;
 
-	target = MyMalloc(sizeof(tgchange));
+	target = rb_malloc(sizeof(tgchange));
 	pnode = make_and_lookup(tgchange_tree, host);
 
 	pnode->data = target;
 	target->pnode = pnode;
 
-	DupString(target->ip, host);
-	target->expiry = CurrentTime + (60*60*12);
+	target->ip = rb_strdup(host);
+	target->expiry = rb_current_time() + (60 * 60 * 12);
 
-	dlinkAdd(target, &target->node, &tgchange_list);
+	rb_dlinkAdd(target, &target->node, &tgchange_list);
 }
 
 tgchange *
 find_tgchange(const char *host)
 {
-	patricia_node_t *pnode;
+	rb_patricia_node_t *pnode;
 
-	if((pnode = match_exact_string(tgchange_tree, host)))
+	if((pnode = rb_match_exact_string(tgchange_tree, host)))
 		return pnode->data;
 
 	return NULL;
 }
-
