@@ -18,27 +18,24 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
  *  USA
  *
- *  $Id: s_user.c 24001 2007-07-17 15:07:43Z androsyn $
+ *  $Id: s_user.c 26688 2009-10-07 20:43:14Z leeh $
  */
 
 #include "stdinc.h"
-#include "tools.h"
+#include "struct.h"
 #include "s_user.h"
 #include "channel.h"
 #include "class.h"
 #include "client.h"
-#include "common.h"
 #include "hash.h"
-#include "irc_string.h"
-#include "sprintf_irc.h"
+#include "match.h"
 #include "ircd.h"
 #include "listener.h"
-#include "msg.h"
+#include "parse.h"
 #include "numeric.h"
-#include "commio.h"
 #include "s_conf.h"
 #include "s_newconf.h"
 #include "s_log.h"
@@ -48,7 +45,6 @@
 #include "send.h"
 #include "supported.h"
 #include "whowas.h"
-#include "memory.h"
 #include "packet.h"
 #include "reject.h"
 #include "cache.h"
@@ -183,80 +179,51 @@ int user_modes_from_c_to_bitmask[] = {
 int
 show_lusers(struct Client *source_p)
 {
+	SetCork(source_p);
 	sendto_one_numeric(source_p, RPL_LUSERCLIENT, form_str(RPL_LUSERCLIENT),
 			   (Count.total - Count.invisi),
-			   Count.invisi, dlink_list_length(&global_serv_list));
+			   Count.invisi, rb_dlink_list_length(&global_serv_list));
 
 	if(Count.oper > 0)
-		sendto_one_numeric(source_p, RPL_LUSEROP, 
-				   form_str(RPL_LUSEROP), Count.oper);
+		sendto_one_numeric(source_p, RPL_LUSEROP, form_str(RPL_LUSEROP), Count.oper);
 
-	if(dlink_list_length(&unknown_list) > 0)
-		sendto_one_numeric(source_p, RPL_LUSERUNKNOWN, 
-				   form_str(RPL_LUSERUNKNOWN),
-				   dlink_list_length(&unknown_list));
+	if(rb_dlink_list_length(&unknown_list) > 0)
+		sendto_one_numeric(source_p, RPL_LUSERUNKNOWN,
+				   form_str(RPL_LUSERUNKNOWN), rb_dlink_list_length(&unknown_list));
 
-	if(dlink_list_length(&global_channel_list) > 0)
-		sendto_one_numeric(source_p, RPL_LUSERCHANNELS, 
+	if(rb_dlink_list_length(&global_channel_list) > 0)
+		sendto_one_numeric(source_p, RPL_LUSERCHANNELS,
 				   form_str(RPL_LUSERCHANNELS),
-				   dlink_list_length(&global_channel_list));
+				   rb_dlink_list_length(&global_channel_list));
 
 	sendto_one_numeric(source_p, RPL_LUSERME, form_str(RPL_LUSERME),
-			   dlink_list_length(&lclient_list),
-			   dlink_list_length(&serv_list));
+			   rb_dlink_list_length(&lclient_list), rb_dlink_list_length(&serv_list));
 
-	sendto_one_numeric(source_p, RPL_LOCALUSERS, 
+	sendto_one_numeric(source_p, RPL_LOCALUSERS,
 			   form_str(RPL_LOCALUSERS),
-			   dlink_list_length(&lclient_list),
-			   Count.max_loc,
-			   dlink_list_length(&lclient_list),
-			   Count.max_loc);
+			   rb_dlink_list_length(&lclient_list),
+			   Count.max_loc, rb_dlink_list_length(&lclient_list), Count.max_loc);
 
 	sendto_one_numeric(source_p, RPL_GLOBALUSERS, form_str(RPL_GLOBALUSERS),
-			   Count.total, Count.max_tot,
-			   Count.total, Count.max_tot);
+			   Count.total, Count.max_tot, Count.total, Count.max_tot);
 
 	sendto_one_numeric(source_p, RPL_STATSCONN,
 			   form_str(RPL_STATSCONN),
-			   MaxConnectionCount, MaxClientCount, 
-			   Count.totalrestartcount);
+			   MaxConnectionCount, MaxClientCount, Count.totalrestartcount);
 
-	if(dlink_list_length(&lclient_list) > (unsigned long)MaxClientCount)
-		MaxClientCount = dlink_list_length(&lclient_list);
+	if(rb_dlink_list_length(&lclient_list) > (unsigned long)MaxClientCount)
+		MaxClientCount = rb_dlink_list_length(&lclient_list);
 
-	if((dlink_list_length(&lclient_list) + dlink_list_length(&serv_list)) >
+	if((rb_dlink_list_length(&lclient_list) + rb_dlink_list_length(&serv_list)) >
 	   (unsigned long)MaxConnectionCount)
-		MaxConnectionCount = dlink_list_length(&lclient_list) + 
-					dlink_list_length(&serv_list);
+		MaxConnectionCount = rb_dlink_list_length(&lclient_list) +
+			rb_dlink_list_length(&serv_list);
+	ClearCork(source_p);
+	send_pop_queue(source_p);
 
 	return 0;
 }
 
-/*
- * show_isupport
- *
- * inputs	- pointer to client
- * output	- 
- * side effects	- display to client what we support (for them)
- */
-void
-show_isupport(struct Client *source_p)
-{
-	char isupportbuffer[512];
-
-	ircsprintf(isupportbuffer, FEATURES, FEATURESVALUES);
-	sendto_one_numeric(source_p, RPL_ISUPPORT, form_str(RPL_ISUPPORT), isupportbuffer);
-
-	ircsprintf(isupportbuffer, FEATURES2, FEATURES2VALUES);
-	sendto_one_numeric(source_p, RPL_ISUPPORT, form_str(RPL_ISUPPORT), isupportbuffer);
-
-	ircsprintf(isupportbuffer, FEATURES3, FEATURES3VALUES);
-	sendto_one_numeric(source_p, RPL_ISUPPORT, form_str(RPL_ISUPPORT), isupportbuffer);
-
-	return;
-}
-
-
 /*
 ** register_local_user
 **      This function is called when both NICK and USER messages
@@ -285,10 +252,9 @@ int
 register_local_user(struct Client *client_p, struct Client *source_p, const char *username)
 {
 	struct ConfItem *aconf;
-	struct User *user = source_p->user;
 	char tmpstr2[IRCD_BUFSIZE];
 	char ipaddr[HOSTIPLEN];
-	char myusername[USERLEN+1];
+	char myusername[USERLEN + 1];
 	int status;
 
 	s_assert(NULL != source_p);
@@ -305,23 +271,23 @@ register_local_user(struct Client *client_p, struct Client *source_p, const char
 	{
 		if(!(source_p->flags & FLAGS_PINGSENT) && source_p->localClient->random_ping == 0)
 		{
-			source_p->localClient->random_ping = (unsigned long) (rand() * rand()) << 1;
-			sendto_one(source_p, "PING :%08lX",
-				   (unsigned long) source_p->localClient->random_ping);
+			source_p->localClient->random_ping = (uint32_t)(rand() * rand()) << 1;
+			sendto_one(source_p, "PING :%08X",
+				   (unsigned int)source_p->localClient->random_ping);
 			source_p->flags |= FLAGS_PINGSENT;
 			return -1;
 		}
-		if(!(source_p->flags2 & FLAGS2_PING_COOKIE))
+		if(!(source_p->flags & FLAGS_PING_COOKIE))
 		{
 			return -1;
 		}
 	}
 
 	/* hasnt finished client cap negotiation */
-	if(source_p->flags2 & FLAGS2_CLICAP)
+	if(source_p->flags & FLAGS_CLICAP)
 		return -1;
 
-	client_p->localClient->last = CurrentTime;
+	client_p->localClient->last = rb_current_time();
 	/* Straight up the maximum rate of flooding... */
 	source_p->localClient->allow_read = MAX_FLOOD_BURST;
 
@@ -349,18 +315,17 @@ register_local_user(struct Client *client_p, struct Client *source_p, const char
 
 	if(!valid_hostname(source_p->host))
 	{
-		sendto_one(source_p,
-			   ":%s NOTICE %s :*** Notice -- You have an illegal character in your hostname",
-			   me.name, source_p->name);
+		sendto_one_notice(source_p, ":*** Notice -- You have an invalid hostname");
 
-		strlcpy(source_p->host, source_p->sockhost, sizeof(source_p->host));
+		rb_strlcpy(source_p->host, source_p->sockhost, sizeof(source_p->host));
 
-#ifdef IPV6
-		if(ConfigFileEntry.dot_in_ip6_addr == 1)
-			strlcat(source_p->host, ".", sizeof(source_p->host));
+#ifdef RB_IPV6
+		if(ConfigFileEntry.dot_in_ip6_addr == 1
+		   && (GET_SS_FAMILY(&source_p->localClient->ip) == AF_INET6))
+			rb_strlcat(source_p->host, ".", sizeof(source_p->host));
 #endif
- 	}
- 
+	}
+
 
 	aconf = source_p->localClient->att_conf;
 
@@ -370,6 +335,15 @@ register_local_user(struct Client *client_p, struct Client *source_p, const char
 		return (CLIENT_EXITED);
 	}
 
+	if(IsConfSSLNeeded(aconf) && !IsSSL(source_p))
+	{
+		ServerStats.is_ref++;
+		sendto_one_notice(source_p,
+				  ":*** Notice -- You need to use SSL/TLS to use this server");
+		exit_client(client_p, source_p, &me, "Use SSL/TLS");
+		return (CLIENT_EXITED);
+	}
+
 	if(!IsGotId(source_p))
 	{
 		const char *p;
@@ -377,23 +351,22 @@ register_local_user(struct Client *client_p, struct Client *source_p, const char
 
 		if(IsNeedIdentd(aconf))
 		{
-			ServerStats->is_ref++;
-			sendto_one(source_p,
-				   ":%s NOTICE %s :*** Notice -- You need to install identd to use this server",
-				   me.name, client_p->name);
+			ServerStats.is_ref++;
+			sendto_one_notice(source_p,
+					  ":*** Notice -- You need to install identd to use this server");
 			exit_client(client_p, source_p, &me, "Install identd");
 			return (CLIENT_EXITED);
 		}
 
 		/* dont replace username if its supposed to be spoofed --fl */
-		if(!IsConfDoSpoofIp(aconf) || !strchr(aconf->name, '@'))
+		if(!IsConfDoSpoofIp(aconf) || !strchr(aconf->info.name, '@'))
 		{
 			p = username;
 
 			if(!IsNoTilde(aconf))
 				source_p->username[i++] = '~';
 
-			while (*p && i < USERLEN)
+			while(*p && i < USERLEN)
 			{
 				if(*p != '[')
 					source_p->username[i++] = *p;
@@ -412,13 +385,13 @@ register_local_user(struct Client *client_p, struct Client *source_p, const char
 		if(EmptyString(source_p->localClient->passwd))
 			encr = "";
 		else if(IsConfEncrypted(aconf))
-			encr = crypt(source_p->localClient->passwd, aconf->passwd);
+			encr = rb_crypt(source_p->localClient->passwd, aconf->passwd);
 		else
 			encr = source_p->localClient->passwd;
 
 		if(strcmp(encr, aconf->passwd))
 		{
-			ServerStats->is_ref++;
+			ServerStats.is_ref++;
 			sendto_one(source_p, form_str(ERR_PASSWDMISMATCH), me.name, source_p->name);
 			exit_client(client_p, source_p, &me, "Bad Password");
 			return (CLIENT_EXITED);
@@ -428,7 +401,7 @@ register_local_user(struct Client *client_p, struct Client *source_p, const char
 	if(source_p->localClient->passwd)
 	{
 		memset(source_p->localClient->passwd, 0, strlen(source_p->localClient->passwd));
-		MyFree(source_p->localClient->passwd);
+		rb_free(source_p->localClient->passwd);
 		source_p->localClient->passwd = NULL;
 	}
 
@@ -444,15 +417,16 @@ register_local_user(struct Client *client_p, struct Client *source_p, const char
 	 *   -Taner
 	 */
 	/* Except "F:" clients */
-	if(((dlink_list_length(&lclient_list) + 1) >= 
-	   ((unsigned long)GlobalSetOptions.maxclients + MAX_BUFFER) ||
-           (dlink_list_length(&lclient_list) + 1) >= 
+	if(((rb_dlink_list_length(&lclient_list) + 1) >=
+	    ((unsigned long)GlobalSetOptions.maxclients + MAX_BUFFER) ||
+	    (rb_dlink_list_length(&lclient_list) + 1) >=
 	    ((unsigned long)GlobalSetOptions.maxclients - 5)) && !(IsExemptLimits(source_p)))
 	{
 		sendto_realops_flags(UMODE_FULL, L_ALL,
-				     "Too many clients, rejecting %s[%s].", source_p->name, source_p->host);
+				     "Too many clients, rejecting %s[%s].", source_p->name,
+				     source_p->host);
 
-		ServerStats->is_ref++;
+		ServerStats.is_ref++;
 		exit_client(client_p, source_p, &me, "Sorry, server is full - try later");
 		return (CLIENT_EXITED);
 	}
@@ -464,8 +438,8 @@ register_local_user(struct Client *client_p, struct Client *source_p, const char
 		sendto_realops_flags(UMODE_REJ, L_ALL,
 				     "Invalid username: %s (%s@%s)",
 				     source_p->name, source_p->username, source_p->host);
-		ServerStats->is_ref++;
-		ircsprintf(tmpstr2, "Invalid username [%s]", source_p->username);
+		ServerStats.is_ref++;
+		rb_sprintf(tmpstr2, "Invalid username [%s]", source_p->username);
 		exit_client(client_p, source_p, &me, tmpstr2);
 		return (CLIENT_EXITED);
 	}
@@ -473,10 +447,9 @@ register_local_user(struct Client *client_p, struct Client *source_p, const char
 	/* end of valid user name check */
 
 	/* kline exemption extends to xline too */
-	if(!IsExemptKline(source_p) &&
-	   find_xline(source_p->info, 1) != NULL)
+	if(!IsExemptKline(source_p) && find_xline(source_p->info, 1) != NULL)
 	{
-		ServerStats->is_ref++;
+		ServerStats.is_ref++;
 		add_reject(source_p);
 		exit_client(client_p, source_p, &me, "Bad user info");
 		return CLIENT_EXITED;
@@ -485,7 +458,7 @@ register_local_user(struct Client *client_p, struct Client *source_p, const char
 	if(IsAnyDead(client_p))
 		return CLIENT_EXITED;
 
-	inetntop_sock((struct sockaddr *)&source_p->localClient->ip, ipaddr, sizeof(ipaddr));
+	rb_inet_ntop_sock((struct sockaddr *)&source_p->localClient->ip, ipaddr, sizeof(ipaddr));
 
 	sendto_realops_flags(UMODE_CCONN, L_ALL,
 			     "Client connecting: %s (%s@%s) [%s] {%s} [%s]",
@@ -494,23 +467,23 @@ register_local_user(struct Client *client_p, struct Client *source_p, const char
 			     get_client_class(source_p), source_p->info);
 
 	sendto_realops_flags(UMODE_CCONNEXT, L_ALL,
-			"CLICONN %s %s %s %s %s %s 0 %s",
-			source_p->name, source_p->username, source_p->host,
-			show_ip(NULL, source_p) ? ipaddr : "255.255.255.255",
-			get_client_class(source_p),
-			/* mirc can sometimes send ips here */
-			show_ip(NULL, source_p) ? source_p->localClient->fullcaps : 
-			 "<hidden> <hidden>", 
-			source_p->info);
+			     "CLICONN %s %s %s %s %s %s 0 %s",
+			     source_p->name, source_p->username, source_p->host,
+			     show_ip(NULL, source_p) ? ipaddr : "255.255.255.255",
+			     get_client_class(source_p),
+			     /* mirc sometimes puts IPs in here */
+			     show_ip(NULL,
+				     source_p) ? source_p->
+			     localClient->fullcaps : "<hidden> <hidden>", source_p->info);
 
 	/* If they have died in send_* don't do anything. */
 	if(IsAnyDead(source_p))
 		return CLIENT_EXITED;
 
-	add_to_hostname_hash(source_p->host, source_p);
+	add_to_hash(HASH_HOSTNAME, source_p->host, source_p);
 
 	strcpy(source_p->id, generate_uid());
-	add_to_id_hash(source_p->id, source_p);
+	add_to_hash(HASH_ID, source_p->id, source_p);
 
 	if(ConfigFileEntry.default_invisible)
 	{
@@ -519,28 +492,26 @@ register_local_user(struct Client *client_p, struct Client *source_p, const char
 	}
 
 	s_assert(!IsClient(source_p));
-	del_unknown_ip(source_p);
-	dlinkMoveNode(&source_p->localClient->tnode, &unknown_list, &lclient_list);
+	rb_dlinkMoveNode(&source_p->localClient->tnode, &unknown_list, &lclient_list);
 	SetClient(source_p);
 
-	/* XXX source_p->servptr is &me, since local client */
-	/* NO SHIT!^*($^, so lets not use the below. --fl */
-	/* source_p->servptr = find_server(NULL, user->server); */
 	source_p->servptr = &me;
 
-	dlinkAdd(source_p, &source_p->lnode, &source_p->servptr->serv->users);
+	rb_dlinkAdd(source_p, &source_p->lnode, &source_p->servptr->serv->users);
 	/* Increment our total user count here */
 	if(++Count.total > Count.max_tot)
 		Count.max_tot = Count.total;
 	source_p->localClient->allow_read = MAX_FLOOD_BURST;
 
+	source_p->localClient->firsttime = rb_current_time();
+
 	Count.totalrestartcount++;
 
 	s_assert(source_p->localClient != NULL);
 
-	if(dlink_list_length(&lclient_list) > (unsigned long)Count.max_loc)
+	if(rb_dlink_list_length(&lclient_list) > (unsigned long)Count.max_loc)
 	{
-		Count.max_loc = dlink_list_length(&lclient_list);
+		Count.max_loc = rb_dlink_list_length(&lclient_list);
 		if(!(Count.max_loc % 10))
 			sendto_realops_flags(UMODE_ALL, L_ALL,
 					     "New Max Local Clients: %d", Count.max_loc);
@@ -550,9 +521,10 @@ register_local_user(struct Client *client_p, struct Client *source_p, const char
 	if(find_tgchange(source_p->sockhost))
 		USED_TARGETS(source_p) = 6;
 
+	inc_global_cidr_count(client_p);
 	monitor_signon(source_p);
 	user_welcome(source_p);
-	introduce_client(client_p, source_p, user, source_p->name);
+	introduce_client(client_p, source_p);
 	return 0;
 }
 
@@ -566,7 +538,7 @@ register_local_user(struct Client *client_p, struct Client *source_p, const char
  *		  from a remote connect.
  */
 void
-introduce_client(struct Client *client_p, struct Client *source_p, struct User *user, const char *nick)
+introduce_client(struct Client *client_p, struct Client *source_p)
 {
 	static char ubuf[12];
 
@@ -586,128 +558,31 @@ introduce_client(struct Client *client_p, struct Client *source_p, struct User *
 	 */
 	if(has_id(source_p))
 	{
-		char sockhost[HOSTLEN];
-		if(source_p->sockhost[0] == ':')
-		{
-			sockhost[0] = '0';
-			sockhost[1] = '\0';
-			strlcat(sockhost, source_p->sockhost, sizeof(sockhost));
-		} else
-			strcpy(sockhost, source_p->sockhost);
-		
 		sendto_server(client_p, NULL, CAP_TS6, NOCAPS,
 			      ":%s UID %s %d %ld %s %s %s %s %s :%s",
-			      source_p->servptr->id, nick,
+			      source_p->servptr->id, source_p->name,
 			      source_p->hopcount + 1,
-			      (long) source_p->tsinfo, ubuf,
+			      (long)source_p->tsinfo, ubuf,
 			      source_p->username, source_p->host,
-			      IsIPSpoof(source_p) ? "0" : sockhost,
+			      IsIPSpoof(source_p) ? "0" : source_p->sockhost,
 			      source_p->id, source_p->info);
 
 		sendto_server(client_p, NULL, NOCAPS, CAP_TS6,
 			      "NICK %s %d %ld %s %s %s %s :%s",
-			      nick, source_p->hopcount + 1,
-			      (long) source_p->tsinfo,
+			      source_p->name, source_p->hopcount + 1,
+			      (long)source_p->tsinfo,
 			      ubuf, source_p->username, source_p->host,
 			      source_p->servptr->name, source_p->info);
 	}
 	else
 		sendto_server(client_p, NULL, NOCAPS, NOCAPS,
 			      "NICK %s %d %ld %s %s %s %s :%s",
-			      nick, source_p->hopcount + 1,
-			      (long) source_p->tsinfo,
+			      source_p->name, source_p->hopcount + 1,
+			      (long)source_p->tsinfo,
 			      ubuf, source_p->username, source_p->host,
 			      source_p->servptr->name, source_p->info);
 }
 
-/* 
- * valid_hostname - check hostname for validity
- *
- * Inputs       - pointer to user
- * Output       - YES if valid, NO if not
- * Side effects - NONE
- *
- * NOTE: this doesn't allow a hostname to begin with a dot and
- * will not allow more dots than chars.
- */
-int
-valid_hostname(const char *hostname)
-{
-	const char *p = hostname;
-	int found_sep = 0;
-
-	s_assert(NULL != p);
-
-	if(hostname == NULL)
-		return NO;
-
-	if('.' == *p || ':' == *p)
-		return NO;
-
-	while (*p)
-	{
-		if(!IsHostChar(*p))
-			return NO;
-                if(*p == '.' || *p == ':')
-  			found_sep++;
-		p++;
-	}
-
-	if(found_sep == 0)
-		return(NO);
-
-	return (YES);
-}
-
-/* 
- * valid_username - check username for validity
- *
- * Inputs       - pointer to user
- * Output       - YES if valid, NO if not
- * Side effects - NONE
- * 
- * Absolutely always reject any '*' '!' '?' '@' in an user name
- * reject any odd control characters names.
- * Allow '.' in username to allow for "first.last"
- * style of username
- */
-int
-valid_username(const char *username)
-{
-	int dots = 0;
-	const char *p = username;
-
-	s_assert(NULL != p);
-
-	if(username == NULL)
-		return NO;
-
-	if('~' == *p)
-		++p;
-
-	/* reject usernames that don't start with an alphanum
-	 * i.e. reject jokers who have '-@somehost' or '.@somehost'
-	 * or "-hi-@somehost", "h-----@somehost" would still be accepted.
-	 */
-	if(!IsAlNum(*p))
-		return NO;
-
-	while (*++p)
-	{
-		if((*p == '.') && ConfigFileEntry.dots_in_ident)
-		{
-			dots++;
-			if(dots > ConfigFileEntry.dots_in_ident)
-				return NO;
-			if(!IsUserChar(p[1]))
-				return NO;
-		}
-		else if(!IsUserChar(*p))
-			return NO;
-	}
-	return YES;
-}
-
 /* report_and_set_user_flags
  *
  * Inputs       - pointer to source_p
@@ -721,20 +596,16 @@ static void
 report_and_set_user_flags(struct Client *source_p, struct ConfItem *aconf)
 {
 	/* If this user is being spoofed, tell them so */
+	SetCork(source_p);
 	if(IsConfDoSpoofIp(aconf))
 	{
-		sendto_one(source_p,
-			   ":%s NOTICE %s :*** Spoofing your IP. congrats.",
-			   me.name, source_p->name);
+		sendto_one_notice(source_p, ":*** Spoofing your IP");
 	}
 
-	/* If this user is in the exception class, Set it "E lined" */
 	if(IsConfExemptKline(aconf))
 	{
 		SetExemptKline(source_p);
-		sendto_one(source_p,
-			   ":%s NOTICE %s :*** You are exempt from K/D/G/X lines. congrats.",
-			   me.name, source_p->name);
+		sendto_one_notice(source_p, ":*** You are exempt from K/G/X lines");
 	}
 
 	if(IsConfExemptGline(aconf))
@@ -743,59 +614,47 @@ report_and_set_user_flags(struct Client *source_p, struct ConfItem *aconf)
 
 		/* dont send both a kline and gline exempt notice */
 		if(!IsConfExemptKline(aconf))
-			sendto_one(source_p,
-				   ":%s NOTICE %s :*** You are exempt from G lines.",
-				   me.name, source_p->name);
+			sendto_one_notice(source_p, ":*** You are exempt from G lines");
 	}
 
 	/* If this user is exempt from user limits set it F lined" */
 	if(IsConfExemptLimits(aconf))
 	{
 		SetExemptLimits(source_p);
-		sendto_one(source_p,
-			   ":%s NOTICE %s :*** You are exempt from user limits. congrats.",
-			   me.name, source_p->name);
+		sendto_one_notice(source_p, ":*** You are exempt from user limits");
 	}
 
 	if(IsConfExemptFlood(aconf))
 	{
 		SetExemptFlood(source_p);
-		sendto_one(source_p,
-			   ":%s NOTICE %s :*** You are exempt from flood limits.",
-			   me.name, source_p->name);
+		sendto_one_notice(source_p, ":*** You are exempt from flood limits");
 	}
 
 	if(IsConfExemptSpambot(aconf))
 	{
 		SetExemptSpambot(source_p);
-		sendto_one(source_p,
-			   ":%s NOTICE %s :*** You are exempt from spambot checks.",
-			   me.name, source_p->name);
+		sendto_one_notice(source_p, ":*** You are exempt from spambot checks");
 	}
 
 	if(IsConfExemptJupe(aconf))
 	{
 		SetExemptJupe(source_p);
-		sendto_one(source_p,
-				":%s NOTICE %s :*** You are exempt from juped channel warnings.",
-				me.name, source_p->name);
-	}
-
-	if(IsConfExemptResv(aconf))
-	{
-		SetExemptResv(source_p);
-		sendto_one(source_p,
-				":%s NOTICE %s :*** You are exempt from resvs.",
-				me.name, source_p->name);
+		sendto_one_notice(source_p, ":*** You are exempt from juped channel warnings");
 	}
 
 	if(IsConfExemptShide(aconf))
 	{
 		SetExemptShide(source_p);
-		sendto_one(source_p,
-			   ":%s NOTICE %s :*** You are exempt from serverhiding.",
-			   me.name, source_p->name);
+		sendto_one_notice(source_p, ":*** You are exempt from serverhiding");
 	}
+
+	if(IsConfExemptResv(aconf))
+	{
+		SetExemptResv(source_p);
+		sendto_one_notice(source_p, ":*** You are exempt from resvs");
+	}
+	ClearCork(source_p);
+	send_pop_queue(source_p);
 }
 
 /*
@@ -827,7 +686,7 @@ user_mode(struct Client *client_p, struct Client *source_p, int parc, const char
 		return 0;
 	}
 
-	if((target_p = MyClient(source_p) ? find_named_person(parv[1]) : find_person(parv[1])) == NULL)
+	if((target_p = find_person(parv[1])) == NULL)
 	{
 		if(MyConnect(source_p))
 			sendto_one_numeric(source_p, ERR_NOSUCHCHANNEL,
@@ -858,7 +717,7 @@ user_mode(struct Client *client_p, struct Client *source_p, int parc, const char
 		m = buf;
 		*m++ = '+';
 
-		for (i = 0; user_modes[i].letter && (m - buf < BUFSIZE - 4); i++)
+		for(i = 0; user_modes[i].letter && (m - buf < BUFSIZE - 4); i++)
 			if(source_p->umodes & user_modes[i].mode)
 				*m++ = user_modes[i].letter;
 		*m = '\0';
@@ -872,8 +731,8 @@ user_mode(struct Client *client_p, struct Client *source_p, int parc, const char
 	/*
 	 * parse mode change string(s)
 	 */
-	for (p = &parv[2]; p && *p; p++)
-		for (pm = *p; *pm; pm++)
+	for(p = &parv[2]; p && *p; p++)
+		for(pm = *p; *pm; pm++)
 			switch (*pm)
 			{
 			case '+':
@@ -907,13 +766,14 @@ user_mode(struct Client *client_p, struct Client *source_p, int parc, const char
 
 					if(MyConnect(source_p))
 					{
-						source_p->umodes &= ~ConfigFileEntry.oper_only_umodes;
-						source_p->flags2 &= ~OPER_FLAGS;
+						source_p->umodes &=
+							~ConfigFileEntry.oper_only_umodes;
+						source_p->operflags &= ~OPER_FLAGS;
 
-						MyFree(source_p->localClient->opername);
+						rb_free(source_p->localClient->opername);
 						source_p->localClient->opername = NULL;
 
-						dlinkFindDestroy(source_p, &oper_list);
+						rb_dlinkFindDestroy(source_p, &oper_list);
 					}
 				}
 				break;
@@ -922,7 +782,7 @@ user_mode(struct Client *client_p, struct Client *source_p, int parc, const char
 				 * but they shouldnt be in default
 				 */
 
-			/* can only be set on burst */
+				/* can only be set on burst */
 #ifdef ENABLE_SERVICES
 			case 'S':
 #endif
@@ -933,7 +793,7 @@ user_mode(struct Client *client_p, struct Client *source_p, int parc, const char
 				break;
 
 			default:
-				if((flag = user_modes_from_c_to_bitmask[(unsigned char) *pm]))
+				if((flag = UserModeBitmask(*pm)))
 				{
 					if(MyConnect(source_p)
 					   && !IsOper(source_p)
@@ -960,18 +820,22 @@ user_mode(struct Client *client_p, struct Client *source_p, int parc, const char
 	if(badflag)
 		sendto_one(source_p, form_str(ERR_UMODEUNKNOWNFLAG), me.name, source_p->name);
 
+	if((source_p->umodes & UMODE_OPERWALL) && !IsOperOperwall(source_p))
+	{
+		sendto_one_notice(source_p, ":*** You need oper and operwall flag for +z");
+		source_p->umodes &= ~UMODE_OPERWALL;
+	}
+
 	if((source_p->umodes & UMODE_NCHANGE) && !IsOperN(source_p))
 	{
-		sendto_one(source_p,
-			   ":%s NOTICE %s :*** You need oper and N flag for +n", me.name, parv[0]);
+		sendto_one_notice(source_p, ":*** You need oper and nick_changes flag for +n");
 		source_p->umodes &= ~UMODE_NCHANGE;	/* only tcm's really need this */
 	}
 
 	if(MyConnect(source_p) && (source_p->umodes & UMODE_ADMIN) &&
 	   (!IsOperAdmin(source_p) || IsOperHiddenAdmin(source_p)))
 	{
-		sendto_one(source_p,
-			   ":%s NOTICE %s :*** You need oper and A flag for +a", me.name, parv[0]);
+		sendto_one_notice(source_p, ":*** You need oper and admin flag for +a");
 		source_p->umodes &= ~UMODE_ADMIN;
 	}
 
@@ -1008,7 +872,7 @@ send_umode(struct Client *client_p, struct Client *source_p, int old, int sendma
 	m = umode_buf;
 	*m = '\0';
 
-	for (i = 0; user_modes[i].letter; i++)
+	for(i = 0; user_modes[i].letter; i++)
 	{
 		flag = user_modes[i].mode;
 
@@ -1054,19 +918,18 @@ send_umode_out(struct Client *client_p, struct Client *source_p, int old)
 {
 	struct Client *target_p;
 	char buf[BUFSIZE];
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 
 	send_umode(NULL, source_p, old, SEND_UMODES, buf);
 
-	DLINK_FOREACH(ptr, serv_list.head)
+	RB_DLINK_FOREACH(ptr, serv_list.head)
 	{
 		target_p = ptr->data;
 
 		if((target_p != client_p) && (target_p != source_p) && (*buf))
 		{
 			sendto_one(target_p, ":%s MODE %s :%s",
-				   get_id(source_p, target_p), 
-				   get_id(source_p, target_p), buf);
+				   get_id(source_p, target_p), get_id(source_p, target_p), buf);
 		}
 	}
 
@@ -1084,6 +947,7 @@ send_umode_out(struct Client *client_p, struct Client *source_p, int old)
 void
 user_welcome(struct Client *source_p)
 {
+	SetCork(source_p);
 	sendto_one(source_p, form_str(RPL_WELCOME), me.name, source_p->name,
 		   ServerInfo.network_name, source_p->name);
 	sendto_one(source_p, form_str(RPL_YOURHOST), me.name,
@@ -1092,7 +956,6 @@ user_welcome(struct Client *source_p)
 
 	sendto_one(source_p, form_str(RPL_CREATED), me.name, source_p->name, creation);
 	sendto_one(source_p, form_str(RPL_MYINFO), me.name, source_p->name, me.name, ircd_version);
-
 	show_isupport(source_p);
 
 	show_lusers(source_p);
@@ -1107,66 +970,16 @@ user_welcome(struct Client *source_p)
 			   "NOTICE %s :*** Notice -- Please read the motd if you haven't read it",
 			   source_p->name);
 
-		sendto_one(source_p, form_str(RPL_MOTDSTART), 
-			   me.name, source_p->name, me.name);
+		sendto_one(source_p, form_str(RPL_MOTDSTART), me.name, source_p->name, me.name);
 
 		sendto_one(source_p, form_str(RPL_MOTD),
 			   me.name, source_p->name, "*** This is the short motd ***");
-
+		ClearCork(source_p);
 		sendto_one(source_p, form_str(RPL_ENDOFMOTD), me.name, source_p->name);
 	}
 	else
+	{
+		ClearCork(source_p);
 		send_user_motd(source_p);
-}
-
-/* oper_up()
- *
- * inputs	- pointer to given client to oper
- *		- pointer to ConfItem to use
- * output	- none
- * side effects	- opers up source_p using aconf for reference
- */
-int
-oper_up(struct Client *source_p, struct oper_conf *oper_p)
-{
-	int old = (source_p->umodes & ALL_UMODES);
-
-	SetOper(source_p);
-
-	if(oper_p->umodes)
-		source_p->umodes |= oper_p->umodes & ALL_UMODES;
-	else if(ConfigFileEntry.oper_umodes)
-		source_p->umodes |= ConfigFileEntry.oper_umodes & ALL_UMODES;
-	else
-		source_p->umodes |= DEFAULT_OPER_UMODES & ALL_UMODES;
-
-	Count.oper++;
-
-	SetExemptKline(source_p);
-
-	source_p->flags2 |= oper_p->flags;
-	MyFree(source_p->localClient->opername);
-	DupString(source_p->localClient->opername, oper_p->name);
-
-	dlinkAddAlloc(source_p, &oper_list);
-
-	if(IsOperAdmin(source_p) && !IsOperHiddenAdmin(source_p))
-		source_p->umodes |= UMODE_ADMIN;
-	if(!IsOperN(source_p))
-		source_p->umodes &= ~UMODE_NCHANGE;
-
-	sendto_realops_flags(UMODE_ALL, L_ALL,
-			     "%s (%s@%s) is now an operator", source_p->name,
-			     source_p->username, source_p->host);
-	if(!(old & UMODE_INVISIBLE) && IsInvisible(source_p))
-		++Count.invisi;
-	if((old & UMODE_INVISIBLE) && !IsInvisible(source_p))
-		--Count.invisi;
-	send_umode_out(source_p, source_p, old);
-	sendto_one(source_p, form_str(RPL_YOUREOPER), me.name, source_p->name);
-	sendto_one(source_p, ":%s NOTICE %s :*** Oper privs are %s", me.name,
-		   source_p->name, get_oper_privs(oper_p->flags));
-	send_oper_motd(source_p);
-
-	return (1);
+	}
 }
